
// ----------------------------------------------------------------------------
// JS VERSION -----------------------------------------------------------------

var jsVersion = 23;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

var $BROWSER = $('.browser'),
	startTime = 0; // new Date().getTime();

// ----------------------------------------------------------------------------
// SYSTEM ---------------------------------------------------------------------

var SYSTEM = new function() {

	var self = this;
	var ua = null,
		isIE = 0,
		isTouch = 0,
		isIOS = 0,
		isIPhone = 0,
		isIPad = 0,
		isIPod = 0,
		isAndroid = 0,
		isOldAndroid = 0,
		isWP = 0,
		isOldWP = 0,
		isDesktop = 0,
		msPointerEvents = 0,
		onTouchEvents = 0,
		onMouseEvents = 0;
	this.init = function () {
		try {
			ua = navigator.userAgent;
			if(ua.match(/IEMobile/i)) isWP = 1;
			if(ua.match(/MSIE/i)) isIE = 1;
			if(ua.match(/iPhone/i)) isIPhone = 1;
			if(ua.match(/iPad/i)) isIPad = 1;
			if(ua.match(/iPod/i)) isIPod = 1;
			if(isIPhone || isIPad || isIPod) isIOS = 1;
			if(ua.match(/Android/i)) isAndroid = 1;
			if(isIOS || isAndroid) isTouch = 1;
			if(!isIOS && !isAndroid && !ua.match(/mobile/i)) isDesktop = 1;
			if(isAndroid) {
				isOldAndroid = (function() {
					var match = ua.match(/Android\s([0-9\.]*)/);
					if(match) return (match[1] >= 4.2);
					else return true;
				})();
			}
			if(isWP) {
				isOldWP = (function() {
					if(ua.match(/IEMobile\/9/i)) return 1;
					return 0;
				})();
			}
			if(window.navigator.msPointerEnabled && window.navigator.msPointerEnabled != 'undefined') {
				msPointerEvents = 1;
			}
			if('ontouchstart' in document.documentElement) {
				onTouchEvents = 1;
			}
			if('onmousemove' in document.documentElement) {
				onMouseEvents = 1;
			}
		} catch (err) {
			
		}
	}();
	this.isAndroid = function() { return isAndroid; }
	this.isOldAndroid = function() { return isOldAndroid; }
	this.isIOS = function() { return isIOS; }
	this.isTouch = function() { return isTouch; }
	this.isWP = function() { return isWP; }
	this.isOldWP = function() { return isOldWP; }
	this.width = $BROWSER.outerWidth();
	this.paddingTop = parseInt($BROWSER.css('padding-top'));
	this.paddingLeft = parseInt($BROWSER.css('padding-left'));
	this.paddingRight = parseInt($BROWSER.css('padding-right'));
	this.paddingBottom = parseInt($BROWSER.css('padding-bottom'));
	this.msPointerEvents = function() { return msPointerEvents; }
	this.onTouchEvents = function() { return onTouchEvents; }
	this.onMouseEvents = function() { return onMouseEvents; }
	this.refreshPaddings = function() {
		self.paddingTop = parseInt($BROWSER.css('padding-top'));
		self.paddingLeft = parseInt($BROWSER.css('padding-left'));
		self.paddingRight = parseInt($BROWSER.css('padding-right'));
		self.paddingBottom = parseInt($BROWSER.css('padding-bottom'));
	}
}

function refreshBrowserMinHeight() {
	if(SYSTEM.isWP()) return;
	SYSTEM.refreshPaddings();
	$BROWSER.css('min-height', ($(window).height()-SYSTEM.paddingTop-SYSTEM.paddingBottom)+'px');
}

refreshBrowserMinHeight();

// ----------------------------------------------------------------------------
// WINDOW EVENTS --------------------------------------------------------------

$(window).resize(function() {
	
	SYSTEM.width = $BROWSER.outerWidth();
	$('.pages').removeClass('ready');
	$('.page-content').width(SYSTEM.width).each(function() {
		var $elem = $(this),
			left = parseInt($elem.css('left'));
		if(left < 0) $elem.css('left', -SYSTEM.width+'px');
		else if(left > 0) $elem.css('left', SYSTEM.width+'px');
	});
	setTimeout(function() {
		$('.pages').addClass('ready');
	}, 1);
	
});

if(SYSTEM.isIOS()) {
	window.onload = function() {
		navigate('js: rendered ' + document.body.offsetHeight);
	}
}

// ----------------------------------------------------------------------------
// LOG ------------------------------------------------------------------------

function bodyLog(what) {
	
	var margin_top = 0;
	if(SYSTEM.isAndroid()) margin_top = 70;
	var $consoleRecord = $('<div class="body-log" style="background:#fff;color:#555;font-size:12px;padding:5px;margin-bottom:5px;margin-top:'+margin_top+'px"></div>'),
        $consoleRecordClose = $('<div style="float:right">X</div>');
	try {
		if(typeof what !== 'object') {
			$consoleRecord.html(what);
		} else {
			var html = '';
			$.each(what, function(k, v) {
				html += '<div style="margin-bottom:10px;">' + k + ' = ' + v + '</div>';
			});
			$consoleRecord.html(html);
		}
	} catch(err) {
		$consoleRecord.html(err);
	}
	$consoleRecord.prepend($consoleRecordClose);
	bindClick($consoleRecordClose, function(e) {
		$(this).parent().remove();
	});
	var $prevBodyLog = $('.body-log:last');
	if($prevBodyLog && $prevBodyLog.length) $prevBodyLog.after($consoleRecord);
	else $('body').prepend($consoleRecord);

}

function log(prefix, text) {
	
	if (typeof text === 'undefined') {
		text = prefix;
		prefix = '';
	}
	prefix = prefix || '';
	if(SYSTEM.isAndroid()) galaBrowser.logv(prefix, text);
	if(SYSTEM.isIOS()) bodyLog(prefix + ' ' + text);
	if(SYSTEM.isWP()) window.external.notify('LOG: ' + prefix + ' ' + text);
	
}

// ----------------------------------------------------------------------------
// NAVIGATION -----------------------------------------------------------------

var isOnBack = 0,
	pageTimer = null,
	pageAutorefreshParams = null;

function navigate(url) {
	
	if(!url) return;
	if(SYSTEM.isWP()) window.external.notify(url);
	else if(SYSTEM.isAndroid()) galaBrowser.callback(url);
	else {
		//document.location = url;
		var iframe = document.createElement('IFRAME');
        iframe.setAttribute('src', url);
        document.documentElement.appendChild(iframe);
        iframe.parentNode.removeChild(iframe);
        iframe = null;
	}
	
}

function bindClickNavigate($elem, href) {
	
	$elem.removeAttr('href');
	bindClick($elem, function() {
		navigate(href);
	});
	$elem.attr('data-click_href',href);
}

function showPage(url, isReload) {
	
	if(!url) return;
	if(isReload) navigate("js: refresh: " + url);
	else navigate("js: goto: " + url);
	
}

function getHrefForModalView(elem, attrs, hlinks) {
	
	elem = elem || null;
	attrs = attrs || [];
	hlinks = hlinks || lastHlinks;
	var allow_pass = false,
		href = null;
	if(elem && typeof elem.getAttribute != 'undefined') {
		if(elem.getAttribute('button') || elem.getAttribute('hint') || elem.getAttribute('action') || elem.getAttribute('lst') || elem.getAttribute('confirm')) {
			allow_pass = true;
			attrs = elem.attributes;
		}
	} else {
		allow_pass = true;
	}
	for(var i = 0, ii = attrs.length; i < ii; i++) {
		if(attrs[i] && attrs[i].name == 'action' && attrs[i].value == 'ajax') {
			allow_pass = false;
			break;
		}
	}
	if(allow_pass) {
		var attrs_array = [];
		for(var i = 0, ii = attrs.length; i < ii; i++) {
			if(attrs[i] && typeof attrs[i].value != 'undefined') {
				if(attrs[i].value.length == 0) continue;
				attrs_array.push(attrs[i].name + '=' + attrs[i].value);
			}
		}
		if(attrs_array.length > 0) {
			href = attrs_array.join(' ' + ATRRIBUTE_SEPARATOR);
			var httpStr = isHttps()? 'https:' : 'http:';
			httpStr += SYSTEM.isIOS? '//' : '////';
			// var httpStr = 'http:////';
			// if(SYSTEM.isIOS) httpStr = 'http://';
			href = httpStr + PREFIX_FOR_ACTION_LINKS + ' ' + ATRRIBUTE_SEPARATOR + href + ' ' + ATRRIBUTE_SEPARATOR;
			if(SYSTEM.isIOS()) href = encodeURI(href);//encodeURIComponent(JSON.stringify(href));
			if(SYSTEM.isAndroid()) href = encodeURI(href);
		}
	}
	if(!href && hlinks && elem && typeof elem.getAttribute != 'undefined') {
		href = elem.getAttribute('data-href');
		if(!href) href = elem.getAttribute('href');
		if(href) {
			var firstHash = href.indexOf('#');
			if(firstHash !== -1 && firstHash > 0 && href.indexOf(' ') != -1) {
				href = decodeURIComponent(href.substr(firstHash));
			}
			href = hlinksReplace(href, hlinks);
			href = href.replace('®', '&reg');
			if (isHttps()) {
				href = href.replace('http://', 'https://');
			}
			var httpStr = isHttps()? 'https' : 'http'; 
			var suffix = href.indexOf(httpStr);
			if(suffix != 0) {
				href =  httpStr + ':////' + href;
			}
		}
	}
	return href;
}

function refreshCurPage(time, url, button, action, cmd, $timer) {
	
	if(url || button || action || cmd) {
		if(time > 0) {
			var last_url = curUrl;
			if($timer && $timer.length) $timer.text(time);
			stopPageAutorefresh();
			pageTimer = setInterval(function() {
				if(time <= 1) {
					stopPageAutorefresh();
					__doIt();
				} else {
					time -= 1;
					if($timer && $timer.length) $timer.text(time);
				}
			}, 1000);
		} else {
			__doIt();
		}
	}
	function __doIt() {
		if(action) {
			
		} else if(button) {
			var $btn = $BROWSER.find('[name="' + button + '"]');
			
			if($btn.length) fireClickEvent($btn);
		} else if (url) {
			showPage(url, true);
		} else {
			showPage(curUrl, true);
		}
		if(cmd) {
			navigate("js: send: " + cmd);
		}
	}
	
}

function stopPageAutorefresh() {
	
	clearTimeout(pageTimer);
	
}

function startPageAutorefresh() {
	
	if(!pageAutorefreshParams) return;
	refreshCurPage(pageAutorefreshParams[0], pageAutorefreshParams[1], pageAutorefreshParams[2], pageAutorefreshParams[3], pageAutorefreshParams[4], pageAutorefreshParams[5]);
	
}

// ----------------------------------------------------------------------------
// CLICK ----------------------------------------------------------------------

var SWIPE_X_POSSIBLE_DELTA = 3,
	SWIPE_Y_POSSIBLE_DELTA = 3,
	PAGES_SWIPE_X_POSSIBLE_DELTA = 40,
	PAGES_SWIPE_Y_POSSIBLE_DELTA = 20,
	SWIPE_X_MENU_DELTA = (SYSTEM.isIOS() ? 70 : 20),
	clickFirstX = 0,
	clickFirstY = 0;

function checkClickEventTouches(e) {
	
	if( e &&
		e.originalEvent &&
		e.originalEvent.touches &&
		e.originalEvent.touches.length &&
		e.originalEvent.touches[0].clientX
	) return true;
	else return false;
	
}

function clickStart(e) {
	
	var $elem = $(this) || $(e.target);
	if($elem.hasClass('disabled')) return;
	$elem.addClass('active');
	if(checkClickEventTouches(e)) {
		clickFirstX = e.originalEvent.touches[0].clientX;
		clickFirstY = e.originalEvent.touches[0].clientY;
	}
	
}

function clickMove(e) {
	
	var $elem = $(this) || $(e.target);
	if(checkClickEventTouches(e)) {
		if(
			Math.abs(e.originalEvent.touches[0].clientX - clickFirstX) > SWIPE_X_POSSIBLE_DELTA ||
			Math.abs(e.originalEvent.touches[0].clientY - clickFirstY) > SWIPE_Y_POSSIBLE_DELTA
		) {
			$elem.removeClass('active');
		}
	} else {
		$elem.removeClass('active');
	}
	
}

function clickEnd(e) {
	
	var $elem = $(this) || $(e.target),
		allowed = 1;
	if(e && e.forceClick) allowed = 1;
	else if($elem.hasClass('disabled') || !$elem.hasClass('active')) allowed = 0;
	if(allowed) {
		if(!e.dont_prevent) {
			if(typeof e.preventDefault == 'function') e.preventDefault();
			if(typeof e.stopPropagation == 'function') e.stopPropagation();
		}
		$elem.removeClass('active');
		var f = e.cb;
		if(typeof f == 'function') f(e, $(this));
	}
	clickFirstX = 0;
	clickFirstY = 0;
	
}

function bindClick($elem, f, dont_prevent) {
	
	if(SYSTEM.msPointerEvents()) {
		$elem.get(0).addEventListener("MSPointerDown", clickStart, false);
		$elem.get(0).addEventListener("MSPointerMove", clickMove, false);
		$elem.get(0).addEventListener("MSPointerUp", function(e) {
			e['cb'] = f;
			if(dont_prevent) e['dont_prevent'] = true;
			clickEnd.call(this, e);
		}, false);
	} else if(SYSTEM.onTouchEvents()) {
		$elem.bind("touchstart", clickStart);
		$elem.bind("touchmove", clickMove);
		$elem.bind("touchend", function(e) {
			e['cb'] = f;
			if(dont_prevent) e['dont_prevent'] = true;
			clickEnd.call(this, e);
		});
	} else if(SYSTEM.onMouseEvents()) {
		$elem.bind("mousedown", clickStart);
		$elem.bind("mousemove", clickMove);
		$elem.bind("mouseup", function(e) {
			e['cb'] = f;
			if(dont_prevent) e['dont_prevent'] = true;
			clickEnd.call(this, e);
		});
	} else {
		$elem.bind("click", function(e) {
			clickEnd.call(this, {cb:f, forceClick:true, dont_prevent:true});
		});
	}
	if(SYSTEM.isWP()) {
		$elem.bind("mouseout", function(e) {
			$(this).removeClass('active');
		});
	}
	if(SYSTEM.isAndroid()) {
		$elem.bind("touchleave", function(e) {
			$(this).removeClass('active');
		});
	}
	
}

function fireClickEvent($elem) {
	
	if(!$elem || !$elem.length) return;
	var href = $elem.attr('data-click_href')||'';
	var func = null;
	if (href){
		func=function(){
			navigate(href);
		};
	}
	clickEnd.call($elem, {forceClick:true,cb:func});
}

// ----------------------------------------------------------------------------
// RESERVED SERVERS -----------------------------------------------------------

try {
	if(typeof reservedServers !== 'undefined') {
		reservedServers = reservedServers.reservedServers;
	} else {
		var reservedServers = null;
	}
} catch(err) {
	var reservedServers = null;
}

function setReservedServers(rs) {
	try {
		if(typeof rs !== 'undefined') {
			if(typeof rs == 'string') rs = JSON.parse(rs);
			reservedServers = rs.reservedServers;
		}
	} catch(err) {
		
	}
}

// ----------------------------------------------------------------------------
// PREDEFINED INPUTS VALUES ---------------------------------------------------

try {
	if(typeof predefinedInputsValues !== 'undefined') {
		
	} else {
		var predefinedInputsValues = null;
	}
} catch(err) {
	var predefinedInputsValues = null;
}

function setPredefinedInputsValues(values) {
	try {
		if(typeof values !== 'undefined') {
			if(typeof values == 'string') values = JSON.parse(values);
			predefinedInputsValues = values;
		}
	} catch(err) {
		
	}
}

function getPredefinedInputValue(name) {
	if(!predefinedInputsValues) return null;
	try {
		return predefinedInputsValues[name];
	} catch(err) {
		return null;
	}
}

// ----------------------------------------------------------------------------
// ELEMENTS -------------------------------------------------------------------

function copyAttributes(from_elem, to_elem) {
	try {
	
		if(!from_elem || !to_elem) return;
		attrs = from_elem.attributes;
		if(attrs && attrs.length) {
			for(var k = 0, kk = attrs.length; k < kk; k++) {
				if(attrs[k] && attrs[k].nodeName != 'src' && attrs[k].nodeName != 'data-src') {
					try {
						to_elem.setAttribute(attrs[k].nodeName, attrs[k].nodeValue);
					} catch(err) {
						
					}
				}
			}
		}
		
	} catch(err) {
		
	}
}

function copyStyleClass($from_elem, $to_elem) {
	if(typeof $from_elem.attr('class') != 'undefined') {
		$to_elem.addClass($from_elem.attr('class'));
	}
	if(typeof $from_elem.attr('style') != 'undefined') {
		if(typeof $to_elem.attr('style') != 'undefined') {
			$to_elem.attr('style', $to_elem.attr('style')+' '+$from_elem.attr('style'));
		} else {
			$to_elem.attr('style', $from_elem.attr('style'));
		}
	}
}

// ----------------------------------------------------------------------------
// CONSTANTS ------------------------------------------------------------------

var HEAD_DELTAY = [-4, -4, -4, -4, -13, -4, -4, -4, -4, -4, -4, -4, -8, -4, -5, -4, -4, -5, -5, -5, -4, -4, -4, -4, -4, -4, -10, -17, -4, -4, -16, -16, -16, -10, -10, -4, -4, -4, -4, -5, -4, -4, -4, -4, -5, -4, -4, -4, -4],
    BODY_DELTAY = [81, 79, 84, 82, 82, 80, 79, 82, 76, 82, 78, 79, 73, 75, 74, 82, 73, 72, 45, 72, 74, 43, 75, 83, 62, 44, 84, 68, 82, 67, 81, 76, 75, 83, 78, 81, 82, 82, 80, 81, 71, 79, 81, 86, 80, 78, 76, 83, 80, 82, 43],
	HEAD_DELTAY_BIG = [-7, -7, -6, -7, -13, -5, -5, -6, -5, -6, -5, -5, -9, -5, -7, -5, -2, -6, -8, -7, -6, -5, -5, -6, -6, -6, -10, -19, -5, -5, -24, -20, -16, -11, -17, -4, -5, -2, -6, -8, -4, -3, -4, -4, -7, -4, -2, -4, -6, -3, -3, -3, -6, -3, -3, -3, -3, -4, -3, -8, -4, -9, -9, -9, -8, -24, -25, -24, -32, -24, -16, -13, -18, -29, -10, -44, -5, -17],
    BODY_DELTAY_BIG = [81, 79, 84, 82, 82, 80, 79, 82, 76, 82, 81, 80, 75, 76, 76, 82, 76, 72, 45, 74, 74, 36, 75, 80, 63, 46, 76, 67, 81, 67, 81, 76, 74, 83, 78, 80, 82, 83, 81, 80, 71, 79, 81, 86, 80, 78, 80, 83, 79, 82, 43, 74, 78, 74, 74, 81, 90, 86, 87, 82, 83, 87, 81],
    OPERATOR_DELTAY = [-2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, -5, 0, 0, 1, 1, 1, 1, 1, -1, 1, 1, 1, 1, -10, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, -3, 1, 1, 1],
    FOUNDER_DELTAY = [-16, -7, -9, -9, -11, -9, -13, -11, -5, -12, -9, -8, -12, -20, -10, -9, -9, -11, -11, -11, -7, -10, -8, -11, -10, -13, -42, -6, -9, -8, -17, -10, -10, -9, -22, -11, -7, -9, -8, -13, -9, -7, -7, -9, -10, -15, -9, -8, -14],
    COUNT_HEADS = HEAD_DELTAY.length,
    COUNT_BODYS = BODY_DELTAY.length,
    AWARD_OFF_X = [-17, -19, -22, -21, -16, -8, +3, +10, +14, +12, +10],
    AWARD_OFF_Y = [-57, -56, -52, -48, -43, -41, -43, -47, -52, -55, -56],
    AWARD_SPEED = 55,
	PARAMS_PER_SUIT = 5,
	ATRRIBUTE_SEPARATOR = ' |';

var PREFIX_FOR_ACTION_LINKS = '';
if (SYSTEM.isAndroid()) PREFIX_FOR_ACTION_LINKS = 'prefix/# ';
if (SYSTEM.isIOS()) PREFIX_FOR_ACTION_LINKS = 'prefix/';
if (SYSTEM.isWP()) PREFIX_FOR_ACTION_LINKS = 'prefix/# ';

if(SYSTEM.isIOS() && KBV >= 199) {
	ATRRIBUTE_SEPARATOR = '_|_';
}

// ----------------------------------------------------------------------------
// IMAGES ---------------------------------------------------------------------

var androidImagesURL = ["http://galaxy.mobstudio.ru/server_pics/", "file:///android_asset/hb/", "file:///android_asset/hb/b", "http://galaxy.mobstudio.ru/flash/pics/", "file:///android_asset/aura/0_", "file:///android_asset/", "file:///android_asset/rating/", "http://galaxy.mobstudio.ru/server_pics/hb/"],
    iOSImagesURL = ["http://galaxy.mobstudio.ru/server_pics/", "file:///", "file:///b", "http://galaxy.mobstudio.ru/flash/pics/", "file:///aura", "file:///", "file:///", "http://galaxy.mobstudio.ru/server_pics/hb/"],
	wpImagesURL = ["http://galaxy.mobstudio.ru/server_pics/", "file:///hb/", "file:///hb/b", "http://galaxy.mobstudio.ru/flash/pics/", "file:///aura/0_", "file:///", "file:///rating/", "http://galaxy.mobstudio.ru/server_pics/hb/"];

var imagesSrcURL = [];
if(SYSTEM.isAndroid()) imagesSrcURL = androidImagesURL;
if(SYSTEM.isIOS()) imagesSrcURL = iOSImagesURL;
if(SYSTEM.isWP()) imagesSrcURL = wpImagesURL;

function getImgSrc(img) {
	//if(!img) return ''; // WOW
    if(img instanceof Image) return img.data_src || img.src;
    else if(img.length) return img.attr('data-src') || img.attr('src');
    else return img.getAttribute('data-src') || img.getAttribute('src');
}

function setImgSrc(img, src) {
	if(!img || !src) return;
	if (isHttps()) src = src.replace('http://', 'https://');
    if(img instanceof Image) img.src = src;
    else if(img.length) img.attr('src', src);
    else img.setAttribute('src', src);
}

function checkImg(img) {
    if(!img) return false;
    if(img instanceof Image) return true;
    if(img.length) return true;
    if(typeof img == 'object' && typeof img.getAttribute === 'function') return true;
}

function getImgHosts(img) {
	//if(!img) return null;	// WOW
    var hosts = null;
    if(img instanceof Image) hosts = img.hosts;
    else if(img.length) hosts = img.attr('data-hosts');
    else hosts = img.getAttribute('data-hosts');
    if(hosts) hosts = hosts.split(',');
    return hosts;
}

function setImgHosts(img, hosts) {
	//if(!img) return;	// WOW
    if(img instanceof Image) img.hosts = hosts;
    else if(img.length) img.attr('data-hosts', hosts);
    else img.setAttribute('data-hosts', hosts);
}

function replaceUrlHost(url, new_host) {
    if(!url || !new_host) return url;
    var protocol = url.substr(0, url.indexOf('://')+3),
        path = url.substr(url.indexOf('://')+3),
        host = path.substr(0, path.indexOf('/'));
    path = path.substr(host.length);
    return protocol + new_host + path;
}

function getImgSrcNextHost(img, predefined_src, load_cb) {
    if(checkImg(img)) {
        var hosts = getImgHosts(img);
        if(!hosts && predefined_src) {
            var h = [];
			if(reservedServers) {
				for(var i = 0, ii = reservedServers.length; i < ii; i++) {
					if(reservedServers[i] && predefined_src.indexOf(reservedServers[i].alias) != -1) {
						h = reservedServers[i].servers.slice();
						break;
					}
				}
			}
            setImgHosts(img, h.join(','));
        }
        var src = predefined_src || getImgSrc(img);
        hosts = getImgHosts(img);
        if(hosts) {
            src = replaceUrlHost(src, hosts[0]);
            hosts.shift();
            setImgHosts(img, hosts.join(','));
        }
        return src;
    } else {
        return '';
    }
}

function imageSrc(id, type, small, $img, load_cb, no_file_check) {

    var src = '';
    try {
		
		if(typeof id == 'string' && id.indexOf('~') != -1) return id;
		if(id <= 0 || id == '') return 'data:image/gif;base64,R0lGODlhAQABAPABAP///wAAACH5BAEKAAAALAAAAAABAAEAAAICRAEAOw%3D%3D'; // return empty gif image
		var cond1 = (type == 0 && !small && (id + '').lastIndexOf('_') != (id + '').length - 1),
            cond2 = (!isNaN(id) && id > 2000),
            cond3 = (isNaN(id) && (id + '').indexOf('http') == -1),
            cond4 = ((SYSTEM.isAndroid() || SYSTEM.isWP()) && type == 2);
		if(cond1 || cond4) id = id + '_';
		src = id + '';
		if(src.indexOf('http://') == -1 && src.indexOf('https://') == -1 && src.indexOf('file:///') == -1) {
			src = imagesSrcURL[type] + id;
		}
		if(!no_file_check && (SYSTEM.isIOS() || SYSTEM.isWP()) && src.indexOf('file:///') != -1) {
			src = src.substr(8);
		}
		if(src.indexOf('.php') == -1 && src.indexOf('.png') == -1 && src.indexOf('.jpeg') == -1 && src.indexOf('.jpg') == -1 && src.indexOf('.gif') == -1 && src.indexOf('.bmp') == -1) {
			src = src + '.png';
		}
		if(reservedServers && $img) {
			src = getImgSrcNextHost($img, src, load_cb);
		}
		if (isHttps()) {
			src = src.replace('http://', 'https://');
		}
		
	} catch(err) {
		
		if(reservedServers && $img) {
			src = getImgSrcNextHost($img, src);
		}
		
	}

    return src;
}

function setupImg($img, load_cb) {
	
	if(!$img) return;
	var src = getImgSrc($img);
	if(reservedServers) {
		var hosts = getImgHosts($img);
		if(!src) return;
		if(!hosts) {
			hosts = [];
			for(var i = 0, ii = reservedServers.length; i < ii; i++) {
				if(reservedServers[i] && src.indexOf(reservedServers[i].alias) != -1) {
					hosts = reservedServers[i].servers.slice();
					break;
				}
			}
			setImgHosts($img, hosts.join(','));
		}
		setupImgHost($img);
	} else {
		setImgSrc($img, src);
	}
	if($img instanceof Image) {
		if(typeof load_cb == 'function') $img.onload = load_cb;
		if(reservedServers) {
			$img.onerror = function() {
				setupImgHost(this, false, true);
			};
		} else if(typeof $img.error_src != 'undefined' && $img.error_src) {
			$img.onerror = function() {
				this.src = this.error_src;
				this.error_src = null;
			};
		}
	} else if($img.length) {
		$img.bind('load', function() {
			$(this).css('visibility', 'visible');
			if(typeof load_cb == 'function') load_cb.call(this);
		});
		if(reservedServers) {
			$img.bind('error', function() {
				$(this).css('visibility', 'hidden');
				setupImgHost($(this), false, true);
			});
		} else if(typeof $img.attr('data-error-src') != 'undefined' && $img.attr('data-error-src')) {
			$img.bind('error', function() {
				$(this).attr('src', $(this).attr('data-error-src')).attr('data-error-src', '');
			});
		}
	}
	
}

function setupImgHost($img, dont_apply_src, on_error) {
	
	if(!$img) return;
    var src = getImgSrc($img);
	var check_error_src = true;
	if(src) {
		check_error_src = false;
		var old_src = src,
			hosts = getImgHosts($img) || [],
			allow = 1;
		if(hosts && hosts.length) {
			src = replaceUrlHost(src, hosts[0]);
			hosts.shift();
			setImgHosts($img, hosts.join(','));
		}
		if(old_src == src && on_error) allow = 0;
		if(allow) setImgSrc($img, src);
		if(!allow && on_error) check_error_src = true;
	}
	if(check_error_src) {
		if($img instanceof Image) {
			if(typeof $img.error_src != 'undefined' && $img.error_src) {
				$img.src = $img.error_src;
				$img.error_src = null;
			}
		} else {
			if(typeof $img.attr('data-error-src') != 'undefined' && $img.attr('data-error-src')) {
				$img.attr('src', $img.attr('data-error-src'));
				$img.attr('data-error-src', '');
			}
		}
	}
	
}

// ----------------------------------------------------------------------------
// SMILES ---------------------------------------------------------------------

var smiles = [['..|..',60],[':fp:',68],[':lol:',69],[':ffuu:',70],[':troll:',71],[':-j',73],[':-J',73],[':-3',74],[':=|',75],[':T',76],[':t',76],[':Т',76],[':т',76],['y.e.',3],['c.u.',3],['u.c.',3],['у.е.',3],['0:)',53],['0:-)',53],['{:)',54],['<:)',54],['<:-)',54],[':plz:',72],[':)',1],[':-)',1],[':]',2],[';)',4],[';-)',4],[':D',5],[':p',6],[':р',6],[':Р',6],[':P',6],['O:}',8],['О:}',8],[':(',9],[':o',10],[':о',10],[':|',11],[':-/',12],[':\\',13],['o_O',14],['о_О',14],[':[',15],['X(',16],['Х(',16],[':O',17],[':О',17],['()',18],['(-)',19],['\\_/',20],[']:->',21],[']:-&gt;',21],['^_^',22],['&circ;_&circ;',22],[':-.',23],[':-*',24],[':-!',41],['8-)',42],['%)',43],['%D',46],['(*)',48],['(|)',49],['(I)',49],['@}',50],['(d=)',51],['(q=)',52],['8(',55],['8-(',55],['(:@',56],['(.)(.)',58],['@=',57]], //[['0:)', 53], ['0:-)', 53], ['{:)', 54], ['<:)', 54], ['<:-)', 54], [':plz:', 72], [':)', 1], [':-)', 1], [':]', 2], [';)', 4], [';-)', 4], [':D', 5], [':p', 6], [':р', 6], [':Р', 6], [':P', 6], ['O:}', 8], ['О:}', 8], [':(', 9], [':o', 10], [':о', 10], [':|', 11], [':-/', 12], [':\\', 13], ['o_O', 14], ['о_О', 14], [':[', 15], ['X(', 16], ['Х(', 16], [':O', 17], [':О', 17], ['()', 18], ['(-)', 19], ['\\_/', 20], [']:->', 21], [']:-&gt;', 21], ['^_^', 22], ['&circ;_&circ;', 22], [':-.', 23], [':-*', 24], [':-!', 41], ['8-)', 42], ['%)', 43], ['%D', 46], ['(*)', 48], ['(|)', 49], ['(I)', 49], ['@}', 50], ['(d=)', 51], ['(q=)', 52], ['8(', 55], ['8-(', 55], ['(:@', 56], ['(.)(.)', 58], ['@=', 57], ['..|..', 60], [':fp:', 68], [':lol:', 69], [':ffuu:', 70], [':troll:', 71], [':-j', 73], [':-J', 73], [':-3', 74], [':=|', 75], [':T', 76], [':t', 76], [':Т', 76], [':т', 76], ['y.e.', 3], ['c.u.', 3], ['u.c.', 3], ['у.е.', 3]],
	hiddenSmiles = [48, 49, 50, 57, 58, 60],
    smilesForPanel = [1, 2, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 41, 42, 43, 46, 51, 52, 53, 54, 55, 56, 50, 68, 69, 70, 71, 72, 73, 74, 75, 76];
if(KBV > 200) {		// 9.1
	var new_smiles = [
		[':blush:', 77], [':rage:', 78], [':joy:', 79], [':haha:', 80], 
		[':eww:', 81], [':pheart:', 82], [':wow:', 83], [':sob:', 84], 
		[':worried:', 85], [':mmm:', 86], [':love:', 87], [':kiss:', 88], 
		[':susp:', 89], [':poop:', 90], [':grumpy:', 91], [':sungl:', 92], 
		[':pray:', 93], [':sun:', 94]
	];
	smiles = new_smiles.concat(smiles);
}
	
RegExp.escape = function(string) {
	return string.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, "\\$1");
};

for(var i = 0, ii = smiles.length; i < ii; i++) {
	smiles[i].push(new RegExp(RegExp.escape(smiles[i][0]), 'g'));
}

function getSmileAbbr(id) {
	for(var i = 0, ii = smiles.length; i < ii; i++)
		if(smiles[i][1] == id) return smiles[i][0];
	return '';
}

function getAbbrForSmile(id, remember_cnt) {
	for(var i = 0, ii = smiles.length; i < ii; i++) {
		if(smiles[i][1] == id) {
			return smiles[i][0];
		}
	}
	return '';
}

function replaceSmiles(txt, big, no_replace) {

    // для WP есть отдельная функция замены смайлов
    // такая же, но вызывается только в 2 местах
    if (SYSTEM.isWP())
        return txt;

	try {
		if(!txt) return txt;
		var _class = 'smile';
		if(big) _class = 'smile-big';
		if(!no_replace) {
			txt = txt.replace(/</g, '&lt;');
			txt = txt.replace(/>/g, '&gt;');
		}
		for(var j = 0, jj = smiles.length; j < jj; j++) {
			txt = txt.replace(smiles[j][2], '<div class="' + _class + ' s' + smiles[j][1] + '"></div>');
		}
		return txt;
	} catch(err) {
		return txt;
	}
}

function replaceSmilesWP(txt) {
    try {
        if (!txt) return txt;
        var _class = 'smile-big';
        txt = txt.replace(/</g, '&lt;');
        txt = txt.replace(/>/g, '&gt;');
        for (var j = 0, jj = smiles.length; j < jj; j++) {
            txt = txt.replace(smiles[j][2], '<div class="' + _class + ' s' + smiles[j][1] + '"></div>');
        }
        return txt;
    } catch (err) {
        return txt;
    }
}

function __bindSmiles($elem, $smiles_btn) {
	var $text = $elem.find('.input'),
        pos = 0;
	bindClick($smiles_btn, function(e) {
		e.preventDefault();
		e.stopPropagation();
		if($smiles_btn.parent().hasClass('disabled')) return;
		pos = getCaretPos($text.get(0));
		navigate('js: showsmiles ' + $elem.attr('id') + ' ' + pos);

	});
}

function __pasteSmileAtPos(elem_id, pos, smile) {
	var $input = $parent.find('#' + elem_id + ' .input');
	if(SYSTEM.isWP()) {
		$input.val($input.val() + smile);
	} else {
       var text = $input.val();
		new_text = text.substr(0, pos) + smile + text.substr(pos);
		$input.val(new_text);
		setCaretToPos($input.get(0), pos + smile.length);
	}
	$input.keyup();
}

function callThisToShowSmilesPanel(id, pos) {
	// WTF?
}


// ----------------------------------------------------------------------------
// OTHERS ---------------------------------------------------------------------

function getWindowScroll() {
	try {
		return (window.pageYOffset || document.documentElement.scrollTop) + "";
	} catch(err) {
		return "";
	}
}

function setWindowScroll(offset) {
	if(offset < 0) offset = 0;
	setTimeout(function() {
		window.scrollTo(0, offset);
	}, 70);
}

function getHiddenElementHeight(element) {
	var tempId = 'tmp-' + Math.floor(Math.random() * 99999);
	$(element).clone()
        .css('position', 'absolute')
        .css('height', 'auto').css('width', '1000px')
    .appendTo($(element).parent())
        .css('left', '-10000em')
        .addClass(tempId).show()
	h = $('.' + tempId).height()
	$('.' + tempId).remove()
	return h;
}

function getHiddenElementSizes($elem, px_only) {
	var w = 0,
        h = 0,
        $clone;
	if ($elem.css('position') == 'absolute') {
		$clone = $('<div>' + $elem.html() + '</div>');
	} else {
		$clone = $elem.clone().css({
			'display': 'block'
		});
	}
	if ($clone && $clone.length) {
		$clone.css({
			'position': 'absolute',
			'top': 'auto',
			'right': 'auto',
			'bottom': 'auto',
			'left': '0',
			'display': 'block',
			'width': '',
			'height': ''
		});
		$('.browser-testing').append($clone);
		h = $clone.outerHeight();
		w = $clone.outerWidth();
		if (px_only) {
			if (!h) {
				h = $clone[0].style.height || 0;
				if (h && h.indexOf('%') != -1) {
					var parent_sizes = getElementSizes($elem.parent());
					h = parseInt(parseFloat(h) * parseInt(parent_sizes.height) / 100);
				}
			}
			if (h) h += 'px';
			if (!w) {
				w = $clone[0].style.width || 0;
				if (w && w.indexOf('%') != -1) {
					var parent_sizes = getElementSizes($elem.parent());
					w = parseInt(parseFloat(w) * parseInt(parent_sizes.width) / 100);
				}
			}
			if (w) w += 'px';
		} else {
			if (!h) h = $clone[0].style.height;
			if (!w) w = $clone[0].style.width;
		}
		$clone.remove();
	}
	return {
		'width': w,
		'height': h
	};
}

function getElementSizes($elem, px_only) {
	if (!$elem.is(':visible')) return getHiddenElementSizes($elem, px_only);
	else return {
		'width': ($elem[0].style.width || $elem.outerWidth() + 'px'),
		'height': ($elem[0].style.height || $elem.outerHeight() + 'px')
	};
}

function getMarginPadding(src) {
	src = src.split(',');
	if(src.length == 1) src = [src, src, src, src];
	if(src.length == 2) src = [src[0], src[1], src[0], src[1]];
	if(src.length == 3) src = [src[0], src[1], src[2], src[1]];
	return src.join('px ') + 'px';
}

function setElementMinSizes($elem) {
	$elem = $($elem);
	var min_h = parseInt($elem.parent().css('min-height')),
        new_min_h = parseInt($elem.height()) + parseInt($elem.css('top'));
	if(new_min_h > 0 && new_min_h > min_h) $elem.parent().css('min-height', new_min_h + 'px');
	var min_w = parseInt($elem.parent().css('min-width')),
        new_min_w = parseInt($elem.width()) + parseInt($elem.css('left'));
	if(new_min_w > 0 && new_min_w > min_w) $elem.parent().css('min-width', new_min_w + 'px');
}

function fixImgVerticalPosition($elem) {
	if(!$elem || !$elem.length) return;
	var h = $elem.height(),
		src = $elem.attr('src') || $elem.attr('data-src');
	if(src && src.indexOf('exp_clock') != -1) return;
	if(!h) h = $elem.attr('h') || $elem.attr('data-h') || $elem.attr('height');
	if(h && h <= 36) {
		$elem.css({'vertical-align':parseInt(6-parseInt(h, 10)/2, 10)+'px'});
	}
}

function fixColorZeroes(val) {
	val = val.toString();
	if (val.length < 6) {
		var zeroes = 6 - val.length;
		for (var i = 0; i < zeroes; i++) val = '0' + val;
		return val;
	} else {
		return val;
	}
}

function getRandomInt(min, max) {
	return Math.floor(Math.random() * (max - min + 1)) + min;
}

function getParentByTagName(obj, tag) {
	var obj_parent = obj.parentNode;
	if (!obj_parent) return false;
	if (obj_parent.tagName && obj_parent.tagName.toLowerCase() == tag) return obj_parent;
	else return getParentByTagName(obj_parent, tag);
}

$.fn.hasParent = function (objs) {
	objs = $(objs);
	var found = false;
	$(this[0]).parents().andSelf().each(function () {
		if ($.inArray(this, objs) != -1) {
			found = true;
			return false;
		}
	});
	return found;
}

function URLToArray(url) {
	if (!url) return {};
	var request = {}, pairs;
	if (url.indexOf('?') >= 0) url = url.substring(url.indexOf('?') + 1);
	pairs = url.split('&');
	for (var i = 0; i < pairs.length; i++) {
		var pair = pairs[i].split('=');
		request[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1]);
	}
	return request;
}

function ArrayToURL(array, with_q_mark) {
	var pairs = [];
	for (var key in array)
		if (array.hasOwnProperty(key))
			pairs.push(encodeURIComponent(key) + '=' + encodeURIComponent(array[key]));
	if (with_q_mark) return '?' + pairs.join('&');
	else return pairs.join('&');
}

function addParamToURL(url, param, value) {
	if (url.indexOf('?') == -1) {
		return url + '?' + param + '=' + value;
	} else {
		var _url = url.substr(0, url.indexOf('?') + 1);
		var found = false;
		url = URLToArray(url);
		try {
			url[param] = value;
		} catch (err) {
			cerr('8001', err);
		}
		return _url + ArrayToURL(url);
	}
}

function URLCompare(url1, url2, except_array) {
	if (url1.toLowerCase() == url2.toLowerCase()) return true;
	if ($.trim(url1.toLowerCase()) == $.trim(url2.toLowerCase())) return true;
	var href1 = url1.substr(0, url1.substr('?'));
	var href2 = url2.substr(0, url2.substr('?'));
	if (href1 != href2) return false;
	var params1 = URLToArray(url1);
	var params2 = URLToArray(url2);
	if (params1.length != params2.length) return false;
	$.each(params1, function (k, param) {
		if (!params2.param) return false;
		if (except_array && !in_array2(params1.param, except_array)) {
			try {
				if ($.trim(params2.param.toLowerCase()) != $.trim(params1.param.toLowerCase())) return false;
			} catch (err) {

			}
		}
	});
	return true;
}

function splitSplit(str, sep1, sep2) {
	var array = [];
	if (!str) return array;
	array = str.split(sep1);
	for (var i = 0, ii = array.length; i < ii; i++) array[i] = array[i].split(sep2);
	return array;
}

function appendLink(url, for_json_request, hosts) {
	if(!hosts) {
		hosts = [];
		if(reservedServers) {
			for(var i = 0, ii = reservedServers.length; i < ii; i++) {
				if(reservedServers[i] && url.indexOf(reservedServers[i].alias) != -1) {
					hosts = reservedServers[i].servers.slice();
				}
			}
		}
	}
	var anchor = '';
	if(url.indexOf('#') != -1) {
		anchor = url.substr(url.indexOf('#'));
		url = url.substr(0, url.indexOf('#'));
	}
	var params = URLToArray(url);
	if(!params.userID || !params.userid) params.userid = USERID;
	if(!params.password) params.password = USERPASS;
	if(!params.usercur) params.usercur = USERID;
	var symb = '?';
	if(url.indexOf('?') != -1) symb = '&';
	if(url[url.length - 1] == '&' || url[url.length - 1] == '?') symb = '';
	url += symb + 'dbl=' + DBL + '&kbv=' + KBV + '&lngg=' + LNGG + '&' + Math.random();
	if(for_json_request) url += '&callback=?';
	url += '&' + anchor;
	return {url:url, hosts:hosts};
}

function getNextLinkHost(url, hosts) {
	if(!hosts) {
		hosts = [];
		if(reservedServers) {
			for(var i = 0, ii = reservedServers.length; i < ii; i++) {
				if(reservedServers[i] && url.indexOf(reservedServers[i].alias) != -1) {
					hosts = reservedServers[i].servers.slice();
				}
			}
		}
	}
	if(reservedServers) {
		if(hosts && hosts.length >= 1) {
			url = replaceUrlHost(url, hosts[0]);
			hosts.shift();
		} else {
			url = null;
		}
	} else {
		url = url;
	}
	return {url:url, hosts:hosts};
}

function setSelectionRange(input, selectionStart, selectionEnd) {
	try {
		if (input.setSelectionRange) {
			input.focus();
			input.setSelectionRange(selectionStart, selectionEnd);
		} else if (input.createTextRange) {
			var range = input.createTextRange();
			range.collapse(true);
			range.moveEnd('character', selectionEnd);
			range.moveStart('character', selectionStart);
			range.select();
		}
	} catch (err) {

	}
}

function setCaretToPos(input, pos) {
	setSelectionRange(input, pos, pos);
}

function getCaretPos(input) {
	if (document.selection && document.selection.createRange) {
		var range = document.selection.createRange(),
            bookmark = range.getBookmark(),
            caret_pos = bookmark.charCodeAt(2) - 2;
	} else {
		if (input.setSelectionRange)
			var caret_pos = input.selectionStart;
	}
	return caret_pos;
}

function getAverageTextWidth(text) {
	if(text && text.length) return text.length*12;
	else return 0;
}

// ------------------------------------------------------------------------------------------------
// TWEAKS -----------------------------------------------------------------------------------------

// --- Android

function sendPost(elem) {
	galaBrowser.sendPost(elem, getPostParams());
}

function sendRegister(url, param) {
	galaBrowser.sendRegister(url, getPostParams(), param);
}

function sendSmsSimple(phone, text, url) {
	galaBrowser.sendSmsSimple(phone, getPostParams(), text, url);
}

function sendSms(phone, text, url) {
	galaBrowser.sendSms(phone, getPostParams(), text, url);
}

function joinGalaxy() {
	galaBrowser.joinGalaxy(getPostParams());
}

function recoveryCode() {
	galaBrowser.recoveryCode(getPostParams());
}

function getPageInformation(action) {
	try {
		galaBrowser.getPageInformation(action, getWindowScroll(), getPostParams());
	} catch(err) {
		
	}
}

function getPostParams() {
	try {
		var data = '';
		function __do($inputs) {
			if(!$inputs || !$inputs.length) return '';
			var data = '';
			$inputs.each(function() {
				var $elem = $(this),
					name = $elem.attr('name') || null;
				if(!name) return '';
				if($elem.attr('data-type') == 'checkbox') {
					if($elem.parent().hasClass('checked')) data += '"' + $elem.attr('name') + '"=' + '"1"' + '&';
					else data += '"' + $(this).attr('name') + '"=' + '"0"' + '&';
				} else if($elem.attr('data-type') == 'radio') {
					if($elem.parent().hasClass('checked')) data += '"' + $elem.attr('name') + '"="' + $elem.attr('value').replace(/&/g, '&amp;').replace(/"/g, '&quot;') + '"&';
				} else {
					data += '"' + $elem.attr('name') + '"="' + $elem.val().replace(/&/g, '&amp;').replace(/"/g, '&quot;').replace(/\n/g, "<br>") + '"&';
				}
			});
			return data;
		}
		data += __do($BROWSER.find(':input'));
		data += __do($BROWSER.find('select'));
		if(hasPagesTag) {
			$BROWSER.find('.pages').each(function() {
				data += '"sel__pages'+$(this).attr('data-sel-index')+'"="'+$(this).attr('data-sel')+'"&';
			});
		}
		if(hasTabsTag) {
			$BROWSER.find('.tabs').each(function() {
				data += '"sel__tabs'+$(this).attr('data-sel-index')+'"="'+$(this).attr('data-sel')+'"&';
			});
		}
		return data;
	} catch(err) {
		return '';
	}
}

function setInputValue(key, value) {
	if(key && key.indexOf('sel__') != -1) {
		if(key.indexOf('sel__tabs') != -1) {
			var index = key.substr(9),
				$tabs = $BROWSER.find('.tabs[data-sel-index="'+index+'"]');
			if($tabs && $tabs.length) {
				$tabs.attr('data-sel', value);
				updateTabsTag($tabs, true);
			}
		}
		if(key.indexOf('sel__pages') != -1) {
			var index = key.substr(10)
				$pages = $BROWSER.find('.pages[data-sel-index="'+index+'"]');
			if($pages && $pages.length) {
				$pages.attr('data-sel', value);
				updatePagesTag($pages, true);
			}
		}
	} else {
		var els = document.getElementsByName(key);
		for(var i = 0; i < els.length; i++) {
			els[i].value = value;
			$(els[i]).change();
		}
	}
}

function getRegInf() {
	galaBrowser.getRegInf(getPostParams());
}

// --- iOS

function detectVerticalSquash(img) {
	var iw = img.naturalWidth,
		ih = img.naturalHeight;
	var canvas = document.createElement('canvas');
	canvas.width = 1;
	canvas.height = ih;
	var ctx = canvas.getContext('2d');
	ctx.drawImage(img, 0, 0);
	var data = ctx.getImageData(0, 0, 1, ih).data;
	var sy = 0;
	var ey = ih;
	var py = ih;
	while (py > sy) {
		var alpha = data[(py - 1) * 4 + 3];
		if (alpha === 0) {
			ey = py;
		} else {
			sy = py;
		}
		py = (ey + sy) >> 1;
	}
	var ratio = (py / ih);
	return (ratio === 0) ? 1 : ratio;
}

function drawImageIOSFix(ctx, img, sx, sy, sw, sh, dx, dy, dw, dh) {
	var vertSquashRatio = 1; // detectVerticalSquash(img);
	dx = dx || 0;
	dy = dy || 0;
	dw = dw || sw;
	dh = dh || sh;
	ctx.drawImage(img, sx, sy, sw, sh, dx, dy, dw, dh / vertSquashRatio);
}

function tabChanged() {
	if(!SYSTEM.isIOS()) return;
	navigate("js: tabChanged");
}

function touchLeaveAll() {
	$('.active').removeClass('active');
}

// --- WP

function correctFixedElements() {
    $('.fixed').each(function () {
        var top = $(this).css('top');
        $(this).css('top', top);
    });
}

// --- COMMON

function repaintCSSPage() {
	$('#FCK').remove();
	$('body').append('<div id="FCK" style="position:absolute"></div>');
}

function repaintCSSPageFast() {
	$('body')[0].offsetHeight;
}

// ------------------------------------------------------------------------------------------------
// DOM --------------------------------------------------------------------------------------------

function configureAnimationChain($elem) {
	var chain = (typeof $elem.attr('animation-chain') ? $elem.attr('animation-chain') : '');
	if (!chain) return;
	if (chain) chain = chain.split(';')
	if (!chain.length) return;
	var start_delay_time = (typeof $elem.attr('animation-chain-start-delay') ? parseInt($elem.attr('animation-chain-start-delay')) : 0);
	var steps_delay_time = (typeof $elem.attr('animation-chain-steps-delay') ? parseInt($elem.attr('animation-chain-steps-delay')) : 0);
	var end_class = (typeof $elem.attr('animation-chain-end') ? $elem.attr('animation-chain-end') : null);
	var repeat_times = (typeof $elem.attr('animation-chain-repeat') ? $elem.attr('animation-chain-repeat') : 0);

	setTimeout(function() {
		__animateChain($elem, chain, steps_delay_time, end_class, repeat_times);
	}, start_delay_time);
}

function __animateChain($elem, chain, steps_delay_time, end_class, repeat_times) {
	var original_chain = chain.slice(0);
	setTimeout(function() {
		__animateChainStep($elem, chain, function() {
			if (repeat_times == 'infinite') {
				__animateChain($elem, original_chain, steps_delay_time, end_class, repeat_times);
			} else {
				repeat_times = parseInt(repeat_times);
				if (repeat_times > 1) {
					repeat_times--;
					__animateChain($elem, original_chain, steps_delay_time, end_class, repeat_times);
				} else {
					if (end_class) $elem.addClass(end_class);
				}
			}
		});
	}, steps_delay_time);
}

function __animateChainStep($elem, chain, end_cb, prev_class) {
	if (!chain || !chain.length) {
		if (prev_class) $elem.removeClass(prev_class);
		end_cb();
		return;
	}
	var step = chain.shift();
	step = step.split(',');
	var step_time = step[0];
	var step_class = step[1];
	$elem.addClass(step_class);
	if (prev_class) $elem.removeClass(prev_class);
	setTimeout(function() {
		__animateChainStep($elem, chain, end_cb, step_class);
	}, step_time);
}

function parseRaitTag(rait) {
	
	var rait = parseInt(rait),
		level = 0,
        mod = 0,
        imgh = 0,
        imgw = 0,
		calcWidth = 0,
		$d = $('<span class="rating" style="display:inline"></span>');
	if(rait > 0) {
		level = 8;
		while(rait >= 5) {
			switch(level) {
				case 8:
					mod = Math.floor(rait / (375 * 25 * 25));
					imgh = 20;
					imgw = 20;
					rait -= mod * 375 * 25 * 25;
					break;
				case 7:
					mod = Math.floor(rait / (375 * 125));
					imgh = 20;
					imgw = 20;
					rait -= mod * 375 * 125;
					break;
				case 6:
					mod = Math.floor(rait / (375 * 25));
					imgh = 20;
					imgw = 25;
					rait -= mod * 375 * 25;
					break;
				case 5:
					mod = Math.floor(rait / (375 * 5));
					imgh = 20;
					imgw = 25;
					rait -= mod * 375 * 5;
					break;
				case 4:
					imgh = 16;
					imgw = 16;
					mod = Math.floor(rait / (375));
					rait -= mod * 375;
					break;
				case 3:
					mod = Math.floor(rait / (75));
					imgh = 13;
					imgw = 13;
					rait -= mod * 75;
					break;
				case 2:
					mod = Math.floor(rait / (25));
					imgh = 11;
					imgw = 11;
					rait -= mod * 25;
					break;
				case 1:
					mod = Math.floor(rait / (5));
					imgh = 11;
					imgw = 11;
					rait -= mod * 5;
					break;
			}
			if(mod > 0) {
				for(var k = 0; k < mod; k++) {
					var $img_tmp = $('<img src="" style="vertical-align:middle" />');
					if(level == 8) {
						$img_tmp.attr('data-src', imageSrc('super1', 6));
						$img_tmp.attr('data-pic', 1);
						$img_tmp.addClass('super-crystal animated');
						animElements.add($img_tmp);
					} else {
						$img_tmp.attr('data-src', imageSrc(92 + level, 6));
					}
					setupImg($img_tmp);
					calcWidth += imgw + 15;
					$d.append($img_tmp);
				}
			}
			level--;
		}
	} else if(rait < 0) {
		level = 5;
		while(rait <= -5) {
			switch(level) {
				case 5:
					mod = Math.floor(-rait / (375 * 5));
					imgh = 18;
					imgw = 25;
					rait += mod * 375 * 5;
					break;
				case 4:
					mod = Math.floor(-rait / (375));
					imgh = 18;
					imgw = 22;
					rait += mod * 375;
					break;
				case 3:
					mod = Math.floor(-rait / (75));
					imgh = 12;
					imgw = 19;
					rait += mod * 75;
					break;
				case 2:
					mod = Math.floor(-rait / (25));
					imgh = 14;
					imgw = 14;
					rait += mod * 25;
					break;
				case 1:
					mod = Math.floor(-rait / (5));
					imgh = 9;
					imgw = 11;
					rait += mod * 5;
					break;
			}
			if(mod > 0) {
				for(var k = 0; k < mod; k++) {
					var $img_tmp = $('<img src="" style="vertical-align:baseline" />');
					$img_tmp.attr('data-src', imageSrc(99 + level, 0, 1));
					$d.append($img_tmp);
					setupImg($img_tmp);
					calcWidth += imgw + 15;
				}
			}
			level--;
		}
	}
	return $d;
	
}

function calcRating(rating) {
	var RATING_RANGES = [100, 500, 1000, 5000, 10000];
	var ret = {
		'aura': false,
		'nimb': false,
		'body': false
	};
	var badRating = false;

	if (rating < 0) {
		badRating = true;
		if (rating < -100)
			rating = -100;
		rating = -rating;
	}

	var indexNimb = -1,
        indexAura = -1;
	
	for (var k = 0; k < RATING_RANGES.length; ++k) {
		if (rating >= RATING_RANGES[k])
			indexAura = 1 + k;
	}

	var r = rating;
	if (r > 500000) r = 500000;
	if (r >= 10000) {
		indexNimb = Math.floor(r / 100000) - 1;
	} else if (r >= 5000) {
		indexNimb = Math.floor(r / 1000) - 5; //0,1,2,3,4
		if (indexNimb == 0) indexNimb = -1;
	} else if (r >= 1000) {
		indexNimb = Math.floor(r / 1000); //1,2,3,4
		if (indexNimb < 2) indexNimb = -1;
	} else if (r >= 500) {
		indexNimb = Math.floor(r / 100) - 5; //0,1,2,3,4
		if (indexNimb == 0) indexNimb = -1;
	} else if (r >= 100) {
		indexNimb = Math.floor(r / 100); //1,2,3,4
		if (indexNimb < 2) indexNimb = -1;
	} else {
		if (rating >= 50) indexNimb = 4;
		else if (rating >= 25) indexNimb = 3;
		else if (rating >= 20) indexNimb = 2;
		else if (rating >= 15) indexNimb = 1;
		else if (rating >= 10) indexNimb = 0;
	}


	if (indexNimb >= 0) {
		if (badRating == true)
			indexNimb = 100 + indexNimb;
		else
			indexNimb = 106 + indexNimb;
	}

	if (indexAura >= 0 && badRating == false)
		ret.aura = indexAura;

	if (indexNimb >= 0)
		ret.nimb = indexNimb;

	if (indexAura >= 0 && badRating == true) {
		ret.body = 105;
	}
	return ret;
}

var persImgs = {};

function persImgLoad(e) { // old function, seems to work properly
	try {

		var trg = this; //e.currentTarget;
		var type = trg.getAttribute('data-type');
		var persid = trg.getAttribute('data-persid');

		var persContainer = $('#pers_' + persid);
		var headId = persContainer.attr('data-head') || -1;
		var bodyId = persContainer.attr('data-body') || -1;

		if (type == 'head') {
			persImgs[persid].headReady = 1;
		} else if (type == 'body') {
			persImgs[persid].bodyReady = 1;
		}

		if (persImgs[persid].bodyReady != 0 && persImgs[persid].headReady != 0) {

			var aCanvas = persContainer.find('canvas');
			if (aCanvas && aCanvas.length) {
				aCanvas = aCanvas.get(0);
				var canvasContext = aCanvas.getContext('2d');
				canvasContext.clearRect(0, 0, aCanvas.width, aCanvas.height);

				if (persImgs[persid].bodyReady == 1) {
					var x = Math.floor(aCanvas.width / 2 - persImgs[persid].body.width / 2);
					var y = aCanvas.height - persImgs[persid].body.height;
					canvasContext.drawImage(persImgs[persid].body, x, y);
				}
				if (persImgs[persid].headReady == 1) {
					var d = 0;
					if (bodyId > 0 && headId > 0) {
						if (BODY_DELTAY[bodyId - 1]) d = BODY_DELTAY[bodyId - 1] + HEAD_DELTAY[headId - 1];
						else d = persImgs[persid].body.height + HEAD_DELTAY[headId - 1];
					}
					var x = Math.floor(aCanvas.width / 2 - persImgs[persid].head.width / 2);
					var y = aCanvas.height - d - persImgs[persid].head.height;
					canvasContext.drawImage(persImgs[persid].head, x, y);
				}

				var nimb = $(aCanvas).attr('data-nimb');
				if(nimb) {
					var $nimb = $('<img src="" class="nimb" style="position:absolute;z-index:0" />');
					$nimb.attr('src', imageSrc(nimb, 6, null, $nimb));
					$nimb.bind('load', function (e) {
						var trg = e.currentTarget;
						//centering
						var l = $(trg).parent().width() / 2 - trg.width / 2;
						trg.style.left = l + 'px';
						//calculating bottom
						b = $(aCanvas).attr('data-body');
						h = $(aCanvas).attr('data-head');
						var calc = BODY_DELTAY[b - 1] + HEAD_DELTAY[h - 1] + persImgs[persid].head.height + 1;
						trg.style.bottom = calc + 'px';
					});
					$(aCanvas).parents('.pers').append($nimb);
				}
			}

		}

	} catch (err) { }
}

function imagesRenderer($canvas, params, animated, params_per_img, offset_x, offset_y) {

	if (!$canvas.length || typeof params != 'object') return;
	params_per_img = params_per_img || 4;
	offset_x = offset_x || 0;
	offset_y = offset_y || 0;
	animated = animated || 0;
	
	var self = this,
        canvas_w = 0,
        canvas_h = 0,
        context = $canvas.get(0).getContext('2d'),
        imgs = [],
        all_imgs_loaded = false,
        imgs_count = 0,
        anim_type, times, seqs, cur_frame = 0,
        last_t = (new Date()).getTime();

	function __centerCanvas() {
		var max_w = 0,
            max_h = 0;
		for (var i = 0, ii = imgs.length; i < ii; i++) {
			if (imgs[i].img.width > max_w) max_w = imgs[i].img.width;
			if (imgs[i].img.height > max_h) max_h = imgs[i].img.height;
		}
		canvas_w = max_w;
		canvas_h = max_h;
		$canvas.attr('width', canvas_w + 'px').attr('height', canvas_h + 'px');
		$canvas.css('left', parseInt($canvas.parent().width() / 2 - canvas_w / 2) + 'px');
		repaintCSSPage();
	}
	
	this.addImage = function (src, dx, dy, anchor) {
		all_imgs_loaded = false;
		var cb = null;
		if (!animated) cb = self.draw;
		var img = new Image(),
            img_obj = {
            	'img': img,
            	'dx': dx,
            	'dy': dy,
            	'anchor': anchor,
            	'loaded': 0
            };
		imgs.push(img_obj);
		img.data_src = imageSrc(src, 0);
		setupImg(img, function() {
			img_obj.loaded = 1;
			all_imgs_loaded = true;
			for (var i = 0, ii = imgs.length; i < ii; i++)
				if (!imgs[i].loaded) all_imgs_loaded = false;
			if (all_imgs_loaded) __centerCanvas();
			if (typeof cb == 'function') cb();
		});
	}

	function __drawImage(img_obj, offset_x, offset_y) {
		offset_x = offset_x || 0;
		offset_y = offset_y || 0;
		var x = parseInt(img_obj.dx),
            y = parseInt(img_obj.dy),
            w = img_obj.img.width,
            h = img_obj.img.height,
            anchor = img_obj.anchor;
		if ((anchor & 2) > 0) y += Math.floor(canvas_h / 2 - h / 2); // V center
		else if (anchor & 16) y = y; // top
		else if ((anchor & 32) > 0) y = canvas_h - y - h; // bottom
		if ((anchor & 1) > 0) x += Math.floor(canvas_w / 2 - w / 2); // H center
		else if ((anchor & 4) > 0) x = x; // left
		else if ((anchor & 8) > 0) x = canvas_w - x; // right
		x += offset_x;
		y += offset_y;
		context.clearRect(0, 0, context.canvas.width, context.canvas.height);
		if (SYSTEM.isIOS()) drawImageIOSFix(context, img_obj.img, x, y, w, h);
		else context.drawImage(img_obj.img, x, y, w, h);
	}
	
	this.draw = function (new_offset_x, new_offset_y) {
		offset_x = new_offset_x || offset_x;
		offset_y = new_offset_y || offset_y;
		if (!all_imgs_loaded) return;
		if (animated) {
			if (cur_frame >= 0) {
				var t = times[cur_frame];
				if ((new Date()).getTime() - last_t >= t) {
					__drawImage(imgs[seqs[cur_frame] - 1], offset_x, offset_y);
					if (cur_frame == seqs.length - 1) {
						if (anim_type == 1) cur_frame = -1;
						if (anim_type == 2) cur_frame = 0;
						if (anim_type == 3) cur_frame = seqs.length - 1;
					} else {
						cur_frame++;
					}
					last_t = (new Date()).getTime();

				}
			}
		} else {
			for (var i = 0, ii = imgs.length; i < ii; i++) __drawImage(imgs[i], offset_x, offset_y);
		}
	}
	
	if (animated) {
		anim_type = params[0];
		imgs_count = params[1];
		times = params[params.length - 2].split(',');
		seqs = params[params.length - 1].split(',');
		params = params.slice(2, params.length - 2);
	} else {
		imgs_count = parseInt(params.length / params_per_img);
	}
	for (var j = 0; j < imgs_count; j++) {
		self.addImage(params[j * params_per_img], params[j * params_per_img + 1], params[j * params_per_img + 2], params[j * params_per_img + 3]);
	}
	
}

function scrollToTab($tabs, tabIndex) {
	
	tabIndex = tabIndex || 0;
	var $tab = $tabs.find('.tabs__tab[data-index="' + tabIndex + '"]'),
        tab_l = $tab.position().left,
        tabs_header_w = $tabs.find('.tabs__header').outerWidth(),
        $tabs_header_wrapper = $tabs.find('.tabs__header .wrapper'),
        wrapper_w = $tabs_header_wrapper.outerWidth(),
        new_wrapper_l = -tab_l + 70;
	if(wrapper_w > tabs_header_w) {
		if(tab_l == 0) {
			new_wrapper_l = 0;
		} else {
			if(new_wrapper_l > 0) new_wrapper_l = 0;
			if(tabs_header_w - wrapper_w - new_wrapper_l > 0) new_wrapper_l = -(wrapper_w - tabs_header_w);
		}
		$tabs_header_wrapper[0].style.cssText = 'left:' + new_wrapper_l + 'px;';
	}

}

function makePersSelector($pers) {
	if (!$pers.length) return $pers;

	var rightHeadI = 0,
        leftHeadI = 0,
        rightBodyI = 0,
        leftBodyI = 0,
        selectorWidth = 400,
        selectorHalfWidth = Math.floor(selectorWidth / 2),
        selectorHeight = 220,
        selectorHalfHeight = Math.floor(selectorHeight / 2),
        headsHeight = 100,
        bodiesHeight = 120,
        changeHead = parseInt($pers.attr('data-change-head')) || 0,
        changeBody = parseInt($pers.attr('data-change-body')) || 0,
        curHeadId = parseInt($pers.attr('data-head')) || 0,
        curHeadI = 0,
        curBodyId = parseInt($pers.attr('data-body')) || 0,
        curBodyI = 0,
        rangeHeads = [],
        rangeBodies = [],
        dyHeads = [],
        dyBodies = [];
	
	//if($parent.width() < selectorWidth) {
	selectorWidth = $parent.width() - 10;
	selectorHalfWidth = Math.floor(selectorWidth / 2);
	//}

	if ($pers.attr('data-range-head')) rangeHeads = $pers.attr('data-range-head').split(',');
	if ($pers.attr('data-range-body')) rangeBodies = $pers.attr('data-range-body').split(',');
	if ($pers.attr('data-dy-head')) dyHeads = $pers.attr('data-dy-head').split(',');
	if ($pers.attr('data-dy-body')) dyBodies = $pers.attr('data-dy-body').split(',');

	if (!rangeHeads.length) {
		rangeHeads = [2, 4, 13, 19, 22, 25, 31, 34, 36, 41, 43, 52, 54, 55, 57, 59, 60, 61, 64, 65, 66, 69, 71, 74, 75, 76, 78];
		dyHeads = [];
	}
	if (!rangeBodies.length) {
		rangeBodies = [9, 13, 19, 21, 27, 29, 31, 36, 40, 43, 45, 47, 49, 50, 51, 52, 54, 55, 56, 59, 61, 62, 63];
		dyBodies = [];
	}
	
	if (curHeadId == -1) {
		curHeadI = getRandomInt(0, rangeHeads.length - 1);
		curHeadId = rangeHeads[curHeadI];
	} else {
		curHeadI = -1;
		for (var i = 0, ii = rangeHeads.length; i < ii; i++) {
			if (parseInt(rangeHeads[i]) == curHeadId) curHeadI = i;
		}
		if (curHeadI == -1) {
			curHeadI = 0;
			curHeadId = rangeHeads[curHeadI];
		}
	}
	if (curBodyId == -1) {
		curBodyI = getRandomInt(0, rangeBodies.length - 1);
		curBodyId = rangeBodies[curBodyI];
	} else {
		curBodyI = -1;
		for (var i = 0, ii = rangeBodies.length; i < ii; i++) {
			if (parseInt(rangeBodies[i]) == curBodyId) curBodyI = i;
		}
		if (curBodyI == -1) {
			curBodyI = 0;
			curBodyId = rangeBodies[curBodyI];
		}
	}

	$pers.css({
		'width': selectorWidth + 'px',
		'height': selectorHeight + 'px'
	});

	var $hidden = $('<input type="hidden" id="headbody" name="headbody" value="' + (curHeadId + ':' + curBodyId) + '"/>');
	$pers.append($hidden);

	function __updateInputHidden() {
		$hidden.val(rangeHeads[curHeadI] + ':' + rangeBodies[curBodyI]);
	}

	var deltaX = 5,
        deltaY = 15,
        elemMaxWidth = 90; // todo: recount it

	function __loadImg($elem) {

		if (!$elem.length) return;
		var elem_id = $elem.attr('data-id'),
            is_head = $elem.hasClass('head'),
            is_body = $elem.hasClass('body'),
            src = '';
		if (typeof elem_id == 'undefined' || elem_id == null) return;
		if (is_head) src = imageSrc(elem_id + '_', 1, 0);
		if (is_body) src = imageSrc('b' + elem_id + '_', 1, 0);

		var $img = $('<img />');
		$elem.append($img);

		$img.attr('src', src);
		$img.error(function () {
			if (is_head) src = imageSrc(elem_id + '_', 7, 0);
			if (is_body) src = imageSrc('b' + elem_id + '_', 7, 0);
			$img.attr('src', src);
		});
		$img.load(function () {
			
			var dy = 0,
			    left = parseInt($elem.width() / 2 - $(this).width() / 2);
			if (is_head) {
				$(this).addClass('head_' + elem_id);
				dy = __getHeadDy(elem_id);
				$(this).css({
					'left': left + 'px',
					'bottom': dy + 'px'
				});
			}
			if (is_body) {
				$(this).addClass('body_' + elem_id);
				dy = __getBodyDy(elem_id) - $(this).height();
				$(this).css({
					'left': left + 'px',
					'top': dy + 'px'
				});
			}
		});

	}

	function __refreshOpacity() {
		return;
		$pers.find('.head').each(function () {
			if (parseInt($(this).css('left')) < selectorHalfWidth - elemMaxWidth / 2 || parseInt($(this).css('left')) > selectorHalfWidth + elemMaxWidth / 2)
				$(this).css('opacity', 0.4);
			else
				$(this).css('opacity', 1.0);
		});
		$pers.find('.body').each(function () {
			if (parseInt($(this).css('left')) < selectorHalfWidth - elemMaxWidth / 2 || parseInt($(this).css('left')) > selectorHalfWidth + elemMaxWidth / 2)
				$(this).css('opacity', 0.4);
			else
				$(this).css('opacity', 1.0);
		});
	}

	function __getHeadDy(head_id) {
		var dy = 0;
		if (dyHeads.length) dy = dyHeads[__findHeadI(head_id)];
		else dy = HEAD_DELTAY_BIG[head_id - 1];
		return dy;
	}

	function __getBodyDy(body_id) {
		var dy = 0;
		if (dyBodies.length) dy = dyBodies[__findBodyI(body_id)];
		else dy = BODY_DELTAY_BIG[body_id - 1];
		return dy;
	}

	function __findHeadI(head_id) {
		for (var i = 0, ii = rangeHeads.length; i < ii; i++) {
			if (head_id == rangeHeads[i]) return i;
		}
		return 0;
	}

	function __findBodyI(body_id) {
		for (var i = 0, ii = rangeBodies.length; i < ii; i++) {
			if (body_id == rangeBodies[i]) return i;
		}
		return 0;
	}

	function __init() {

		var head_i, body_i, x, y;

		head_i = __findHeadI(curHeadId);
		if (head_i > rangeHeads.length - 1) head_i = 0;
		curHeadI = head_i;
		x = selectorHalfWidth - parseInt(elemMaxWidth / 2);
		y = bodiesHeight;
		for (; ; head_i++) {
			if (head_i > rangeHeads.length - 1) head_i = 0;
			var head_id = rangeHeads[head_i],
                $elem = $('<div class="head elem" data-id="' + head_id + '"></div>');
			$elem.css({
				'left': x + 'px',
				'bottom': y + 'px',
				'width': elemMaxWidth + 'px'
			});
			$pers.append($elem);
			__loadImg($elem);
			x += deltaX + elemMaxWidth;
			y += deltaY;
			if (x >= selectorWidth + elemMaxWidth) {
				rightHeadI = head_i;
				break;
			}
		}
		body_i = __findBodyI(curBodyId);
		if (body_i > rangeBodies.length - 1) body_i = 0;
		curBodyI = body_i;
		x = selectorHalfWidth - parseInt(elemMaxWidth / 2),
        y = headsHeight;
		for (; ; body_i++) {
			if (body_i > rangeBodies.length - 1) body_i = 0;
			var body_id = rangeBodies[body_i],
                $elem = $('<div class="body elem" data-id="' + body_id + '"></div>');
			$elem.css({
				'left': x + 'px',
				'top': y + 'px',
				'width': elemMaxWidth + 'px'
			});
			$pers.append($elem);
			__loadImg($elem);
			x += deltaX + elemMaxWidth;
			y += deltaY;
			if (x >= selectorWidth + elemMaxWidth) {
				rightBodyI = body_i;
				break;
			}
		}
		head_i = __findHeadI(curHeadId) - 1;
		if (head_i < 0) head_i = rangeHeads.length - 1;
		x = selectorHalfWidth - deltaX - elemMaxWidth - parseInt(elemMaxWidth / 2);
		y = bodiesHeight + deltaY;
		for (; ; head_i--) {
			if (head_i < 0) head_i = rangeHeads.length - 1;
			var head_id = rangeHeads[head_i],
                $elem = $('<div class="head elem" data-id="' + head_id + '"></div>');
			$elem.css({
				'left': x + 'px',
				'bottom': y + 'px',
				'width': elemMaxWidth + 'px'
			});
			$pers.append($elem);
			__loadImg($elem);
			x -= deltaX + elemMaxWidth;
			y += deltaY;
			if (x <= -elemMaxWidth * 2) {
				leftHeadI = head_i;
				break;
			}
		}
		body_i = __findBodyI(curBodyId) - 1;
		if (body_i < 0) body_i = rangeBodies.length - 1;
		x = selectorHalfWidth - deltaX - elemMaxWidth - parseInt(elemMaxWidth / 2);
		y = headsHeight + deltaY;
		for (; ; body_i--) {
			if (body_i < 0) body_i = rangeBodies.length - 1;
			var body_id = rangeBodies[body_i],
                $elem = $('<div class="body elem" data-id="' + body_id + '"></div>');
			$elem.css({
				'left': x + 'px',
				'top': y + 'px',
				'width': elemMaxWidth + 'px'
			});
			$pers.append($elem);
			__loadImg($elem);
			x -= deltaX + elemMaxWidth;
			y += deltaY;
			if (x <= -elemMaxWidth * 2) {
				leftBodyI = body_i;
				break;
			}
		}

		if (changeHead) {
			var $prevHead = $('<div class="selector prev-head"></div>');
			$pers.append($prevHead);
			bindClick($prevHead, function (e) {
				e.preventDefault();
				__changeHead(-1);
			});
			var $nextHead = $('<div class="selector next-head"></div>');
			$pers.append($nextHead);
			bindClick($nextHead, function (e) {
				e.preventDefault();
				__changeHead(1);
			});
		}

		if (changeBody) {
			var $prevBody = $('<div class="selector prev-body"></div>');
			$pers.append($prevBody);
			bindClick($prevBody, function () {
				__changeBody(-1);
			});
			var $nextBody = $('<div class="selector next-body"></div>');
			$pers.append($nextBody);
			bindClick($nextBody, function () {
				__changeBody(1);
			});
		}

		var $leftFader = $('<div class="fader left"></div>');
		$pers.append($leftFader);

		var $rightFader = $('<div class="fader right"></div>');
		$pers.append($rightFader);

		//__refreshOpacity();
	}
	__init();

	var allowChangeHead = changeHead;

	function __changeHead(direction) {
		if (!changeHead || !allowChangeHead) return;
		allowChangeHead = 0;
		direction = direction || 1;
		if (direction == 1) {
			// add
			var $lastHead = $pers.find('.head[data-id="' + rangeHeads[rightHeadI] + '"]'),
                x = parseInt($lastHead.css('left')) + deltaX + elemMaxWidth,
                y = parseInt($lastHead.css('bottom')) + deltaY;
			rightHeadI++;
			if (rightHeadI > rangeHeads.length - 1) rightHeadI = 0;
			var $elem = $('<div class="head elem" data-id="' + rangeHeads[rightHeadI] + '"></div>');
			$elem.css({
				'left': x + 'px',
				'bottom': y + 'px',
				'width': elemMaxWidth + 'px'
			});
			$pers.append($elem);
			__loadImg($elem);
			// remove
			$pers.find('.head[data-id="' + rangeHeads[leftHeadI] + '"]').remove();
			leftHeadI++;
			if (leftHeadI > rangeHeads.length - 1) leftHeadI = 0;
		} else {
			// add
			var $lastHead = $pers.find('.head[data-id="' + rangeHeads[leftHeadI] + '"]'),
                x = parseInt($lastHead.css('left')) - deltaX - elemMaxWidth,
                y = parseInt($lastHead.css('bottom')) + deltaY;
			leftHeadI--;
			if (leftHeadI < 0) leftHeadI = rangeHeads.length - 1;
			var $elem = $('<div class="head elem" data-id="' + rangeHeads[leftHeadI] + '"></div>');
			$elem.css({
				'left': x + 'px',
				'bottom': y + 'px',
				'width': elemMaxWidth + 'px'
			});
			$pers.append($elem);
			__loadImg($elem);
			// remove
			$pers.find('.head[data-id="' + rangeHeads[rightHeadI] + '"]').remove();
			rightHeadI--;
			if (rightHeadI < 0) rightHeadI = rangeHeads.length - 1;
		}

		$pers.find('.head').each(function () {
			var $head = $(this),
                id = $head.attr('data-id'),
                x = parseInt($head.css('left')) - direction * (deltaX + elemMaxWidth);
			if (direction == 1) { // to the right
				if (id == curHeadId || parseInt($head.css('left')) < selectorHalfWidth) y = parseInt($head.css('bottom')) + deltaY;
				else y = parseInt($head.css('bottom')) - deltaY;
			} else { // to the left
				if (id == curHeadId || parseInt($head.css('left')) > selectorHalfWidth) y = parseInt($head.css('bottom')) + deltaY;
				else y = parseInt($head.css('bottom')) - deltaY;
			}
			$head.css({
				'left': x + 'px',
				'bottom': y + 'px'
			});
		});
		if (direction == 1) {
			curHeadI++;
			if (curHeadI > rangeHeads.length - 1) curHeadI = 0;
			curHeadId = rangeHeads[curHeadI];
		} else {
			curHeadI--;
			if (curHeadI < 0) curHeadI = rangeHeads.length - 1;
			curHeadId = rangeHeads[curHeadI];
		}
		__updateInputHidden();
		//setTimeout(function () {
		// __refreshOpacity();
		allowChangeHead = 1;
		//}, 210);
	}

	var allowChangeBody = changeBody;

	function __changeBody(direction) {
		if (!changeBody || !allowChangeBody) return;
		allowChangeBody = 0;
		direction = direction || 1;
		if (direction == 1) {
			// add
			var $lastBody = $pers.find('.body[data-id="' + rangeBodies[rightBodyI] + '"]'),
                x = parseInt($lastBody.css('left')) + deltaX + elemMaxWidth,
                y = parseInt($lastBody.css('top')) + deltaY;
			rightBodyI++;
			if (rightBodyI > rangeBodies.length - 1) rightBodyI = 0;
			var $elem = $('<div class="body elem" data-id="' + rangeBodies[rightBodyI] + '"></div>');
			$elem.css({
				'left': x + 'px',
				'top': y + 'px',
				'width': elemMaxWidth + 'px'
			});
			$pers.append($elem);
			__loadImg($elem);
			// remove
			$pers.find('.body[data-id="' + rangeBodies[leftBodyI] + '"]').remove();
			leftBodyI++;
			if (leftBodyI > rangeBodies.length - 1) leftBodyI = 0;
		} else {
			// add
			var $lastBody = $pers.find('.body[data-id="' + rangeBodies[leftBodyI] + '"]'),
                x = parseInt($lastBody.css('left')) - deltaX - elemMaxWidth,
                y = parseInt($lastBody.css('top')) + deltaY;
			leftBodyI--;
			if (leftBodyI < 0) leftBodyI = rangeBodies.length - 1;
			var $elem = $('<div class="body elem" data-id="' + rangeBodies[leftBodyI] + '"></div>');
			$elem.css({
				'left': x + 'px',
				'top': y + 'px',
				'width': elemMaxWidth + 'px'
			});
			$pers.append($elem);
			__loadImg($elem);
			// remove
			$pers.find('.body[data-id="' + rangeBodies[rightBodyI] + '"]').remove();
			rightBodyI--;
			if (rightBodyI < 0) rightBodyI = rangeBodies.length - 1;
		}
		$pers.find('.body').each(function () {
			var $body = $(this),
                id = $body.attr('data-id'),
                x = parseInt($body.css('left')) - direction * (deltaX + elemMaxWidth),
                opacity = 0.4;
			if (direction == 1) { // to the right
				if (id == curBodyId || parseInt($body.css('left')) < selectorHalfWidth) y = parseInt($body.css('top')) + deltaY;
				else y = parseInt($body.css('top')) - deltaY;
			} else { // to the left
				if (id == curBodyId || parseInt($body.css('left')) > selectorHalfWidth - elemMaxWidth) y = parseInt($body.css('top')) + deltaY;
				else y = parseInt($body.css('top')) - deltaY;
			}
			$body.css({
				'left': x + 'px',
				'top': y + 'px'
			});
		});
		if (direction == 1) {
			curBodyI++;
			if (curBodyI > rangeBodies.length - 1) curBodyI = 0;
			curBodyId = rangeBodies[curBodyI];
		} else {
			curBodyI--;
			if (curBodyI < 0) curBodyI = rangeBodies.length - 1;
			curBodyId = rangeBodies[curBodyI];
		}
		__updateInputHidden();
		//setTimeout(function () {
		//__refreshOpacity();
		allowChangeBody = 1;
		//}, 210);
	}

	return $pers;
}

function checkboxRadioClick($elem) {
	if (!$elem) return;
	if ($elem.getAttribute('type') == 'checkbox') {
		if ($elem.checked) $elem.setAttribute('checked', 'checked');
		else $elem.removeAttribute('checked');
	}
	if ($elem.getAttribute('type') == 'radio') {
		$elem.setAttribute('checked', 'checked');
		var name = $elem.getAttribute('name');
		if (name) {
			var $parent = getParentByTagName($elem, 'form');
			if (!$parent) $parent = document.getElementsByTagName('body')[0];
			var $elems = $parent.getElementsByTagName('input');
			for (var i = 0, ii = $elems.length; i < ii; i++) {
				if ($elems[i] && $elems[i] != $elem && $elems[i].getAttribute('type') == 'radio' && $elems[i].getAttribute('name') == name)
					$elems[i].removeAttribute('checked');
			}
		}
	}
}


// --------------------------------------------------------------------------------------------
// PORTAL -------------------------------------------------------------------------------------

var PORTAL_SPEED = 1,
    PORTAL_MAX_WIDTH = 110;

function Portal($canvas, params, cb_open, cb_finish, clear_canvas, again_on_finish, again_on_finish_timeout) {
	
	var self = this,
		canvas_w = null, canvas_h = null, context = null,
		type = params[0],
		params1 = {'color1':null, 'color2':null, 'color3':null},
		params2 = {'gradients':[], 'imgs_grid':[], 'img':null, 'img_w':0, 'img_w':0, 'imgReady':0, 'imgsOffsetY':0, 'imgsFinished':0},
		canvas = $canvas.get(0);
	if($canvas && $canvas.length) {
		canvas_w = $canvas.attr('width') || $canvas.width();
		canvas_h = $canvas.attr('height') || $canvas.height();
		context = canvas.getContext('2d');
	}
	if(type != -2) type = -1;
	
	function __fixColor(color) {
		color += '';
		if(color.indexOf('#') == 0) color = color.substr(1);
		for(var i = 0, ii = 6-color.length; i < ii; i++) color = '0'+color;
		color = '#'+color;
		return color;
	}
	
	if(type == -2) {
		
		params2.imgsOffsetY = -1.2*canvas_h;
		
		params = params.slice(1);
		
		var params_per_gradient = 3,
			gradients_count = parseInt(params.length / params_per_gradient);
		
		for(var i = 0; i < gradients_count; i++) {
			params2.gradients.push([
				__fixColor(params[i*params_per_gradient]),
				__fixColor(params[i*params_per_gradient+1]),
				parseInt(params[i*params_per_gradient+2],16)/100]
			);
		}
		
		params = params.slice(gradients_count*params_per_gradient);
		if(params.length > 0) {
			params2.img = new Image();
			params2.img.src = imageSrc(params[0], 0, null, params2.img);
			params2.img.onload = function() {
				params2.img_w = params2.img.width;
				params2.img_h = params2.img.height;
				__makeImgsGrid();
				params2.imgReady = 1;
			}
		}
		
	} else {
		
		if(params[1] == -2 || params[2] == -2) {
			params1.color1 = '#ff0000';
			params1.color2 = '#ffffff';
			params1.color3 = '#000000';
		} else {
			params1.color1 = __fixColor(params[0] == -1 ? '0000ff' : params[0]);
			params1.color2 = __fixColor(params[1] == -1 ? 'ffffff' : params[1]);
			params1.color3 = __fixColor(params[2] == -1 ? '000000' : params[2]);
		}
		
	}
	
	function __makeImgsGrid() {
		try {
			if(!canvas_w && !canvas_h) return;
			params2.imgs_grid = [];
			var rows = 1 + parseInt(canvas_h / params2.img_h),
				cols = 0, col_w = 0,
				x = 0, y = 0;
			for(var row = 0; row < rows; row++) {
				cols = getRandomInt(1, 2);
				col_w = parseInt(portalMaxWidth / cols);
				for(var col = 0; col < cols; col++) {
					x = getRandomInt(1, col_w-params2.img_w*(2/3)) + col_w*col;
					if(x > portalMaxWidth - params2.img_w) x = portalMaxWidth - params2.img_w;
					y = row*params2.img_h + getRandomInt(0, params2.img_h/2);
					params2.imgs_grid.push([x,y]);
				}
			}
			params2.imgsOffsetY = -1.2*canvas_h;
		} catch(err) {
			
		}
	}
	
	var last_t = (new Date()).getTime(),
		allowDrawPortal = (type == -1) ? true : false,
		imgsSpeed = PORTAL_SPEED / 4,
		portalSpeed = PORTAL_SPEED / 5.7,
		portalWidth = 0,
		portalDirection = 1,
		portalFinished = false,
		portalMaxWidth = PORTAL_MAX_WIDTH,
		portalIsOpen = false;
	portalSpeed *= 0.9;
	
	function __reset() {
		try {
			if(clear_canvas) {
				//context.clearRect(0, 0, canvas_w, canvas_h);
				canvas.width = canvas.width;
			}
			last_t = (new Date()).getTime();
			allowDrawPortal = (type == -1) ? true : false;
			params2.imgsOffsetY = -1.2*canvas_h;
			params2.imgsFinished = false;
			portalWidth = 0;
			portalDirection = 1;
			portalFinished = false;
			portalIsOpen = false;
			__makeImgsGrid();
		} catch(err) {
			
		}
	}
	
	function __draw1(pos) {
		
		if(clear_canvas) {
			//context.clearRect(0, 0, canvas_w, canvas_h);
			canvas.width = canvas.width;
		}
		
		var cur_t = (new Date()).getTime();
		
		if(portalDirection == 1) {
			
			portalIsOpen = false;
			portalFinished = false;
			if(portalWidth >= portalMaxWidth) {
				portalDirection = -1;
				portalIsOpen = true;
				if(typeof cb_open == 'function') cb_open($canvas);
			}
			
		} else {
			
			portalIsOpen = true;
			portalFinished = false;
			if(portalWidth <= 0) {
				portalFinished = true;
				if(typeof cb_finish == 'function') cb_finish($canvas);
			}
			
		}
		
		if(!portalFinished) {
			
			portalWidth = parseInt(portalWidth + (cur_t - last_t) * portalSpeed * portalDirection);
			if(portalWidth >= portalMaxWidth && portalDirection == 1) {
				portalDirection = -1;
				portalWidth = portalMaxWidth;
				last_t = cur_t;
				if(typeof cb_open == 'function') cb_open($canvas);
				__draw1(pos);
				return;
			}
			
			var gradient,
				l_length,
				ly,
				lmax_length = 10,
				x1 = pos - portalWidth / 2,
				x2 = pos + portalWidth / 2;
			gradient = context.createLinearGradient(x1, 0, x2, 0);
			gradient.addColorStop(0, params1.color1);
			gradient.addColorStop(0.5, params1.color2);
			gradient.addColorStop(1, params1.color1);
			context.fillStyle = gradient;
			context.fillRect(x1, 0, portalWidth, canvas_h);
			context.fillStyle = params1.color3;
			for(var i =- portalWidth / 2; i < portalWidth / 2; i++) {
				l_length = Math.round(Math.random() * lmax_length);
				ly = Math.round(Math.random()*(canvas_h - l_length));
				context.fillRect(pos + i, ly, 1, l_length);
			}
			
		}
		
		last_t = cur_t;
		
		if(portalFinished) {
			if(again_on_finish) {
				if(again_on_finish_timeout) setTimeout(function() { __reset(); }, again_on_finish_timeout);
				else __reset();
			}
		}
		
	}
	
	function __draw2(pos) {
		
		if(clear_canvas) {
			//context.clearRect(0, 0, canvas_w, canvas_h);
			canvas.width = canvas.width;
		}
		
		var cur_t = (new Date()).getTime();
		
		if(portalDirection == 1) {
			
			portalIsOpen = false;
			portalFinished = false;
			if(portalWidth >= portalMaxWidth) {
				portalDirection = -1;
				portalIsOpen = true;
				if(typeof cb_open == 'function') cb_open($canvas);
			}
			
		} else {
			
			portalIsOpen = true;
			portalFinished = false;
			if(portalWidth <= 0) {
				portalFinished = true;
			}
			
		}
		
		if(!portalFinished) {
			
			portalWidth = parseInt(portalWidth + (cur_t - last_t) * portalSpeed * portalDirection);
			if(portalWidth >= portalMaxWidth && portalDirection == 1) {
				portalDirection = -1;
				portalWidth = portalMaxWidth;
				last_t = cur_t;
				if(typeof cb_open == 'function') cb_open($canvas);
				__draw2(pos);
				return;
			}
			
			var x1 = pos - portalWidth / 2,
				x2 = pos + portalWidth / 2,
				gradient = context.createLinearGradient(x1, 0, x2, 0),
				width = 0;
			for(var i = 0, ii = params2.gradients.length; i < ii; i++) {
				width += parseFloat(params2.gradients[i][2]);
				gradient.addColorStop(width - parseFloat(params2.gradients[i][2]), params2.gradients[i][0]);
				gradient.addColorStop(width, params2.gradients[i][1]);
			}
			context.fillStyle = gradient;
			context.fillRect(x1, 0, portalWidth, canvas_h);
			
		}
		
		if(!params2.imgsFinished) {
			params2.imgsOffsetY += (cur_t - last_t) * imgsSpeed;
			for(var i = 0, ii = params2.imgs_grid.length; i < ii; i++) {
				context.drawImage(params2.img, params2.imgs_grid[i][0] + pos - portalMaxWidth/2, params2.imgs_grid[i][1] + params2.imgsOffsetY, params2.img_w, params2.img_h);
			}
			if(params2.imgsOffsetY >= canvas_h) params2.imgsFinished = true;
		}
		
		last_t = cur_t;
		
		if(portalDirection == 1 && params2.imgsOffsetY >= -(canvas_h*2/3)) {
			portalFinished = false;
		}
		
		if(portalFinished && params2.imgsFinished) {
			if(typeof cb_finish == 'function') cb_finish($canvas);
			if(again_on_finish) {
				if(again_on_finish_timeout) setTimeout(function() { __reset(); }, again_on_finish_timeout);
				else __reset();
			}
		}
		
	}
	
	this.isOpen = function() {
		return portalIsOpen;
	}
	
	this.isFinished = function() {
		if(type == -1) return portalFinished;
		if(type == -2) return (portalFinished && params2.imgsFinished);
	}
	
	this.draw = function(ctx, pos, again) {
		
		if(again) __reset();
		else if(self.isFinished()) return;
		if(ctx) {
			context = ctx;
			canvas_w = context.canvas.width;
			canvas_h = context.canvas.height;
			if(!params2.imgs_grid.length) __makeImgsGrid();
		}
		if(!context) return;
		if(!pos && canvas_w) pos = canvas_w / 2;
		if(type == -1) __draw1(pos);
		else if(type == -2) __draw2(pos);
	}
	
}

function refreshPersImgsPositions($pers) {
	if ($pers.hasClass('ready')) return;
	$pers.find('img').each(function () {
		tagPersImgLoadCb({ target: $(this) });
	});
}

function parsePersTag($pers, $dom_elem, hlinks) {
	
	var pers_id = $dom_elem.attr('name') || Math.floor(Math.random() * 999),
        w = $dom_elem.attr('w') || 100,
        h = $dom_elem.attr('h') || 150,
        chh = $dom_elem.attr('change_head') || 0,
        chb = $dom_elem.attr('change_body') || 0,
        body_id = parseInt($dom_elem.attr('body')) || -1,
        head_id = parseInt($dom_elem.attr('head')) || -1,
        params = $dom_elem.attr('params'),
        rating = {},
        nimb = null,
        crown = null,
        aura = null,
        portal = null;
	if (h < 150) h = 150;
	$pers.css({
		'width': w + 'px',
		'height': h + 'px',
		'margin':'0'
	}).attr('id', 'pers_' + pers_id).attr('data-persid', pers_id);

	persImgs[pers_id] = {
		'head': 0,
		'body': 0,
		'headReady': 0,
		'bodyReady': 0
	};

	if (params) {

		params = params.split(/\s+/);
		rating = calcRating(params[params.length - 1]);

		if (rating.body) {

			var $img = $('<img style="z-index:10;position:absolute;bottom:0"/>');
			$img.attr('src', imageSrc(rating.body, 2, null, $img)).load(function (e) {
				var l = $(this).parent().width() / 2 - $(this).get(0).width / 2;
				$(this).css('left', l + 'px');
			});
			$pers.append($img);

		} else {

			head_id = parseInt(params[0]);
			$pers.attr('data-head', head_id);

			if (rating.nimb) $pers.attr('data-nimb', rating.nimb);
			nimb = $pers.attr('data-nimb');

			if (typeof $dom_elem.attr('crown') != 'undefined') crown = $dom_elem.attr('crown');

			aura = $dom_elem.attr('aura') || null;
			portal = $dom_elem.attr('portal');

			if (head_id > 0) {

				body_id = parseInt(params[1]);
				$pers.attr('data-body', body_id);

			} else {

				var imgs_count = -head_id,
                    img_id = null,
                    img_dx = null,
                    img_dy = null,
                    img_anchor = null,
                    img_extra = null,
                    img_type = null,
                    img_src = null,
                    $img = null,
                    loaded_suit_imgs = 0,
                    minTop = 999,
                    crownDy = null;
				var at_found_i = -1;
				for (var i = 0, ii = params.length; i < ii; i++) {
					if (params[i] == '@') at_found_i = i;
				}
				if (at_found_i != -1) {
					var crown_star_params = params.slice(at_found_i + 1, at_found_i + 5);
					params.splice(at_found_i, 5);
					crownDy = crown_star_params[0];
					imgs_count--;
				}

				if (crownDy != null) $pers.attr('data-crowdy', crownDy);
				for (var j = -0, jj = imgs_count; j < jj; j++) {

					img_id = params[1 + j * PARAMS_PER_SUIT];
					img_dx = params[1 + j * PARAMS_PER_SUIT + 1];
					img_dy = params[1 + j * PARAMS_PER_SUIT + 2];
					img_anchor = params[1 + j * PARAMS_PER_SUIT + 3];
					img_extra = params[1 + j * PARAMS_PER_SUIT + 4];

					img_type = 0;
					if (!isNaN(img_id)) {
						if (img_id >= 0 && img_id < 53) img_type = 2;
						if (img_id > 52 && img_id <= 102) {
							img_type = 1;
							img_id -= 53;
						}
					}

					$img = $('<img />');
					var head_h = 0;
					$img.attr('src', imageSrc(img_id, img_type, null, $img))
                        .attr('data-id', img_id)
                        .attr('data-dx', img_dx)
                        .attr('data-dy', img_dy)
                        .attr('data-anchor', img_anchor)
                        .attr('data-extra', img_extra)
                        .addClass('suit-part')
                        .css('z-index', (PARAMS_PER_SUIT + (imgs_count - j)))
                        .load(tagPersImgLoadCb).error(function (e) {
                        	$img.attr('src', imageSrc($(this).attr('data-id'), 0, 1, $img));
                        	var curonload = $img.load;
                        });

					$pers.append($img);

				}

				if (nimb) {
					var $nimb = $('<img class="nimb" style="display:none;position:absolute;z-index:12" />');
					$nimb.attr('src', imageSrc(nimb, 6, null, $nimb)).load(tagPersImgLoadCb);
					$pers.append($nimb);
				}
				if (crown) {
					var $crown = $('<img class="crown" style="display:none;position:absolute;z-index:15" />');
					if (crown == '') {
						crown = 91;
						$crown.attr('src', imageSrc(crown, 5, 1, $crown));
					} else {
						$crown.attr('src', imageSrc(crown + '_', 0, 1, $crown));
					}
					$crown.load(tagPersImgLoadCb);
					$pers.append($crown);
				}

			}

			var $aura;

			if(aura) {
				var aura_params = $dom_elem.attr('aura').split(';'),
                    type = aura_params[0];
				$aura = $('<canvas class="aura" style="z-index:0"></canvas>');
				$pers.append($aura);
				aura_params = aura_params.slice(1);
				if (type == 1) imagesRenderer($aura, aura_params, 0);
				else if (type == 2) animElements.add(new imagesRenderer($aura, aura_params, 1));
			} else if(rating.aura) {
				$aura = $('<img class="aura" src="'+imageSrc(rating.aura, 4, null, $aura)+'" style="z-index:0" />');
				$aura.load(function(e) {
					var l = 0;
					if($(this).parent().width() > 0 && $(this).get(0).width > 0) {
						l = $(this).parent().width() / 2 - $(this).get(0).width / 2;
					}
					$(this).css({
						'left': l + 'px',
						'bottom': 0
					});
				});
				$pers.append($aura);
			}
			
			if(portal) {
				$pers.attr('data-portal', 1);
				$pers.find('*').hide();
				var portal_params = $dom_elem.attr('portal').split(/\s+/),
                    $portal = $('<canvas class="portal"></canvas>');
				$portal.attr('width', $pers.width());
				$portal.attr('height', $pers.height());
				$pers.append($portal);
				animElements.add(new Portal($portal, portal_params, function($canvas) {
					$canvas.parent().find('*').show();
					refreshPersImgsPositions($canvas.parents('.pers'));
				}, function ($canvas) {
					setTimeout(function () {
						$canvas.parent().find('*').hide();
						$canvas.parent().find('.portal').show();
					}, 1900);
				}, true, true, 2000), 10);
			}

		}

	}

	if (head_id > 0) {
		var head = new Image();
		head.onload = persImgLoad;
		head.src = imageSrc(head_id, 1);
		head.setAttribute('data-persid', pers_id);
		head.setAttribute('data-type', 'head');
		$pers.attr('data-head', head_id);
		persImgs[pers_id]['head'] = head;
	} else {
		persImgs[pers_id]['headReady'] = -1;
	}

	if (body_id > 0) {
		var body = new Image();
		body.onload = persImgLoad;
		body.src = imageSrc(body_id, 2);
		body.setAttribute('data-persid', pers_id);
		body.setAttribute('data-type', 'body');
		$pers.attr('data-body', body_id);
		persImgs[pers_id]['body'] = body;
	} else {
		persImgs[pers_id]['bodyReady'] = -1;
	}
	
	var confirm = $dom_elem.attr('confirm') || $dom_elem.attr('data-confirm');
	$pers.attr('confirm', confirm);
	
	var href = getHrefForModalView($dom_elem.get(0), null, hlinks);
	if(href) {
		bindClickNavigate($pers, href);
	}
	
	if(chh || chb) {
		$pers.attr('data-head', $dom_elem.attr('head'));
		$pers.attr('data-body', $dom_elem.attr('body'));
		$pers.attr('data-range-head', $dom_elem.attr('range_head'));
		$pers.attr('data-range-body', $dom_elem.attr('range_body'));
		$pers.attr('data-dy-head', $dom_elem.attr('dy_head'));
		$pers.attr('data-dy-body', $dom_elem.attr('dy_body'));
		$pers.attr('data-change-head', $dom_elem.attr('change_head'));
		$pers.attr('data-change-body', $dom_elem.attr('change_body'));
		$pers = makePersSelector($pers);
	}

}

function tagPersImgLoadCb(e) {
	try {

		if (!e) return;

		var $me = $(e.target);

		if (!$me || !$me.length || !$me.is(':visible')) return;

		var $pers = $me.parent();

		if ($me.hasClass('ready')) return;
		$me.addClass('ready');
		
		$me.attr('width', $me.width());
		if ($me.attr('src').indexOf('heads') != -1) $me.addClass('head');

		var anchor = $me.attr('data-anchor'),
			dx = parseInt($me.attr('data-dx'), 10),
			dy = parseInt($me.attr('data-dy'), 10),
			top;
		if ((anchor & 2) > 0) { // V center
			top = $pers.height() / 2 - $me.height() / 2 + dy;
			$me.css('top', top + 'px');
		} else if (anchor & 16) { // top
			top = dy;
			$me.css('top', dy + 'px');
		} else if ((anchor & 32) > 0) { // bottom
			$me.css('bottom', (-dy) + 'px');
			top = $pers.height() - dy - $me.height();
		}

		if ((anchor & 1) > 0) { // H center
			var l = $pers.width() / 2 - $me.get(0).width / 2 + dx;
			$me.css('left', l + 'px');
		} else if ((anchor & 4) > 0) { // left
			$me.css('left', (dx) + 'px');
		} else if ((anchor & 8) > 0) { // right
			$me.css('right', dx + 'px');
		}
		
		repaintCSSPage();
		
		if ($pers.find('.suit-part.ready').length == $pers.find('.suit-part').length) {

			$pers.find('.nimb').each(function () {
				var $trg = $(this),
					portal = parseInt($pers.attr('data-portal')),
					l = $pers.width() / 2 - $trg.width() / 2;
				if ($pers.find('.head').length) {
					var head_bottom = parseInt($pers.find('.head').css('bottom')),
						head_height = $pers.find('.head').height(),
						calc = head_bottom + head_height + 1;
					$trg.css('left', l + 'px').css('bottom', calc + 'px');
					if (!portal) $trg.show();
				} else {
					var max_bottom = 0,
						cur_bottom = 0;
					$pers.find('img').each(function () {
						if (!$(this).hasClass('crown') && !$(this).hasClass('nimb') && !$(this).hasClass('aura') && !$(this).parent().hasClass('aura')) {
							cur_bottom = parseInt($(this).css('bottom')) + $(this).height();
							if (cur_bottom > max_bottom) max_bottom = cur_bottom;
						}
					});
					$trg.css('left', l + 'px').css('bottom', (max_bottom + 1) + 'px');
					if (!portal) $trg.show();
				}
			});
			$pers.find('.crown').each(function () {
				var $trg = $(this),
					crownDy = (typeof $pers.attr('data-crowndy') != 'undefined' ? $pers.attr('data-crowndy') : null),
					portal = parseInt($pers.attr('data-portal')),
					l = $pers.width() / 2 - $trg.width() / 2;
				if ($pers.find('.head').length) {
					var head_bottom = parseInt($pers.find('.head').css('bottom')),
						head_height = $pers.find('.head').height(),
						calc = head_bottom + head_height + 1;
					$trg.css('left', l + 'px').css('bottom', calc + 'px');
					if (!portal) $trg.show();
				} else {
					var max_bottom = 0,
						cur_bottom = 0;
					if (crownDy != null) {
						max_bottom = crownDy;
					} else {
						$pers.find('img').each(function () {
							if (!$(this).hasClass('crown') && !$(this).hasClass('nimb') && !$(this).hasClass('aura') && !$(this).parent().hasClass('aura')) {
								cur_bottom = parseInt($(this).css('bottom')) + $(this).height();
								if (cur_bottom > max_bottom) max_bottom = cur_bottom;
							}
						});
						max_bottom += 1;
					}
					$trg.css('left', l + 'px').css('bottom', max_bottom + 'px');
					if (!portal) $trg.show();
				}
			});
			
			$pers.addClass('ready');
			
			return;
		}

	} catch (err) {

	}
}

var SEX_MALE = 1,
	SEX_FEMALE = 2,
	SEX_NEUTRAL = 0,
	PART_HEAD = 1,
	PART_BODY = 2;

function parsePersRegTag($pers_reg, $dom_elem, hlinks) {
	
	var $chooser = $('<div class="pers-reg__chooser"></div>'),
		$chooser_header = $('<div class="pers-reg__header"></div>'),
		$chooser_pers = $('<div class="pers-reg__pers"></div>'),
		$chooser_parts = $('<div class="pers-reg__parts"></div>'),
		$previewer = $('<div class="pers-reg__previewer"></div>'),
		$previewer_pers = $('<div class="pers-reg__pers"></div>'),
		minimized = parseInt($dom_elem.attr('minimized')) || 0,
		range_head = ($dom_elem.attr('range_head') || '').split(','),
		range_body = ($dom_elem.attr('range_body') || '').split(','),
		sex_head = ($dom_elem.attr('sex_head') || '').split(','),
		sex_body = ($dom_elem.attr('sex_body') || '').split(','),
		dy_head = ($dom_elem.attr('dy_head') || '').split(','),
		dy_body = ($dom_elem.attr('dy_body') || '').split(','),
		male_head = [], male_body = [],
		female_head = [], female_body = [],
		sel = ($dom_elem.attr('sel') || '').split(','),
		sel_sex = parseInt(sel[0]),
		sel_part = parseInt(sel[1]),
		sel_male_head = parseInt(sel[2]),
		sel_male_body = parseInt(sel[3]),
		sel_female_head = parseInt(sel[4]),
		sel_female_body = parseInt(sel[5]),
		w = $dom_elem.attr('w') || SYSTEM.width,
		pers_head_html = '',
		pers_head_dy = 0,
		pers_body_html = '',
		pers_body_dy = 0;
	
	if(range_head.length != sex_head.length || range_head.length != dy_head.length) return;
	if(range_body.length != sex_body.length || range_body.length != dy_body.length) return;
	
	$pers_reg.attr('data-sel-male-head', sel_male_head)
		.attr('data-sel-male-body', sel_male_body)
		.attr('data-sel-female-head', sel_female_head)
		.attr('data-sel-female-body', sel_female_body)
		.css('width', w+'px');
	if(minimized) $pers_reg.addClass('minimized');
	
	var sex, object, dy;
	for(var i = 0, ii = sex_head.length; i < ii; i++) {
		sex = parseInt(sex_head[i]);
		object = parseInt(range_head[i]);
		dy = parseInt(dy_head[i]);
		if(sex == SEX_MALE) male_head.push([object, dy]);
		else if(sex == SEX_FEMALE) female_head.push([object, dy]);
		else if(sex == SEX_NEUTRAL) {
			male_head.push([object, dy]);
			female_head.push([object, dy]);
		}
	}
	for(var i = 0, ii = sex_body.length; i < ii; i++) {
		sex = parseInt(sex_body[i]);
		object = parseInt(range_body[i]);
		dy = parseInt(dy_body[i]);
		if(sex == SEX_MALE) male_body.push([object, dy]);
		else if(sex == SEX_FEMALE) female_body.push([object, dy]);
		else if(sex == SEX_NEUTRAL) {
			male_body.push([object, dy]);
			female_body.push([object, dy]);
		}
	}
	
	function makePages(objects, sex, part, visible, sel_object) {
		if(typeof visible == 'undefined') visible = true;
		var cols = Math.floor(w / 105),
			col_w = Math.floor(w / cols),
			col_h = 120,
			rows = 2,
			parts_per_page = rows * cols,
			pages = Math.ceil(objects.length / parts_per_page);
		var sel_page = 0;
		for(var i = 0, ii = objects.length; i < ii; i++) {
			if(objects[i][0] == sel_object) {
				sel_page = Math.floor((i+1) / parts_per_page);
				break;
			}
		}
		var html = '<pages data-sex="'+sex+'" data-part="'+part+'" style="display:'+(visible?'block':'none')+'" total="'+pages+'" sel="'+sel_page+'" h="'+col_h*rows+'" indic="0">';
		for(var p = 0; p < pages; p++) {
			if(objects.length) {
				html += '<page>'+makeTable(objects, rows, cols, col_w, col_h, sex, part, sel_object)+'</page>';
				objects = objects.slice(parts_per_page);
			}
		}
		html += '</pages>';
		return html;
	}
	
	function makeTable(objects, rows, cols, col_w, col_h, sex, part, sel_object) {
		var html = '<table>', i = 0, id = 0, dy = 0;
		for(var r = 0; r < rows; r++) {
			html += '<tr>';
			for(var c = 0; c < cols; c++) {
				var new_col_w = (c == cols - 1 ? 'auto' : col_w),
					first_in_row_class = (c == 0 ? 'first-in-row' : '');
				if(c == cols - 1) html += '<td>';
				else html += '<td style="width:'+new_col_w+'px;height:'+col_h+'px">';
				if(typeof objects[i] != 'undefined') {
					id = parseInt(objects[i][0]);
					dy = parseInt(objects[i][1]);
					var line_h = (part == PART_HEAD ? col_h : parseInt(col_h * 0.9));
					html += '<div data-part="'+part+'" data-sex="'+sex+'" data-id="'+id+'" data-dy="'+dy+'" class="pers-reg__part '+(sel_object == id ? 'sel' : '')+' '+first_in_row_class+'" style="width:'+new_col_w+'px;height:'+col_h+'px;line-height:'+line_h+'px">';
					var src = '', error_src = '';
					if(part == PART_HEAD) {
						src = imageSrc(id+'_',1,0,null,null,1);
						error_src = imageSrc(id+'_',7,0,null,null,1);
					} else {
						src = imageSrc('b'+id+'_',1,0,null,null,1);
						error_src = imageSrc('b'+id+'_',7,0,null,null,1);
					}
					if(part == PART_HEAD) {
						if(sex == sel_sex && sel_object == id) {
							pers_head_html += '<img style="vertical-align:bottom" data-src="'+src+'" data-error-src="'+error_src+'" />';
							pers_head_dy = dy;
						}
						html += '<img margin="0,0,'+dy+',0" data-src="'+src+'" data-error-src="'+error_src+'" />';
					} else {
						if(sex == sel_sex && sel_object == id) {
							pers_body_html += '<img style="vertical-align:bottom" data-src="'+src+'" data-error-src="'+error_src+'" />';
							pers_body_dy = dy;
						}
						html += '<img style="vertical-align:bottom" data-src="'+src+'" data-error-src="'+error_src+'" />';
					}
					html += '</div>';
				} else {
					html += '<div data-part="'+part+'" data-sex="'+sex+'" data-id="0" data-dy="0" class="pers-reg__part '+first_in_row_class+'" style="height:'+col_h+'px;line-height:'+col_h+'px"></div>';
				}
				html += '</td>';
				i++;
			}
			html += '</tr>';
		}
		html += '</table>';
		return html;
	}
	
	$chooser.append(
		$chooser_header,
		$chooser_parts,
		'<input type="hidden" name="sex" value="'+sel_sex+'" />',
		'<input type="hidden" name="part" value="'+sel_part+'" />',
		'<input type="hidden" name="head" value="'+(sel_sex == SEX_MALE ? sel_male_head : sel_female_head)+'" />',
		'<input type="hidden" name="body" value="'+(sel_sex == SEX_MALE ? sel_male_body : sel_female_body)+'" />'
	);
	$pers_reg.append($chooser, $previewer);
	
	var parts_html = 
		makePages(male_head, SEX_MALE, PART_HEAD, sel_sex == SEX_MALE && sel_part == PART_HEAD, sel_male_head) +
		makePages(male_body, SEX_MALE, PART_BODY, sel_sex == SEX_MALE && sel_part == PART_BODY, sel_male_body) +
		makePages(female_head, SEX_FEMALE, PART_HEAD, sel_sex == SEX_FEMALE && sel_part == PART_HEAD, sel_female_head) +
		makePages(female_body, SEX_FEMALE, PART_BODY, sel_sex == SEX_FEMALE && sel_part == PART_BODY, sel_female_body);
	convertDOM('<body>'+parts_html+'</body>', $chooser_parts, null, function($parent) {
		$parent.find('.pers-reg__part').each(function() {
			bindClick($(this), function(e, $elem) {
				e.preventDefault();
				var id = parseInt($elem.attr('data-id'));
				if(!id) return;
				var $pers_reg = $elem.parents('.pers-reg'),
					dy = parseInt($elem.attr('data-dy')),
					sex = parseInt($elem.attr('data-sex')),
					part = parseInt($elem.attr('data-part')),
					$pers = $pers_reg.find('.pers-reg__pers'),
					$pers_head = $pers.find('.head'),
					$pers_body = $pers.find('.body');
				$elem.parents('.pages').find('.pers-reg__part').removeClass('sel');
				$elem.addClass('sel');
				if(part == PART_HEAD) {
					$pers_head.find('img').remove();
					$pers_head.append($elem.find('img').clone(true).css({'margin':0,'vertical-align':'bottom'}));
					$pers_head.attr('data-dy', dy);
					$pers_reg.find('input[name="head"]').val(id);
				} else {
					$pers_body.find('img').remove();
					$pers_body.append($elem.find('img').clone(true).css({'margin':0,'vertical-align':'bottom'}));
					$pers_body.attr('data-dy', dy);
					$pers_reg.find('input[name="body"]').val(id);
				}
				if(sex == SEX_MALE) {
					if(part == PART_HEAD) $pers_reg.attr('data-sel-male-head', id);
					else $pers_reg.attr('data-sel-male-body', id);
				} else {
					if(part == PART_HEAD) $pers_reg.attr('data-sel-female-head', id);
					else $pers_reg.attr('data-sel-female-body', id);
				}
				fixPersRegTag($pers);
			});
		});
	});
	
	$chooser_header.append(
		$chooser_pers,
		'<div class="pers-reg__switch '+(sel_sex == SEX_MALE?'sel male':'male')+'" data-type="sex" data-sex="'+SEX_MALE+'"></div>',
		'<div class="pers-reg__switch '+(sel_sex == SEX_FEMALE?'sel female':'female')+'" data-type="sex" data-sex="'+SEX_FEMALE+'"></div>',
		'<div class="pers-reg__switch '+(sel_part == PART_HEAD?'sel head':'head')+'" data-type="part" data-part="'+PART_HEAD+'"></div>',
		'<div class="pers-reg__switch '+(sel_part == PART_BODY?'sel body':'body')+'" data-type="part" data-part="'+PART_BODY+'"></div>'
	);
	$previewer.append($previewer_pers);
	
	var pers_html = '<div class="body" data-dy="'+pers_body_dy+'">'+pers_body_html+'</div><div class="head" style="bottom:'+(pers_head_dy + pers_body_dy)+'px" data-dy="'+pers_head_dy+'">'+pers_head_html+'</div>';
	convertDOM('<body>'+pers_html+'</body>', $chooser_pers, null, function() {
		var $pers = $chooser_header.find('.pers-reg__pers').clone(true);
		$previewer.find('*').remove();
		$previewer.append($pers);
		fixPersRegTag($pers, true);
	});
	
	$chooser_header.find('.pers-reg__switch').each(function() {
		bindClick($(this), function(e, $elem) {
			e.preventDefault();
			var $pers_reg = $elem.parents('.pers-reg');
			$pers_reg.find('.pers-reg__parts .pages').hide();
			var cur_sex = parseInt($pers_reg.find('input[name="sex"]').val()),
				cur_part = parseInt($pers_reg.find('input[name="part"]').val()),
				new_type = $elem.attr('data-type'),
				new_sex = (new_type == 'sex' ? parseInt($elem.attr('data-sex')) : cur_sex),
				new_part = (new_type == 'part' ? parseInt($elem.attr('data-part')) : cur_part),
				sex_changed = (new_type == 'sex' && cur_sex != new_sex ? 1 : 0),
				$new_page = $pers_reg.find('.pages[data-sex="'+new_sex+'"][data-part="'+new_part+'"]');
			$new_page.show();
			$('.pers-reg__switch').removeClass('sel');
			$('.pers-reg__switch[data-type="sex"][data-sex="'+new_sex+'"]').addClass('sel');
			$('.pers-reg__switch[data-type="part"][data-part="'+new_part+'"]').addClass('sel');
			if(sex_changed) {
				var head_id = (new_sex == SEX_MALE ? $pers_reg.attr('data-sel-male-head') : $pers_reg.attr('data-sel-female-head')),
					body_id = (new_sex == SEX_MALE ? $pers_reg.attr('data-sel-male-body') : $pers_reg.attr('data-sel-female-body')),
					$elem_head = $('.pages[data-sex="'+new_sex+'"][data-part="'+PART_HEAD+'"] .pers-reg__part[data-part="'+PART_HEAD+'"][data-id="'+head_id+'"]'),
					$elem_body = $('.pages[data-sex="'+new_sex+'"][data-part="'+PART_BODY+'"] .pers-reg__part[data-part="'+PART_BODY+'"][data-id="'+body_id+'"]'),
					head_dy = parseInt($elem_head.attr('data-dy')),
					body_dy = parseInt($elem_body.attr('data-dy')),
					$pers_head = $pers_reg.find('.pers-reg__pers .head'),
					$pers_body = $pers_reg.find('.pers-reg__pers .body');
				$pers_head.find('img').remove();
				$pers_head.append($elem_head.find('img').clone(true).css({'margin':0,'vertical-align':'bottom'}));
				$pers_head.attr('data-dy', head_dy);
				$pers_reg.find('input[name="head"]').val(head_id);
				$pers_body.find('img').remove();
				$pers_body.append($elem_body.find('img').clone(true).css({'margin':0,'vertical-align':'bottom'}));
				$pers_body.attr('data-dy', body_dy);
				$pers_reg.find('input[name="body"]').val(body_id);
				$pers_head.css('bottom', head_dy+body_dy+'px');
			}
			$pers_reg.find('input[name="sex"]').val(new_sex);
			$pers_reg.find('input[name="part"]').val(new_part);
		});
	});
	
	bindClick($previewer, function(e, $elem) {
		$('.pers-reg').removeClass('minimized');
		$('[name="reg-container1"]').show();
		$('[name="reg-container2"]').hide();
		$(':focus').blur();
	});
	
}

function fixPersRegTag($pers_reg, recount_bottom) {
	
	var loaded = 2,
		$head = $pers_reg.find('.head'),
		head_h = 0,
		head_dy = parseInt($head.attr('data-dy')),
		$body = $pers_reg.find('.body'),
		body_h = 0,
		body_dy = parseInt($body.attr('data-dy'));
	
	$head.css('bottom', (head_dy + body_dy)+'px');
	
	function __do() {
		loaded = 2;
		head_h = $head.find('img').height();
		body_h = $body.find('img').height();
		if(!head_h) loaded--;
		if(!body_h) loaded--;
		if(loaded < 2) {
			$pers_reg.find('img').load(function() {
				__do();
			});
		} else {
			var pers_h = head_h + head_dy + body_dy;
			$pers_reg.height(pers_h);
			if(recount_bottom) {
				$pers_reg.css('bottom', (65 - pers_h)+'px');
			}
		}
	}
	
	__do();
	
}

function loadAjaxContent($elem, url) {
	
	if($elem.attr('data-loading') == 1) return;
	if(!url) return;
	var last_opacity = $elem.css('opacity'),
        last_cursor = $elem.css('cursor');
	$elem.css({
		'cursor': 'default'
	}).attr('data-loading', 1);
	setTimeout(function () {
		if ($elem.attr('data-loading') == 1) $elem.css({
			'opacity': 0.5
		});
	}, 700);
	var link = appendLink(url, true);
	link = getNextLinkHost(link.url, link.hosts);
	__do(link.url, link.hosts);
	
	function __do(url, hosts) {
		
		if(!url) return;
		$.ajax({
			url: url,
			dataType: 'json',
			headers: {
				'w': DEVICE_WIDTH,
				'h': DEVICE_HEIGHT,
				'KBversion': KBV
			},
			success: function(data) {
				try {
					$.each(data, function(k, v) {
						try {
							var $target = $('body [name="'+k+'"]');
							if($target && $target.length) {
								if($target.hasClass('span')) {
									$target = $target.find('.span__content2');
									if(!$target || !$target.length) return;
								}
								$target.html('');
								convertDOM('<body>'+v+'</body>', $target);
							}
						} catch(err) {
							
						}
					});
				} catch (err) {

				}
			},
			complete: function(data) {
				$elem.css({
					'opacity': last_opacity,
					'cursor': last_cursor
				}).attr('data-loading', 0);
			},
			error: function() {
				var new_link = getNextLinkHost(url, hosts);
				__do(new_link.url, new_link.hosts);
			}
		});
		
	}
	
}

function parseSpanTag($dom_elem, hlinks) {

	$dom_elem.addClass('span');
	var align = $dom_elem.attr('align'),
		bgcolor = $dom_elem.attr('bgcolor'),
		valign = $dom_elem.attr('valign');

	var $span_content1 = $('<div class="span__content1"></div>'),
		$span_content2 = $('<div class="span__content2"></div>');
	
	var padding = '0,0,0,0',
		padding_px;
	if (typeof $dom_elem.attr('padding') !== 'undefined') padding = $dom_elem.attr('padding');
	padding = padding.split(',');
	if (padding.length == 1) padding = [padding, padding, padding, padding];
	if (padding.length == 2) padding = [padding[0], padding[1], padding[0], padding[1]];
	if (padding.length == 3) padding = [padding[0], padding[1], padding[2], padding[1]];
	padding_px = padding.join('px ') + 'px';
	$span_content2.css('padding', padding_px);
	
	if(typeof $dom_elem.attr('margin') != 'undefined' && $dom_elem.attr('margin') != '') {
		$dom_elem.css('margin', getMarginPadding($dom_elem.attr('margin')));
	}

	if (valign == 'center') valign = 'middle';
	$span_content2.css('vertical-align', valign);

	if (align) $span_content2.css('text-align', align);

	w = $dom_elem.attr('w');
	if (w) {
		if (w.indexOf('%') == -1 && w.indexOf('px') == -1) w += 'px';
		$span_content1.css('width', w);
	}

	h = $dom_elem.attr('h');
	if (h) {
		if (h.indexOf('%') == -1 && h.indexOf('px') == -1) h += 'px';
		$dom_elem.css('height', h);
	}

	if (bgcolor) {
		bgcolor = '#' + fixColorZeroes(bgcolor);
		$dom_elem.css('background-color', bgcolor);
	}

	$dom_elem.wrapInner($span_content2);
	$dom_elem.wrapInner($span_content1);

}

function fixPlanksSizes() {
	$('.plank.has-many-blocks .plank-content[data-type="right"]').each(function () {
		var $plank = $(this).parents('.plank'),
            plank_h = getElementSizes($plank).height,
            plank_content_h = getElementSizes($(this)).height;
		if (plank_content_h > plank_h) $plank.height(parseInt(plank_content_h) + 5);
	});

}

function parsePlankTag($dom_elem, hlinks) {
	
	var $plank = $('<div class="plank"></div>'),
        HUYA = 'http://galaxy.mobstudio.ru/';

	hlinks = hlinks || lastHlinks;

	if (isHttps()) {
		HUYA = 'https://galaxy.mobstudio.ru/';
		// for (tmpLink in hlinks) {
		// 	tmpLink = tmpLink.replace('http://','https://');
		// }
	}
	
	$plank.attr('hint', $dom_elem.attr('hint'));
	$plank.attr('action', $dom_elem.attr('action'));
	$plank.attr('lst', $dom_elem.attr('lst'));
	$plank.attr('lst_action', $dom_elem.attr('lst_action'));
	$plank.attr('href', $dom_elem.attr('href'));
	$plank.attr('title', $dom_elem.attr('title'));
	$plank.attr('confirm', $dom_elem.attr('confirm'));

	if ($dom_elem.attr('border_bottom') > 0) $plank.addClass('border-bottom');

	var w = $dom_elem.attr('w');
	if(!w) w = '100%';
	if(w) {
		if (w.indexOf('%') == -1 && w.indexOf('px') == -1) w += 'px';
		$plank.css('width', w);
	}

	var padding = '10,10,10,10',
		padding_px;
	if(typeof $dom_elem.attr('padding') !== 'undefined') padding = $dom_elem.attr('padding');
	padding = padding.split(',');
	if(padding.length == 1) padding = [padding, padding, padding, padding];
	if(padding.length == 2) padding = [padding[0], padding[1], padding[0], padding[1]];
	if(padding.length == 3) padding = [padding[0], padding[1], padding[2], padding[1]];
	padding_px = padding.join('px ') + 'px';
	
	var txt_c = $dom_elem.attr('txt-c'),
		txt_l = $dom_elem.attr('txt-l'),
		txt_r = $dom_elem.attr('txt-r'),
		class_addons = '';

	var h = $dom_elem.attr('h');
	if(h) {
		if (h.indexOf('%') == -1 && h.indexOf('px') == -1) h += 'px';
		$plank.css('height', h);
	}
	
	if(txt_c) {
		if($.trim(txt_c).indexOf('<span') == 0) class_addons = 'fix-span';
		$plank.append('<div class="plank-content ' + class_addons + '" style="padding:' + padding_px + '" data-type="center"></div>');
	} else {
		if(txt_r) {
			if($.trim(txt_r).indexOf('<span') == 0) class_addons = 'fix-span';
			$plank.append('<div class="plank-content ' + class_addons + '" style="padding:' + padding_px + '" data-type="right"></div>');
		}
		if(txt_l) {
			if($.trim(txt_l).indexOf('<span') == 0) class_addons = 'fix-span';
			$plank.append('<div class="plank-content ' + class_addons + '" style="padding:' + padding_px + '" data-type="left"></div>');
		}
	}

	if(txt_c) {
		convertDOM('<body>' + txt_c + '</body>', $plank.find('.plank-content[data-type="center"]')); //, null, fixPlankContentSizes);
	} else {
		if(txt_r) convertDOM('<body>' + txt_r + '</body>', $plank.find('.plank-content[data-type="right"]')); //, null, fixPlankContentSizes);
		if(txt_l) convertDOM('<body>' + txt_l + '</body>', $plank.find('.plank-content[data-type="left"]')); //, null, fixPlankContentSizes);
		if(txt_r && txt_l) $plank.addClass('has-many-blocks');
	}
	
	var margin = '0,0,0,0',
		margin_px;
	if(typeof $dom_elem.attr('margin') !== 'undefined') margin = $dom_elem.attr('margin');
	margin = margin.split(',');
	if(margin.length == 1) margin = [margin, margin, margin, margin];
	if(margin.length == 2) margin = [margin[0], margin[1], margin[0], margin[1]];
	if(margin.length == 3) margin = [margin[0], margin[1], margin[2], margin[1]];
	margin_px = margin.join('px ') + 'px';
	$plank.css('margin', margin_px);
	
	var birka = $dom_elem.attr('birka');
	if(birka) {
		var $birka = $('<img class="plank-birka" src="'+imageSrc(birka, null, null, $birka)+'"/>');
		$birka.bind('load ready', function() {
			var $elem = $(this);
			if($elem.hasClass('ready')) return;
			$elem.addClass('ready').css({
				'width':parseInt($elem.width()/2),
				'height':parseInt($elem.height()/2),
				'margin-top':parseInt(-$elem.height()/4) + 'px'
			});
		});
		$plank.append($birka);
	}
	
	var arrow = $dom_elem.attr('arrow');
	if(arrow) {
		var $arrow = $('<div class="plank-arrow"></div>');
		$plank.append($arrow);
		if (arrow == -1) {
			//$plank.css('margin-left','8px');
			$arrow.addClass('left');
		}
		if (arrow == 1) {
			//$plank.css('margin-right','8px');
			$arrow.addClass('right');
		}
		// todo: below
		/*if(w.indexOf('%') != -1) w = Math.round(self.$browser.find('.content:first').width() * (parseInt(w) / 100)) - Math.abs(parseInt(self.$browser.find('.content').css('margin-right'))) - 4;
		 else w = parseInt(w);
		 w -= 8;
		 $plank.css('width',w+'px');*/
	}
	
	if(typeof $dom_elem.attr('bgcolor') && $dom_elem.attr('bgcolor') == -1) {
		$plank.addClass('no-bg');
	}
	
	if(typeof $dom_elem.attr('ww') !== 'undefined' && $dom_elem.attr('ww') == 1) {
		$plank.css({
			'display': 'block',
			'width': 'auto',
			'margin-left': '-10px',
			'margin-right': '-10px'
		});
	}
	
	return $plank;

}

function getRegParams() {
	var inputs, index;
	var pCount = 0;
	var returnStr = '';

	//return '555';
	inputs = document.getElementsByTagName('input');
	for (index = 0; index < inputs.length; ++index) {
		if (inputs[index].name == 'reg_id' ||
            inputs[index].name == 'reg_nick' ||
            inputs[index].name == 'reg_pass') {
			++pCount;
			returnStr += inputs[index].name + '=' + inputs[index].value + ' |';
		}
		if (pCount == 3)
			return returnStr;
		// deal with inputs[index] element.
	}

	return '';
}

function replaceFirstMeta(str, param, value) {
	try {
		var new_str = str,
            pos = str.indexOf('meta-' + param);
		if (pos >= 0) {
			str = str.substr(pos + ('meta-' + param).length);
			if (str[0] == '(') {
				var close_bracket_pos = str.indexOf(')');
				if (close_bracket_pos != -1) {
					var params_str = str.substr(1, close_bracket_pos - 1),
                        params = splitSplit(params_str, ',', '='),
                        p = '',
                        v = '';
					try {
						var params2 = params_str.split(',');
						for (var i = 0, ii = params2.length; i < ii; i++) {
							params2[i] = [params2[i].substr(0, params2[i].indexOf('=')), params2[i].substr(params2[i].indexOf('=') + 1)];
						}
						params = params2;
					} catch (err) {

					}
					if (params.length) {
						for (var i = 0, ii = params.length; i < ii; i++) {
							p = params[i][0];
							v = params[i][1];
							value = value.replace(new RegExp(p, 'g'), v);
							new_str = new_str.substr(0, pos) + value + str.substr(close_bracket_pos + 1);
						}
					} else {
						new_str = new_str.substr(0, pos) + value + str.substr(close_bracket_pos + 1);
					}
				}
			}
		}
		return new_str;
	} catch (err) {
		return str;
	}
}

function animImage(canvas, start_anim) {
	
	var self = this,
		ctx = canvas.get(0).getContext('2d'),
		src = canvas.attr('src') || canvas.attr('data-src'),
		files = (src ? src.split('~') : []),
		delays = canvas.attr('frames_delay').split(':'),
		imgs = [],
		frame = -1,
		max_frames = files.length,
		c_width = canvas.width(),
		c_height = canvas.height(),
		img_w = 0,
		img_h = 0;
	if(typeof canvas.attr('w') != 'undefined') img_w = canvas.attr('w');
	if(typeof canvas.attr('h') != 'undefined') img_h = canvas.attr('h');
	
	ctx.clearRect(0, 0, c_width, c_height);
	
	this.setImages = function() {
		try {
			var file = '';
			for(var j = 0; j < max_frames; j++) {
				file = files[j];
				var img = new Image();
				img.src = imageSrc(file, 0, 1, img);
				imgs.push(img);
			}
		} catch(err) {
			
		}
	}
	
	self.setImages();
	
	this.animate = function() {
		if(!canvas || !ctx) return;
		frame++;
		if(frame > max_frames - 1) frame = 0;
		if(imgs.length < frame + 1) {
			setTimeout(arguments.callee, 1000);
		}
		ctx.clearRect(0, 0, c_width, c_height);
		var img = imgs[frame];
		if(img) {
			var half_w;
			try {
				half_w = canvas.attr('w') / 2;
			} catch(e) {
				half_w = c_width / 2;
			}
			try {
				//ctx.drawImage(img, Math.floor(half_w - img.width / 2), 0, img.width, img.height);
				ctx.drawImage(img, Math.floor(half_w - (img_w || img.width) / 2), 0, (img_w || img.width), (img_h || img.height));
			} catch(e) {
				
			}
		}
		setTimeout(function() {
			self.animate();
		}, delays[frame]);
	}
	
}

function refreshTextareaHeight($textarea) {
	var elem = $textarea.get(0);
	if(elem.scrollTop > 0) {
		elem.style.height = (elem.scrollTop + elem.clientHeight + 40) + 'px';
	}
}

this.openAjaxContainer = function () {
	// todo
}

// ANIMATIONS ---------------------------------------------------------------------------------

var animElements = new (function () {
	var ae_self = this,
        elements = [],
        //timer = null,
        timers = [];
	//last_t = new Date().getTime();

	this.add = function ($elem, timeout) {
		ae_self.stop();
		elements.push([$elem, timeout]);
		ae_self.start();
	}

	function animateElement(elem, timeout) {
		try {
			if (!elem) return;
			if (typeof elem.draw == 'function') {
				timeout = timeout || 50;
				timers.push(setInterval(function () {
					elem.draw();
				}, timeout));
			} else {
				elem = $(elem);
				if (typeof elem.is == 'function') {
					if (elem.hasClass('super-crystal')) {
						timeout = timeout || 270;
						timers.push(setInterval(function () {
							var num = parseInt(elem.attr('data-pic'));
							if (++num > 3) num = 1;
							elem.attr('src', imageSrc('super' + num, 6)).attr('data-pic', num);
						}, timeout));
					} else if (elem.hasClass('scrollbox')) { // todo
						
					}
				}
			}
		} catch (err) {
			//console.log(err);
		}
	}
	
	this.start = function () {
		var elem = null;
		for (var i = 0, ii = elements.length; i < ii; i++) {
			animateElement(elements[i][0], elements[i][1]);
		}
	}

	this.stop = function () {
		for (var i = 0, ii = timers.length; i < ii; i++) {
			clearInterval(timers[i]);
		}
		timers = [];
	}

	this.remove = function () {
		ae_self.stop();
		for (var i = 0, ii = elements.length; i < ii; i++) {
			if (elements[i]) {
				delete elements[i];
				elements[i] = null;
			}
		}
		elements = [];
	}

});

function updatePagesTag($pages, refresh_dom) {
	
	var indic = $pages.attr('data-indic'),
		$indic = $pages.find('.indic');
	if(indic == 3) {
		var start = parseInt($pages.attr('data-start')) || 0,
			sel = parseInt($pages.attr('data-sel')) || 0,
			total = parseInt($pages.attr('data-total')) || 0,
			sep = parseInt($pages.attr('data-sep')) || 0;
		if(LOCAL.PAGES_SEP && !sep) sep = ' '+LOCAL.PAGES_SEP+' ';
		else sep = '/';
		$indic.find('.counter').text(sel + start + 1 + sep + total);
	} else {
		var sel = $pages.attr('data-sel') || 0;
		$pages.find('.indic').each(function () {
			var $indic = $(this);
			$indic.find('.dot').removeClass('sel');
			$indic.find('.dot:nth(' + sel + ')').addClass('sel');
		});
	}
	var cur = sel + start;
	if(sel + start + 1 == total) {
		$indic.find('.first').removeClass('disabled');
		$indic.find('.prev').removeClass('disabled');
		$indic.find('.next').addClass('disabled');
		$indic.find('.last').addClass('disabled');
	} else if(sel + start + 1 == 1) {
		$indic.find('.first').addClass('disabled');
		$indic.find('.prev').addClass('disabled');
		$indic.find('.next').removeClass('disabled');
		$indic.find('.last').removeClass('disabled');
	} else {
		$indic.find('.first').removeClass('disabled');
		$indic.find('.prev').removeClass('disabled');
		$indic.find('.next').removeClass('disabled');
		$indic.find('.last').removeClass('disabled');
	}
	$pages.find('.pers:visible').each(function () {
		refreshPersImgsPositions($(this));
	});
	if(refresh_dom) {
		var h = $pages.attr('data-h') || $pages.attr('h'),
			w = SYSTEM.width,
			$cur_page = $pages.find('.page-content:nth-child('+(sel+1)+')');
		if(h) {
			$pages.addClass('no-transition');
			$pages.find('.page-content').width(w);
			$pages.css('height', 'auto').addClass('with-h');
			$pages.find('.wrapper').height(h);
			$pages.find('.page-content').height(h);
			$cur_page.prevAll().css('left', -w+'px');
			$cur_page.css('left', 0);
			$cur_page.nextAll().css('left', w+'px');
			repaintCSSPageFast();
			$pages.removeClass('no-transition');
		} else {
			$cur_page.prevAll().hide();
			$cur_page.show();
			$cur_page.nextAll().hide();
		}
	}
	fixPlanksSizes();
	
}

function parsePagesTag($pages, predefined_sel) {
	
	var sel = parseInt($pages.attr('data-sel')) || 0;
	if(typeof predefined_sel != 'undefined' && predefined_sel != null) {
		sel = parseInt(predefined_sel) || 0;
	}
    var indic = parseInt($pages.attr('data-indic')) || 0,
        start = parseInt($pages.attr('data-start')) || 0,
        total = parseInt($pages.attr('data-total')) || 0,
        first = $pages.attr('data-first') || 0,
        prev = $pages.attr('data-prev') || 0,
        next = $pages.attr('data-next') || 0,
        last = $pages.attr('data-last') || 0,
        $wrapper = $pages.find('.wrapper'),
		h = $pages.attr('data-h') || $pages.attr('h'),
		w = SYSTEM.width,
		$cur_page = $pages.find('.page-content:nth-child('+(sel+1)+')');
	
	if((!SYSTEM.msPointerEvents() && !SYSTEM.onTouchEvents() && !SYSTEM.onMouseEvents()) || (SYSTEM.isWP()) || (SYSTEM.isAndroid())) {
		indic = 3;
		$pages.attr('data-indic', indic);
	}
	
	if(h) {
		$pages.find('.page-content').width(w);
		$pages.css('height', 'auto').addClass('with-h');
		$pages.find('.wrapper').height(h);
		$pages.find('.page-content').height(h);
		$cur_page.prevAll().css('left', -w+'px');
		$cur_page.css('left', 0);
		$cur_page.nextAll().css('left', w+'px');
	} else {
		$cur_page.prevAll().hide();
		$cur_page.show();
		$cur_page.nextAll().hide();
	}
	
	var title = $pages.attr('data-title'),
        icon = $pages.attr('data-icon'),
        iw = $pages.attr('data-iw'),
        ih = $pages.attr('data-ih');
	if(title) {
		var $title = $('<div class="title"></div>');
		$pages.prepend($title);
		if (icon && icon != 'undefined') title = '<img class="icon" src="' + icon + '" width="' + iw + '" height="' + ih + '"/>' + title;
		convertDOM('<body>' + title + '</body>', $title);
	}
	
	var indic_html = '<div class="indic" data-type="' + indic + '"></div>';
	switch(indic) {
		case 0:
		case 3:
			$pages.append($(indic_html));
			break;
		case 1:
			$wrapper.before($(indic_html));
			break;
		case 2:
			$wrapper.before($(indic_html));
			$pages.append($(indic_html));
			break;
	}
	if(total == 1) {
		$pages.find('.indic').hide();
		//$pages.attr('data-indic', '');
	}
	if(indic == 3) {
		$pages.find('.indic').html('<span class="counter"></span>');
		var $prev = $('<span class="arrow prev"></span>');
		$pages.find('.indic').prepend($prev);
		bindClick($prev, function (e, $elem) {
			e.preventDefault();
			if($elem.hasClass('disabled')) return;
			__changePage('prev');
		});
		var $first = $('<span class="arrow first"></span>');
		$pages.find('.indic').prepend($first);
		bindClick($first, function (e, $elem) {
			e.preventDefault();
			if($elem.hasClass('disabled')) return;
			__changePage('first');
		});
		var $next = $('<span class="arrow next"></span>');
		$pages.find('.indic').append($next);
		bindClick($next, function (e, $elem) {
			e.preventDefault();
			if($elem.hasClass('disabled')) return;
			__changePage('next');
		});
		var $last = $('<span class="arrow last"></span>');
		$pages.find('.indic').append($last);
		bindClick($last, function (e, $elem) {
			e.preventDefault();
			if($elem.hasClass('disabled')) return;
			__changePage('last');
		});
		if(!first && !last && total <= 5) {
			$first.hide();
			$last.hide();
		}
	} else {
		for (var j = 0; j < total; j++) {
			$pages.find('.indic').append('<div class="dot-wrapper"><span data-page="' + j + '" class="arrow dot"></span></div>');
		}
	}
	
	function __changePage(dir) {
		
		var sel = parseInt($pages.attr('data-sel')) || 0,
			start = parseInt($pages.attr('data-start')) || 0,
			total = parseInt($pages.attr('data-total')),
			$cur_page = $pages.find('.page-content:nth-child('+(sel+1)+')'),
			w = SYSTEM.width,
			url = null,
			h = $pages.attr('data-h') || $pages.attr('h');
		if(dir == 'next' && sel + start + 1 == total) return;
		if(dir == 'prev' && sel + start + 1 == 1) return;
		switch(dir) {
			case 'first':
				sel = 0;
				url = $pages.attr('data-first') || null;
				break;
			case 'prev':
				sel--;
				url = $pages.attr('data-prev') || null;
				break;
			case 'next':
				sel++;
				url = $pages.attr('data-next') || null;
				break;
			case 'last':
				sel = $pages.find('.page-content').length-1;
				url = $pages.attr('data-last') || null;
				break;
		}
		var $new_page = $pages.find('.page-content:nth-child('+(sel+1)+')');
		if((dir == 'first' || dir == 'last') && url) {
			navigate(getHrefForModalView(null, [{name:'href', value:url}, {name:'action', value:'post'}]));
		} else if($new_page && $new_page.length) {
			if(h) {
				$new_page.prevAll().css('left', -w+'px');
				$new_page.css('left', 0);
				$new_page.nextAll().css('left', w+'px');
			} else {
				$new_page.prevAll().hide();
				$new_page.show();
				$new_page.nextAll().hide();
			}
			$pages.attr('data-sel', sel);
			updatePagesTag($pages);
		} else {
			navigate(getHrefForModalView(null, [{name:'href', value:url}, {name:'action', value:'post'}]));
		}
		$pages.find('.pers').each(function() {
			refreshPersImgsPositions($(this));
		});
		
	}
	
	var firstTouchX = 0,
        firstTouchY = 0,
        lastTouchX = 0,
        lastTouchY = 0,
        allowSwipe = false;
	
	if(SYSTEM.msPointerEvents()) {
		$wrapper.get(0).addEventListener("MSPointerDown", touchStart, false);
		$wrapper.get(0).addEventListener("MSPointerMove", touchMove, false);
		$wrapper.get(0).addEventListener("MSPointerUp", touchEnd, false);
	} else if(SYSTEM.onTouchEvents()) {
		$wrapper.get(0).addEventListener("touchstart", touchStart, false);
		$wrapper.get(0).addEventListener("touchmove", touchMove, false);
		$wrapper.get(0).addEventListener("touchend", touchEnd, false);
	} else if(SYSTEM.onMouseEvents()) {
		$wrapper.get(0).addEventListener("mousedown", touchStart, false);
		$wrapper.get(0).addEventListener("mousemove", touchMove, false);
		$wrapper.get(0).addEventListener("mouseup", touchEnd, false);
	}
	
	function touchStart(e) {
		if(e.target) {
			if(e.originalEvent) e = e.originalEvent;
			if(e.touches && e.touches.length > 1) {
				allowSwipe = false;
				return;
			}
			if(e.touches && e.touches.length) {
				var touch = e.touches[0];
				firstTouchX = touch.clientX;	// - $wrapper.get(0).offsetLeft;
				firstTouchY = touch.clientY;	// - $wrapper.get(0).offsetTop;
				if(firstTouchX >= SWIPE_X_MENU_DELTA) allowSwipe = true;
			} else {
				firstTouchX = e.clientX;		// - $wrapper.get(0).offsetLeft;
				firstTouchY = e.clientY;		// - $wrapper.get(0).offsetTop;
				if(firstTouchX >= SWIPE_X_MENU_DELTA) allowSwipe = true;
			}
		} else {
			allowSwipe = false;
		}
	}
	
	function touchMove(e) {
		if(e.target && allowSwipe) {
			if(e.originalEvent) e = e.originalEvent;
			if(e.touches && e.touches.length > 1) {
				allowSwipe = false;
				return;
			}
			if(e.touches && e.touches.length) {
				var touch = e.touches[0];
				lastTouchX = touch.clientX;// - $wrapper.get(0).offsetLeft;
				lastTouchY = touch.clientY;// - $wrapper.get(0).offsetTop;
			} else {
				lastTouchX = e.clientX;// - $wrapper.get(0).offsetLeft;
				lastTouchY = e.clientY;// - $wrapper.get(0).offsetTop;
			}
			if(Math.abs(lastTouchY - firstTouchY) > PAGES_SWIPE_Y_POSSIBLE_DELTA) {
				allowSwipe = false;
				return;
			}
			if(allowSwipe && Math.abs(lastTouchX - firstTouchX) > PAGES_SWIPE_X_POSSIBLE_DELTA) {
				e.preventDefault();
				allowSwipe = false;
				if(firstTouchX < lastTouchX) __changePage('prev');
				else __changePage('next');
			}
		} else {
			allowSwipe = false;
		}
	}
	
	function touchEnd(e) {
		allowSwipe = false;
	}
	
	updatePagesTag($pages);
	
	setTimeout(function() {
		$pages.addClass('ready');
	}, 1);
	
}

function updateTabsTag($tabs, refresh_dom) {
	
	var sel = parseInt($tabs.attr('data-sel')) || 0,
		$tab = $tabs.find('.tabs__tab[data-index="' + sel + '"]'),
		tab_index = $tab.attr('data-index'),
		$tabs_header = $tabs.find('.tabs__header'),
		$tabs_header_wrapper = $tabs_header.find('.wrapper');
	$tabs_header_wrapper.find('.tabs__tab').removeClass('sel');
	$tab.addClass('sel');
	$tabs.find('.tabs__tab__content').hide();
	$tabs.find('.tabs__tab__content[data-index="' + tab_index + '"]').show();
	if(refresh_dom) {
		$tabs.addClass('no-transition');
	}
	if($tabs_header_wrapper.outerWidth() > $tabs_header.outerWidth()) scrollToTab($tabs, tab_index);
	if(refresh_dom) {
		repaintCSSPageFast();
		$tabs.removeClass('no-transition');
	}
	$tabs.find('.pers').each(function () {
		refreshPersImgsPositions($tab);
	});
	tabChanged();
	
}

function parseTabsTag($tabs, predefined_sel) {
	
	var sel = parseInt($tabs.attr('data-sel')) || 0;
	if(typeof predefined_sel != 'undefined' && predefined_sel != null) {
		sel = parseInt(predefined_sel) || 0;
	}
	if(!sel) sel = 0;
	var $tabs_header = $tabs.find('.tabs__header'),
		$tabs_header_wrapper = $tabs_header.find('.wrapper');
	var tab_i = 0;
	$tabs.find('.tabs__tab__content').each(function() {
		var $tab_content = $(this);
		$tab_content.attr('data-index', tab_i);
		$tab = $('<div class="tabs__tab" data-index="'+tab_i+'"></div>');
		var href = null;
		if(typeof $tab_content.attr('href') != 'undefined') {
			href = $tab_content.attr('href');
			href = getHrefForModalView(null, [{name:'href', value:href}, {name:'action', value:'post'}], lastHlinks);
			$tab.attr('data-href', href);
			$tab_content.removeAttr('href');
		}
		$tabs_header_wrapper.append($tab);
		if($tabs_header.outerHeight() < $tab.outerHeight()) $tabs_header.css('height', $tab.outerHeight() + 'px');
		var tab_title = $tab_content.attr('data-title');
		if(typeof $tab_content.attr('data-icon') != 'undefined' && $tab_content.attr('data-icon')) {
			var tab_icon = $tab_content.attr('data-icon'),
				tab_iw = $tab_content.attr('data-iw') || '',
				tab_ih = $tab_content.attr('data-ih') || '',
				tab_icon2 = $tab_content.attr('data-icon2') || tab_icon,
				tab_iw2 = $tab_content.attr('data-iw2') || tab_iw,
				tab_ih2 = $tab_content.attr('data-ih2') || tab_ih;
			tab_title = '<img class="icon unsel" src="' + tab_icon + '" w="' + tab_iw + '" h="' + tab_ih + '"/><img class="icon sel" src="' + tab_icon2 + '" w="' + tab_iw2 + '" h="' + tab_ih2 + '"/>';
			convertDOM('<body>' + tab_title + '</body>', $tab, null, function () {
				//$tab.css('max-width', parseInt($content.width() * 0.7) + 'px');
			});
		} else {
			$tab.text(tab_title);
		}
		if(href) {
			bindClickNavigate($tab, href);
		} else {
			bindClick($tab, function(e, $elem) {
				e.preventDefault();
				$(':focus').blur();
				if($elem.hasClass('sel')) return;
				var tab_index = $elem.attr('data-index');
				if(!tab_index) return;
				$elem.parents('.tabs').attr('data-sel', $elem.index());
				updateTabsTag($elem.parents('.tabs'));
			});
		}
		tab_i++;
	});
	if($tabs.hasClass('fixed')) $tabs_header.css('top', SYSTEM.paddingTop - 10 + 'px');
	$tabs.find('.tabs__tab__content[data-index="' + sel + '"]').show();
	$tabs.find('.tabs__tab[data-index="' + sel + '"]').addClass('sel');
	var tabs_count = $tabs.find('.tabs__tab').length;
	if(tabs_count > 1 && tabs_count <= 3) {
		var tab_w = Math.floor($tabs_header.width() / tabs_count) - 8;
		$tabs.find('.tabs__tab').width(tab_w); // 8 = paddingLeft + paddingRight
	} else {
		$tabs.addClass('many-tabs');
	}
	scrollToTab($tabs, sel);
	
}

function switchClicked($elem) {
	
	if($elem.hasClass('disabled')) return;
	var value1 = $elem.attr('data-value1'),
		value2 = $elem.attr('data-value2'),
		checked_value = $elem.find(':hidden').val();

	var httpStr = isHttps()? 'https' : 'http'; 

	if(checked_value == value1) {
		$elem.addClass('checked').find(':hidden').val(value2);
		var url2 = appendLink(hlinksReplace(value2, lastHlinks)).url;
		if (isHttps()) url2 = url2.replace('http://','https://');
		if(url2.indexOf(httpStr) != -1) {
			if(SYSTEM.isAndroid() || SYSTEM.isWP()) {
				navigate('js: ajaxRequest ' + url2);
			} else {
				$.ajax({
					type: 'GET',
					url: url2,
					headers: {
						'w': DEVICE_WIDTH,
						'h': DEVICE_HEIGHT,
						'KBversion': KBV
					}
				});
			}
		}
	} else if(checked_value == value2) {
		$elem.removeClass('checked').find(':hidden').val(value1);
		var url1 = appendLink(hlinksReplace(value1, lastHlinks)).url;
		if (isHttps()) url1 = url1.replace('http://','https://');
		if(url1.indexOf(httpStr) != -1) {
			if(SYSTEM.isAndroid() || SYSTEM.isWP()) {
				navigate('js: ajaxRequest ' + url1);
			} else {
				$.ajax({
					type: 'GET',
					url: url1,
					headers: {
						'w': DEVICE_WIDTH,
						'h': DEVICE_HEIGHT,
						'KBversion': KBV
					}
				});
			}
		}
	}
	
}

// ------------------------------------------------------------------------------

var msgBoxArray = [],
	noteArray = [],
	lastHlinks = [],
	lastMeta = {};

function hlinksReplace(hr, hlinks) {
	if(hr) {
		for(var j = 0, jj = hlinks.length; j < jj; j++) {
			//hr = hr.replace('#' + hlinks[j][0] + '#', hlinks[j][1]);
			hr = hr.replace(new RegExp('#' + hlinks[j][0] + '#','gm'), hlinks[j][1]);
		}
	}
	if (isHttps()) hr = hr.replace('http://', 'https://');
	return hr;
}

function convertDOM(dom, $parent, hlinks, cb, scrollTo, meta) {
	
	var orig_html = '',
		check_meta = true;
	
	//bodyLog('test');

	if(typeof dom != 'object') {
		orig_html = dom;
		//if(orig_html.indexOf('meta') != -1) check_meta = false;
		dom = $(dom);
		if(dom.length == 0 && orig_html.length > 0) {
			$parent.append(orig_html);
		}
	}
	
	lastHlinks = hlinks || lastHlinks || [];
	hlinks = lastHlinks;
	
	if(meta) lastMeta = meta;
	meta = meta || lastMeta || {};
	
	var allow_append = false,
		$dom_elem = null,
		$temp_elem = null,
		w = null,
		h = null,
		allow_smiles = false,
		size = null;
	
	//if(check_meta) {
	try {
		var attrs = '',
			meta_name = '',
			_attr = '',
			_value = '',
			_params = {};
		for(var i = 0, ii = dom.length; i < ii; i++) {
			attrs = dom[i].attributes;
			$dom_elem = $(dom[i]);
			switch (dom[i].nodeName) {
				case 'META':
				case 'META2':
					for(var k = 0, kk = attrs.length; k < kk; k++) {
						_attr = attrs[k].nodeName;
						_value = attrs[k].nodeValue;
						if(_attr == 'name') meta_name = _value;
						else _params[_attr] = _value;
					}
					if(meta_name) meta[$dom_elem.attr('name')] = _params;
					break;
			}
		}
		lastMeta = meta;
		if(!$.isEmptyObject(meta)) {	
			var _meta = '',
				_html = '';
			for(var i = 0, ii = dom.length; i < ii; i++) {
				$dom_elem = $(dom[i]);
				_meta = $dom_elem.attr('meta');
				_html = $dom_elem.html();
				attrs = dom[i].attributes;
				if(_meta && meta[_meta]) {
					for (var k = 0, kk = attrs.length; k < kk; k++) {
						_attr = attrs[k].nodeName;
						_value = attrs[k].nodeValue;
						if(_attr != 'name') {
							$.each(meta[_meta], function(param, value) {
								while (_value.indexOf('meta-' + param + '(') != -1) {
									_value = replaceFirstMeta(_value, param, value);
								}
							});
							$dom_elem.attr(_attr, _value);
						}
					}
					$.each(meta[_meta], function(param, value) {
						while(_html.indexOf('meta-' + param + '(') != -1) {
							_html = replaceFirstMeta(_html, param, value);
						}
					});
					_html = _html.replace(/\\"/g, '"');
					$dom_elem.html(_html).removeAttr('meta');
				}
			}
		}
	} catch(err) {
		
	}
	//}
	
	for(var i = 0, ii = dom.length; i < ii; i++) {
		
		allow_append = true;
		$dom_elem = $(dom[i]);
		
		if(SYSTEM.isWP()) {
			if($dom_elem.children().length > 0) {
				var tt = $dom_elem.html();
				$dom_elem.html('');
				convertDOM(tt, $dom_elem, hlinks, null, null, meta);
			} else {
			    $dom_elem.html(replaceSmilesWP($dom_elem.html()));
			}
		} else {
			var $contents = $dom_elem.contents();
			if($contents && $contents.length > 0) {
				var $contents = $dom_elem.contents();
				$dom_elem.html('');
				convertDOM($contents, $dom_elem, hlinks, null, null, meta);
			}
		}
		
		if (typeof $dom_elem.attr('font') != 'undefined') {
			$dom_elem.addClass('font' + $dom_elem.attr('font')).removeAttr('font');
		}
		
		w = null;
		if(typeof $dom_elem.attr('w') != 'undefined') {
			w = $dom_elem.attr('w');
			if(w[w.length - 1] == '%' || w > 0) {
				if(w[w.length - 1] == '%') {
					w = parseInt(w.substr(0, w.length - 1));
					w += '%';
				}
				$dom_elem.css('width', w);
			}
		}
		
		h = null;
		if(typeof $dom_elem.attr('h') != 'undefined') {
			h = $dom_elem.attr('h');
		}
		
		try {
			switch(dom[i].nodeName) {
				case 'BODY':
					
					allow_append = false;
					break;
					
				case 'IFRAME':
					
					$dom_elem.attr('src', $dom_elem.attr('src') || $dom_elem.attr('data-src'));
					$dom_elem.css('height', $dom_elem.attr('h') + 'px');
					break;
					
				case 'HLINK':
					
					var arr = [];
					arr[0] = $dom_elem.attr('name');
					arr[1] = $dom_elem.attr('value');
					if (isHttps()) {
						arr[1] = arr[1].replace('http://', 'https://');
					}
					hlinks.push(arr);
					allow_append = false;
					break;
					
				case 'META':
					
					allow_append = false;
					break;
					
				case 'SCROLLBOX': // todo

					var div = document.createElement('div');
					var div2 = document.createElement('div');
					if ($dom_elem.children().length > 0)
						$(div2).append($dom_elem.children());
					else {
						$(div2).html('<span style="display:inline;">' + $dom_elem.html() + '</span>');
					}
					if (w == null) w = '95%';
					$(div).css({
						'width': w,
						'display': 'inline-block',
						'overflow': 'hidden'
					});
					$(div2).css({
						'width': '5000px',
						'position': 'relative',
						'left': '0px'
					});
					$(div).append(div2);
					$(div)
						.addClass('scrollbox')
						.addClass('animated')
						.attr('data-anim', (new Date()).getTime())
						.attr('data-dir', -10);
					dom[i] = div;
					animElements.add($(div));
					break;
					
				case 'SCROLLAREA':
					
					$temp_elem = $('<div class="scrollarea"></div>');
					$dom_elem.clone(true).appendTo($temp_elem);
					$temp_elem.find('img').each(function () {
						$(this).wrap('<div class="item"></div>');
					});
					$temp_elem.wrapInner('<div class="scrollarea-content"></div>');
					if ($dom_elem.attr('href')) {
						var $a = $('<a class="item" data-href="' + hlinksReplace($dom_elem.attr('href'), hlinks) + '"></a>');
						var serviceIconSrc = 'http://galaxy.mobstudio.ru/web_dev/img/go_service_icon.png';
						if (isHttps()) serviceIconSrc = 'https://galaxy.mobstudio.ru/web_dev/img/go_service_icon.png';
						$a.append('<img src="' + serviceIconSrc + '" width="70" height="70">');
						bindClick($a, function (e, $elem) {
							showPage($elem.attr('data-href'));
						});
						$temp_elem.find('scrollarea').append($a);
					}
					var orient = $temp_elem.attr('orient');
					if (orient = 'h') {
						$temp_elem.addClass('scrollarea-horizontal');
						var w = 0,
							h = 0;
						$temp_elem.find('img').each(function () {
							w += $(this).width() + 10;
							if ($(this).outerHeight() > h) h = $(this).outerHeight();
						});
						$temp_elem.find('.scrollarea-content').css({
							'width': w + 'px'
						});
					}
					var $btn_right = $('<div class="scrollarea-btn" data-action="right"></div>');
					bindClick($btn_right, function (e, $elem) {
						var $parent = $elem.parents('.scrollarea'),
							$content = $parent.find('.scrollarea-content'),
							delta = Math.floor($content.outerWidth() / $parent.outerWidth() * 100),
							new_m = parseInt($content.css('margin-left')) - delta;
						if ($parent.outerWidth() + Math.abs(new_m) - $content.outerWidth() > 0) {
							new_m = $parent.outerWidth() - $content.outerWidth();
							$elem.hide();
						}
						$content.transition({
							'margin-left': new_m
						});
						$parent.find('.scrollarea-btn[data-action="left"]').fadeIn('fast');
					});
					$temp_elem.append($btn_right);
					var $btn_left = $('<div class="scrollarea-btn" data-action="left" style="display:none"></div>');
					bindClick($btn_left, function (e, $elem) {
						var $parent = $elem.parents('.scrollarea'),
							$content = $parent.find('.scrollarea-content'),
							delta = Math.floor($content.outerWidth() / $parent.outerWidth() * 100),
							new_m = parseInt($content.css('margin-left')) + delta;
						if (new_m > 0) {
							new_m = 0;
							$elem.hide();
						}
						$content.transition({
							'margin-left': new_m
						});
						$parent.find('.scrollarea-btn[data-action="right"]').fadeIn('fast');
					});
					$temp_elem.append($btn_left);
					$dom_elem = $temp_elem;
					break;
					
				case 'SPACE':

					$temp_elem = $('<div class="space"></div>');
					$temp_elem.css('display', 'inline-block');
					if (h != null && h != 0) {
						if (h.indexOf('%') != -1) {
							var fontSize = $parent.css('font-size'),
								lineHeight = Math.floor(parseInt(fontSize.replace('px', '')));
							var perc = parseInt(parseInt(h) / 100 * lineHeight);
							h = perc;
						}
						if (dom[i - 1] && dom[i - 1].nodeName == 'BR') $temp_elem.css('display', 'block');
						else $temp_elem.addClass('need-br-after');
					}
					if (!h) h = 1;
					if (w == null || w == 0) w = '1';
					if (w.indexOf('%') != -1) w = parseInt($parent.width() * parseInt(w) / 100);
					$temp_elem.css('width', w + 'px');
					$temp_elem.css('height', h + 'px');
					$dom_elem = $temp_elem;
					break;

				case 'MSGBOX':

					var time = (typeof $dom_elem.attr('time') != 'undefined' ? parseInt($dom_elem.attr('time')) : 0);
					msgBoxArray.push({
						'type': 'alert',
						'title': (typeof $dom_elem.attr('title') != 'undefined' ? $dom_elem.attr('title') : ''),
						'text': hlinksReplace($dom_elem.attr('text'), hlinks),
						'time':time
					});
					break;
					
				case 'NOTE':
					
					noteArray.push({
						'params': $dom_elem.attr('params')
					});
					break;
					
				case 'PLANK':
					
					$dom_elem = parsePlankTag($dom_elem, hlinks);
					$dom_elem.removeAttr('txt-l').removeAttr('txt-r').removeAttr('txt-c');
					dom[i].removeAttribute('txt-l');
					dom[i].removeAttribute('txt-r');
					dom[i].removeAttribute('txt-c');

				case 'DIV':
					
					if(typeof $dom_elem.attr('margin') != 'undefined' && $dom_elem.attr('margin') != '') {
						$dom_elem.css('margin', getMarginPadding($dom_elem.attr('margin')));
					}
					if(typeof $dom_elem.attr('padding') != 'undefined' && $dom_elem.attr('padding') != '') {
						$dom_elem.css('padding', getMarginPadding($dom_elem.attr('padding')));
					}
					if(typeof $dom_elem.attr('align') != 'undefined') {
						$dom_elem.css('text-align', $dom_elem.attr('align'));
					}
					if(typeof $dom_elem.attr('animation-chain') != 'undefined') {
						$dom_elem.addClass('hasAnimationChain');
					}
					
				case 'A':
					
					var action = $dom_elem.attr('action'),
						href = (action == 'anchor' ? $dom_elem.attr('href') : getHrefForModalView(dom[i], null, hlinks));
					
					$dom_elem.attr('data-href', href).removeAttr('href');
					var disabled = false;
					if(typeof $dom_elem.attr('disabled') != 'undefined' && (parseInt($dom_elem.attr('disabled')) == 1 || $dom_elem.attr('disabled') == 'disabled')) {
						disabled = true;
						$dom_elem.attr('disabled', 'disabled').addClass('disabled');
					}
					if(typeof $dom_elem.attr('margin') != 'undefined' && $dom_elem.attr('margin') != '') {
						$dom_elem.css('margin', getMarginPadding($dom_elem.attr('margin')));
					}
					if(parseInt($dom_elem.attr('button')) == 1) {
						$dom_elem.addClass('button').removeAttr('button').wrapInner('<div class="button__wrapper"></div>');
						if(!$.trim($dom_elem.text()).length) $dom_elem.css('vertical-align', '-13px');
						var decor = $dom_elem.attr('decor');
						if(!decor) decor = '0 0';
						decor = decor.split(/\s+/);
						if(decor.length == 1) decor.unshift(0);
						if(decor[0] == 0) $dom_elem.addClass('button-raised');
						else $dom_elem.addClass('button-flat');
						if(!disabled) $dom_elem.addClass('button-decor'+decor[1]);
						if(typeof $dom_elem.attr('case') != 'undefined') {
							var text_case = $dom_elem.attr('case');
							if(text_case == 'lower') $dom_elem.css('text-transform', 'lowercase');
							if(text_case == 'upper') $dom_elem.css('text-transform', 'uppercase');
							if(text_case == 'none') $dom_elem.css('text-transform', 'none');
						}
					} else {
						if($dom_elem.attr('lst_action')) $dom_elem.addClass('a-menu');
						if($dom_elem.attr('hint')) $dom_elem.addClass('a-hint');
					}
					if(dom[i].nodeName == 'A') {
						if($dom_elem.attr('image') || $dom_elem.attr('image_anim')) {
							var img = document.createElement('img');
							if($dom_elem.attr('image_anim')) {
								var src = $dom_elem.attr('src') || $dom_elem.attr('data-src'),
									anim = $dom_elem.attr('image_anim');
								if(anim) {
									var canv = document.createElement('canvas'),
										attrs = $dom_elem.prop("attributes");
									$.each(attrs, function () {
										$(canv).attr(this.name, this.value);
									});
									$dom_elem = $(canv);
								}
								if($dom_elem[0].nodeName != 'DIV') {
									$dom_elem.css({ 'position': 'relative' });
								}
								if(typeof $dom_elem.attr('h') != 'undefined') {
									$dom_elem.css('height', $dom_elem.attr('h') + 'px')
										.attr('height', $dom_elem.attr('h'))
										.removeAttr('h');
								}
								if(typeof $dom_elem.attr('store') != 'undefined') {
									$dom_elem.removeAttr('store');
								}
								if(anim) {
									var ia = new animImage($dom_elem, 1);
								}
								if(typeof $dom_elem.attr('hint') != 'undefined') {
									$dom_elem.addClass('hint').attr('data-hint', $dom_elem.attr('hint')).removeAttr('hint');
								}
								$dom_elem.css('width', $dom_elem.attr('w') + 'px');
								$dom_elem.attr('width', $dom_elem.attr('w'));
								if(anim) {
									ia.animate();
								}
							} else {
								$dom_elem.removeAttr('image').removeAttr('store').removeAttr('image_anim').removeAttr('store');
								img.src = $dom_elem.attr('src') || $dom_elem.attr('data-src');
								$dom_elem.removeAttr('src');
								if($dom_elem.attr('h')) $(img).css('height', $dom_elem.attr('h') + 'px');
								$dom_elem.addClass('no-border').append(img);
							}
						}
						$dom_elem.html(replaceSmiles($dom_elem.html(), 1, 1));
					}
					switch(action) {
						case 'anchor':
							$dom_elem.addClass('a-menu');
							bindClick($dom_elem, function(e, $elem) {
								e.preventDefault();
								var anchor = $elem.attr('data-href'),
									$anchor = $BROWSER.find('#anchor_' + anchor),
									offset = $anchor.position().top - 20;
								if(offset < 0) offset = 0;
								window.scrollTo(0, offset);
							});
							break;
						case 'register-next':
							bindClick($dom_elem, function(e, $elem) {
								$BROWSER.find('.pers-reg').addClass('minimized');
								$BROWSER.find('[name="reg-container1"]').hide();
								$BROWSER.find('[name="reg-container2"]').show();
								var $pers = $('.browser .pers-reg__chooser .pers-reg__pers').clone(true),
									$previewer = $('.browser .pers-reg__previewer');
								$previewer.find('*').remove();
								$previewer.append($pers);
								fixPersRegTag($pers, true);
								$(':focus').blur();
							});
							break;
						case 'extlink':
							$dom_elem.addClass('extlink');
						default:
							if(href) {
								if(action == 'ajax') {
									bindClick($dom_elem, function(e, $elem) {
										e.preventDefault();
										e.stopPropagation();
										loadAjaxContent($elem, $elem.attr('data-href'));
									});
								} else {
									bindClickNavigate($dom_elem, href);
								}
							}
							break;
					}
					break;
					
				case 'ANCHOR':

					$temp_elem = $('<a class="anchor" id="anchor_' + $dom_elem.attr('name') + '" name="' + $dom_elem.attr('name') + '"></a>');
					$dom_elem = $temp_elem;
					break;
					
				case 'SWITCH':
					
					var $temp_elem = $('<div class="switch"></div>'),
						checked = parseInt($dom_elem.attr('sel')) || 0,
						value1 = $dom_elem.attr('value1') || 0,
						value2 = $dom_elem.attr('value2') || 1;
					if(checked) $temp_elem.addClass('checked');
					$temp_elem.append('<div class="switch__thumb"></div>');
					if(typeof $dom_elem.attr('disabled') != 'undefined' && (parseInt($dom_elem.attr('disabled')) == 1 || $dom_elem.attr('disabled') == 'disabled')) {
						$temp_elem.addClass('disabled');
					}
					$temp_elem.attr('data-value1', value1).attr('data-value2', value2);
					$temp_elem.append('<input type="hidden" name="'+$dom_elem.attr('name')+'" value="'+(checked ? value2 : value1)+'" />');
					copyStyleClass($dom_elem, $temp_elem);
					if(typeof $dom_elem.attr('margin') != 'undefined' && $dom_elem.attr('margin') != '') {
						$temp_elem.css('margin', getMarginPadding($dom_elem.attr('margin')));
					}
					if(typeof $dom_elem.attr('id') != 'undefined') {
						$temp_elem.attr('id', $dom_elem.attr('id'));
					}
					$dom_elem = $temp_elem;
					bindClick($dom_elem, function(e, $elem) {
						e.preventDefault();
						e.stopPropagation();
						switchClicked($elem);
					});
					break;

				case 'PAGES':
					
					hasPagesTag = true;
					var $temp_elem = $('<div class="pages" data-sel-index="'+pagesTagSelIndex+'"></div>');
					copyAttributes($dom_elem[0], $temp_elem[0]);
					$temp_elem.attr('style', $dom_elem.attr('style'));
					if(typeof $dom_elem.attr('sep') != 'undefined' && $dom_elem.attr('sep')) $temp_elem.attr('data-sep', $dom_elem.attr('sep'));
					$temp_elem.attr('data-indic', $dom_elem.attr('indic') ? $dom_elem.attr('indic') : '');
					$temp_elem.attr('data-first', $dom_elem.attr('first') ? $dom_elem.attr('first') : '');
					$temp_elem.attr('data-prev', $dom_elem.attr('prev') ? $dom_elem.attr('prev') : '');
					$temp_elem.attr('data-start', $dom_elem.attr('start') ? $dom_elem.attr('start') : '');
					$temp_elem.attr('data-total', $dom_elem.attr('total') ? $dom_elem.attr('total') : '');
					$temp_elem.attr('data-next', $dom_elem.attr('next') ? $dom_elem.attr('next') : '');
					$temp_elem.attr('data-last', $dom_elem.attr('last') ? $dom_elem.attr('last') : '');
					$temp_elem.attr('data-sel', $dom_elem.attr('sel') ? $dom_elem.attr('sel') : '');
					var predefined_sel = getPredefinedInputValue('sel__pages'+pagesTagSelIndex);
					if(predefined_sel != null) $temp_elem.attr('data-sel', predefined_sel);
					$temp_elem.attr('data-title', $dom_elem.attr('title') ? $dom_elem.attr('title') : '');
					$temp_elem.attr('data-icon', $dom_elem.attr('icon') ? $dom_elem.attr('icon') : '');
					$temp_elem.attr('data-h', $dom_elem.attr('h') ? $dom_elem.attr('h') : '');
					$temp_elem.html('<div class="wrapper"></div>');
					$temp_elem.find('.wrapper').append($dom_elem.contents());
					$dom_elem = $temp_elem;
					pagesTagSelIndex++;
					break;

				case 'PAGE':

					$temp_elem = $('<div class="page-content"></div>');
					$temp_elem.append($dom_elem.contents());
					$dom_elem = $temp_elem;
					break;

				case 'TABS':
					
					hasTabsTag = true;
					var fixed_class = '';
					if(typeof $dom_elem.attr('fixed') != 'undefined' && parseInt($dom_elem.attr('fixed'))) fixed_class = 'fixed';
					$temp_elem = $('<div class="tabs '+fixed_class+'" data-sel-index="'+tabsTagSelIndex+'"></div>');
					if($dom_elem.attr('sel')) $temp_elem.attr('data-sel', $dom_elem.attr('sel'));
					var predefined_sel = getPredefinedInputValue('sel__tabs'+tabsTagSelIndex);
					if(predefined_sel != null) $temp_elem.attr('data-sel', predefined_sel);
					if($dom_elem.attr('padding')) $temp_elem.attr('data-padding', $dom_elem.attr('data-padding').split(';').join('px ') + 'px');
					$temp_elem.html('<div class="tabs__header"><div class="wrapper"></div></div></div>');
					$temp_elem.append($dom_elem.contents());
					$dom_elem = $temp_elem;
					tabsTagSelIndex++;
					break;
					
				case 'TAB':
					
					$temp_elem = $('<div class="tabs__tab__content" style="display:none"></div>');
					if($dom_elem.attr('href')) $temp_elem.attr('href', $dom_elem.attr('href'));
					if($dom_elem.attr('icon')) $temp_elem.attr('data-icon', $dom_elem.attr('icon'));
					if($dom_elem.attr('iw')) $temp_elem.attr('data-iw', $dom_elem.attr('iw'));
					if($dom_elem.attr('ih')) $temp_elem.attr('data-ih', $dom_elem.attr('ih'));
					if($dom_elem.attr('icon2')) $temp_elem.attr('data-icon2', $dom_elem.attr('icon2'));
					if($dom_elem.attr('iw2')) $temp_elem.attr('data-iw2', $dom_elem.attr('iw2'));
					if($dom_elem.attr('ih2')) $temp_elem.attr('data-ih2', $dom_elem.attr('ih2'));
					if($dom_elem.attr('title')) $temp_elem.attr('data-title', $dom_elem.attr('title'));
					$temp_elem.append($dom_elem.contents());
					$dom_elem = $temp_elem;
					break;
					
				case 'CENTER':

					$temp_elem = $('<div style="text-align:center;"></div>');
					copyAttributes($dom_elem[0], $temp_elem[0]);
					$temp_elem.append($dom_elem.contents());
					$dom_elem = $temp_elem;
					break;

				case 'SPAN':

					parseSpanTag($dom_elem, hlinks);
					break;
				
				case 'HR':

					var color = $dom_elem.attr('color');
					if (color) {
						if (color.indexOf('#') == -1) color = '#' + color;
						$dom_elem.css({
							'border-color': color,
							'opacity': 1
						});
					}
					break;
				case 'TABLE':
				case 'TD':
				case 'TR':
					
					if (h) {
						if (h.indexOf('%') == -1 && h.indexOf('px') == -1) h += 'px';
						$dom_elem.css('height', h);
					}
					var align = $dom_elem.attr('align') || $dom_elem.attr('talign');
					if (align) $dom_elem.addClass('align' + align);
					var valign = $dom_elem.attr('valign');
					if (valign) $dom_elem.addClass('valign' + valign);
					if (parseInt($dom_elem.attr('nowrap')) == 1) $dom_elem.addClass('nowrap');
					if (typeof $dom_elem.attr('padding') != 'undefined' && $dom_elem.attr('padding') != '') {
						$dom_elem.css('padding', getMarginPadding($dom_elem.attr('padding')));
						if ($dom_elem[0] && ($dom_elem[0].tagName == 'TABLE' || $dom_elem[0].tagName == 'TR')) $dom_elem.find('td').css('padding', getMarginPadding($dom_elem.attr('padding')));
					}
					if (typeof $dom_elem.attr('padding-x') != 'undefined' && $dom_elem.attr('padding-x') != '')
						$('td', $dom_elem).css({
							'padding-right': $dom_elem.attr('padding-x') + 'px',
							'padding-left': $dom_elem.attr('padding-x') + 'px'
						}).removeAttr('padding-x');
					if (typeof $dom_elem.attr('padding-y') != 'undefined' && $dom_elem.attr('padding-y') != '')
						$('td', $dom_elem).css({
							'padding-top': $dom_elem.attr('padding-y') + 'px',
							'padding-bottom': $dom_elem.attr('padding-y') + 'px'
						}).removeAttr('padding-y');
					if (typeof $dom_elem.attr('margin') != 'undefined' && $dom_elem.attr('margin') != '') $dom_elem.css('margin', getMarginPadding($dom_elem.attr('margin')));
					if ($dom_elem.attr('bgcolor')) $dom_elem.css('background-color', $dom_elem.attr('bgcolor')).removeAttr('bgcolor');
					if ($dom_elem.attr('bordercolor')) $('td', $dom_elem).css('border-color', $dom_elem.attr('bordercolor')).removeAttr('bordercolor');
					if ($dom_elem.attr('border')) $($dom_elem).addClass('border' + $dom_elem.attr('border'));
					if ($dom_elem.attr('col_bgcolor')) {
						var colors = $dom_elem.attr('col_bgcolor').split(':'),
							$elems = $('td', $dom_elem);
						for (var i = 0, ii = $elems.lenght; i < ii; i++) $($elems[i]).css('background-color', colors[i]);
					}
					if ($dom_elem[0].nodeName == 'TD' && $dom_elem.html()[0] == ' ') $dom_elem.prepend('&nbsp;');
					break;
					
				case 'P':

					if ($dom_elem.attr('valign')) $dom_elem.addClass('valign' + $dom_elem.attr('valign'));
					if ($dom_elem.children().length > 0) break;
					$dom_elem.html(replaceSmiles($dom_elem.html(), 1));
					break;
					
				case '#text':
					
					var txt = $dom_elem.text();
					if (txt == '') break;
				    if (SYSTEM.isWP()) {
				        $dom_elem = replaceSmilesWP(txt);
				    } else {
				        $dom_elem = replaceSmiles(txt, 1);
				    }
				    $dom_elem = $dom_elem.replace(/&op;/g, '#');
					break;
					
				case 'REFRESH':

					var time = Math.floor($dom_elem.attr('time'));
					if ($dom_elem.attr('show') == '1') $temp_elem = $('<div class="countdown-timer"></div>');
					if ($dom_elem.attr('onback') == '1') isOnBack = 1;
					pageAutorefreshParams = [
						time,
						hlinksReplace($dom_elem.attr('url'), hlinks),
						$dom_elem.attr('button'),
						$dom_elem.attr('action'),
						$dom_elem.attr('cmd'),
						$temp_elem
					];
					$dom_elem = $temp_elem;
					break;

				case 'SPOILER':

					$temp_elem = $('<div class="spoiler"></div>');
					var $spoiler_header = $('<div class="spoiler-header"></div>'),
						$spoiler_content = $('<div class="spoiler-content" style="display:none"></div>'),
						collapse = parseInt($dom_elem.attr('collapse')),
						img_src1 = $dom_elem.attr('img_src1'),
						img_src2 = $dom_elem.attr('img_src2'),
						img_w = $dom_elem.attr('img_w'),
						img_h = $dom_elem.attr('img_h');
					$temp_elem.append($spoiler_header);
					$temp_elem.append($spoiler_content);
					$spoiler_header.html(replaceSmiles($dom_elem.attr('title'), 1));
					if (img_src1) $spoiler_header.prepend('<img class="icon open" style="display:none" src="' + img_src1 + '" width="' + img_w + '" height="' + img_h + '"/>');
					if (img_src2) $spoiler_header.prepend('<img class="icon close" style="display:none" src="' + img_src2 + '" width="' + img_w + '" height="' + img_h + '"/>');
					$spoiler_content.append($dom_elem.contents());
					if (collapse == 0) {
						$spoiler_content.show();
						$spoiler_header.find('.icon').hide();
						$spoiler_header.find('.icon.open').show();
					} else {
						$spoiler_header.find('.icon').hide();
						$spoiler_header.find('.icon.close').show();
					}
					$spoiler_header.bind('click', function () {
						var $spoiler_content = $(this).parent().find('.spoiler-content');
						if ($spoiler_content.is(':visible')) {
							$spoiler_content.slideUp('fast');
							$(this).find('.icon').hide();
							$(this).find('.icon.close').show();
						} else {
							$spoiler_content.slideDown('fast');
							$(this).find('.icon').hide();
							$(this).find('.icon.open').show();
						}
					});
					$dom_elem = $temp_elem;
					break;
					
				case 'IMG':
					
					var href = getHrefForModalView(dom[i], null, hlinks),
						action = $dom_elem.attr('action');
					
					var src = $dom_elem.attr('data-src') || $dom_elem.attr('src') || '',
						anim = parseInt($dom_elem.attr('anim')) || 0;
					$dom_elem.attr('data-src', src).removeAttr('src');
					
					if(anim) {
						
						var canv = document.createElement('canvas'),
							attrs = $dom_elem.prop("attributes");
						$.each(attrs, function () {
							$(canv).attr(this.name, this.value);
						});
						$dom_elem = $(canv);
						
					} else {
						
						if(src && src.indexOf('~') != -1) {
							var srcs = src.split('~'),
								delta_xy = $dom_elem.attr('delta_xy') ? $dom_elem.attr('delta_xy').splitSplit(';', ',') : [];
							$temp_elem = $('<div style="position:relative;display:inline-block"></div>');
							if($dom_elem.attr('lst_action')) $temp_elem.attr('lst_action', $dom_elem.attr('lst_action'));
							if($dom_elem.attr('h')) $temp_elem.attr('h', $dom_elem.attr('h'));
							if($dom_elem.attr('store')) $temp_elem.attr('store', $dom_elem.attr('store'));
							if($dom_elem.attr('hint')) $temp_elem.attr('hint', $dom_elem.attr('hint'));
							srcs.reverse();
							var w = '', h = '';
							if(typeof $dom_elem.attr('w') != 'undefined') w = $dom_elem.attr('w');
							if(typeof $dom_elem.attr('h') != 'undefined') h = $dom_elem.attr('h');
							for(var j = 0, jj = srcs.length; j < jj; j++) {
								var $img = $('<img style="position:absolute" src="' + imageSrc(srcs[j], 0, 1, $img) + '" width="'+w+'" height="'+h+'" />'),
									delta = [0, 0];
								if(delta_xy[j]) delta = delta_xy[j];
								$img.css({
									'left': delta[0] || 0,
									'top': delta[1] || 0
								}).load(function () {
									setElementMinSizes(this);
								});
								$temp_elem.append($img);
							}
							$dom_elem = $temp_elem;
						} else {
							if(src == 'undefined' || typeof src == 'undefined') src = '0.gif';
							else src = imageSrc(src, 0, 1);
							$dom_elem.attr('data-src', src);
							setupImg($dom_elem, function() {
								setElementMinSizes(this);
							});
						}
						
					}
					if(!anim && src && src.indexOf('game_civ') != -1) {
						$dom_elem.css({
							'vertical-align': 'baseline',
							'border': 0,
							'position': 'relative',
							'margin-top': '-3px',
							'margin-bottom': '-2px',
							'margin-left': '-1px'
						});
					}
					
					if(h != "" && typeof $dom_elem.attr('h') != 'undefined') {
						$dom_elem.css('height', $dom_elem.attr('h') + 'px');
						$dom_elem.attr('height', $dom_elem.attr('h'));
					}
					
					if(typeof $dom_elem.attr('crop') != 'undefined') {
						var crop = $dom_elem.attr('crop') || '';
						if(crop.indexOf(',') != -1) crop = crop.split(',');
						if(crop.indexOf(';') != -1) crop = crop.split(';');
						if(crop.indexOf(':') != -1) crop = crop.split(':');
						crop[0] = parseInt(crop[0]);
						crop[1] = parseInt(crop[1]);
						var $crop = $('<div class="img-crop"></div>'),
							$wrapper = $('<div class="img-crop__wrapper"></div>');
						var crop_w = crop[0] || 0,
							crop_h = crop[1] || 0;
						if(crop_w) $crop.width(crop_w);
						if(w) $wrapper.css({'left':'50%', 'margin-left':-parseInt(crop_w/2)+'px'});
						if(crop_h) $crop.height(crop_h);
						if(h) $wrapper.css({'top':'50%', 'margin-top':-parseInt(crop_h/2)+'px'});
						$crop.append($dom_elem);
						$crop.css('margin', $dom_elem.css('margin'));
						copyStyleClass($dom_elem, $crop);
						$dom_elem.css('margin', 0);
						if(anim) var ia = new animImage($dom_elem, 1); // todo
						$crop.wrapInner($wrapper);
						$dom_elem = $crop;
						$dom_elem.find('img').one('load', function(e) {
							var $wrapper = $(this).parent('.img-crop__wrapper'),
								w = $(this).width(),
								h = $(this).height();
							if(w) {
								$wrapper.css({'left':'50%', 'margin-left':-parseInt(w/2)+'px'});
							}
							if(h) {
								$wrapper.css({'top':'50%', 'margin-top':-parseInt(h/2)+'px'});
							}
						});
						
					} else {
						
						if(anim) var ia = new animImage($dom_elem, 1); // todo
						
					}
					
					if(typeof $dom_elem.attr('hint') != 'undefined') {
						$dom_elem.addClass('hint').attr('data-hint', $dom_elem.attr('hint'));//.removeAttr('hint');
					}
					
					if(w != "" && typeof w !== 'undefined') {
						$dom_elem.css('width', $dom_elem.attr('w') + 'px');
						$dom_elem.attr('width', $dom_elem.attr('w'));
					}
					
					if(anim) {
						ia.animate();
					}
					
					$dom_elem.attr('href', href);
					if(href) {
						if(action == 'ajax') {
							bindClick($dom_elem, function(e, $elem) {
								e.preventDefault();
								e.stopPropagation();
								loadAjaxContent($elem, $elem.attr('href'));
							});
						} else {
							bindClick($dom_elem, function(e, $elem) {
								navigate($elem.attr('href'));
							});
						}
					}
					
					break;
					
				case 'HIDDEN':
					
					$temp_elem = $('<input type="hidden" name="">');
					var name = $dom_elem.attr('name') || '',
						value = $dom_elem.attr('value') || '';
					if (name === 'jsVersion') {
					    value = '' + jsVersion;
					} else {
					    var predefined_value = getPredefinedInputValue(name);
					    if (predefined_value != null) value = predefined_value;
					}
					$temp_elem.attr('name', name).val(value);
					$dom_elem = $temp_elem;
					if(name == 'reg_id') reg_id = value;
					else if(name == 'reg_nick') reg_nick = value;
					else if(name == 'reg_pass') reg_pass = value;
					break;
					
				case 'INPUT':
					
					var type = (typeof $dom_elem.attr('type') != 'undefined' ? $dom_elem.attr('type') : 'text'),
						size = parseInt($dom_elem.attr('size')) || '',
						minsize = parseInt($dom_elem.attr('minsize')) || 0,
						rows = parseInt($dom_elem.attr('rows')) || 0,
						format = (typeof $dom_elem.attr('format') != 'undefined' ? $dom_elem.attr('format') : null),
						$input = null,
						smiles = false,
						disabled = false,
						name = $dom_elem.attr('name') || '',
						value = $dom_elem.attr('value') || '',
						$wrapper = $('<div class="smart-input__wrapper"></div>');
					var predefined_value = getPredefinedInputValue(name);
					if(predefined_value != null) value = predefined_value;
					if(size > 128 || rows > 0) type = 'textarea';
					if(format) {
						if(format == 'decimal') format = 'number';
						if(format == 'phone') format = 'tel';
						type = format;
					}
					$temp_elem = $('<div class="smart-input" id="smart_input_'+getRandomInt(0,99999)+'" data-type="'+type+'"></div>');
					$temp_elem.append($wrapper);
					if(type == 'textarea') $input = $('<textarea class="input"></textarea>');
					else $input = $('<input class="input" type="'+type+'"/>');
					$wrapper.append($input);
					if(format == 'number' || format == 'tel' || format == 'email' || parseInt($dom_elem.attr('smiles')) == 0 || type == 'submit') {
						// nothing to do
					} else {
						smiles = true;
						$temp_elem.addClass('with-smiles');
						var $smiles_btn = $('<div class="smart-input__smiles-btn"></div>');
						$wrapper.prepend($smiles_btn);
						__bindSmiles($temp_elem, $smiles_btn);

					}
					if(typeof $dom_elem.attr('disabled') != 'undefined' && (parseInt($dom_elem.attr('disabled')) == 1 || $dom_elem.attr('disabled') == 'disabled')) {
						disabled = true;
						$temp_elem.addClass('disabled');
						$input.attr('disabled', 'disabled');
					}
					if(typeof $dom_elem.attr('pattern') != 'undefined') {
						$input.attr('pattern', $dom_elem.attr('pattern'));
					}
					if(typeof $dom_elem.attr('button') != 'undefined' || typeof $dom_elem.attr('button_text') != 'undefined') {
						var button = $dom_elem.attr('button') || null,
							button_text = $dom_elem.attr('button_text');
						$temp_elem.addClass('with-send');
						var $send_btn = $('<div class="smart-input__send-btn"></div>');
						if(button) {
							$send_btn.attr('data-type', button);
						} else {
							var button_w = getAverageTextWidth(button_text);
							if(button_w) {
								$send_btn.text(button_text).css('width', button_w+'px');
								if(smiles) $wrapper.css('padding-left', button_w+4+'px');
								else $wrapper.css('padding-right', button_w+4+'px');
							}
						}
						if(typeof $dom_elem.attr('action') == 'undefined') {
							$dom_elem.attr('action', 'post');
						}
						if(minsize && value.length < minsize) {
							$send_btn.addClass('disabled');
						}
						$wrapper.prepend($send_btn);
						if(disabled) $send_btn.addClass('disabled');
						var href = getHrefForModalView($dom_elem[0], null, hlinks);
						if(href) {
							bindClickNavigate($send_btn, href);
						}
					}
					$input
						.attr('name', name)
						.attr('maxlength', size)
						.attr('minlength', minsize)
						.attr('placeholder', $dom_elem.attr('alt') || '');
					if(type == 'textarea') {
						value = value.replace(/<br\s*[\/]?>/gi, "\n");
					}
					$input.val(value);
					if(size > 128 || rows > 0) {
						//$input.css('height', '30px');
						refreshTextareaHeight($input);
					}
					if(typeof $dom_elem.attr('error_text') != 'undefined' && $dom_elem.attr('error_text') != '') {
						$temp_elem.addClass('error');
						$wrapper.append('<div class="smart-input__error error-text"></div>');
						var error_text = $dom_elem.attr('error_text');
						convertDOM('<body>'+error_text+'</body>', $temp_elem.find('.smart-input__error'));
					}
					copyStyleClass($dom_elem, $temp_elem.find('.input'));
					if(typeof $dom_elem.attr('margin') != 'undefined' && $dom_elem.attr('margin') != '') {
						$temp_elem.css('margin', getMarginPadding($dom_elem.attr('margin')));
					}
					if(typeof $dom_elem.attr('id') != 'undefined') {
						$input.attr('id', $dom_elem.attr('id'));
					}
					if(w) $temp_elem.width(w);
					$input.css('width', '100%');
					$input.bind('focus blur keydown keyup edit input paste change', function(e) {
						var $elem = $(this);
						if(e.type == 'focus') {
							stopPageAutorefresh();
							$elem.parent().parent().addClass('focus');
							if(SYSTEM.isWP()) {
							    navigate('js: focus: 1');
							}
						} else if(e.type == 'blur') {
						    $elem.parent().parent().removeClass('focus');
						    if(SYSTEM.isWP()) {
						        navigate('js: focus: 0');
						    }
						}
						else if(e.type == 'keydown' || e.type == 'keyup' || e.type == 'edit' || e.type == 'input' || e.type == 'paste' || e.type == 'change') {
							if(typeof $elem.attr('minlength') != 'undefined') {
								var minlength = parseInt($elem.attr('minlength')) || 0;
								if(minlength) {
									if($elem.val().length < minlength) {
										$elem.parent().parent().find('.smart-input__send-btn').addClass('disabled');
									} else {
										$elem.parent().parent().find('.smart-input__send-btn').removeClass('disabled');
									}
								}
							}
							refreshTextareaHeight($elem);	// fucking function!
						}
					});
					$dom_elem = $temp_elem;
					break;
					
				case 'SELECT':
					
					var name = $dom_elem.attr('name') || '';
					var predefined_sel = getPredefinedInputValue(name);
					if(predefined_sel != null) $dom_elem.attr('sel', predefined_sel);
					$temp_elem = $('<div class="smart-input" data-type="select"></div>');
					$temp_elem.append($dom_elem.clone());
					if(typeof $dom_elem.attr('sel') != 'undefined') {
						$temp_elem.attr('sel', $dom_elem.attr('sel'));
					}
					if(typeof $dom_elem.attr('disabled') != 'undefined' && (parseInt($dom_elem.attr('disabled')) == 1 || $dom_elem.attr('disabled') == 'disabled')) {
						$temp_elem.addClass('disabled').find('select').attr('disabled', 'disabled');
					}
					copyStyleClass($dom_elem, $temp_elem);
					if(typeof $dom_elem.attr('margin') != 'undefined' && $dom_elem.attr('margin') != '') {
						$temp_elem.css('margin', getMarginPadding($dom_elem.attr('margin')));
					}
					if(typeof $dom_elem.attr('id') != 'undefined') {
						$temp_elem.find('select').attr('id', $dom_elem.attr('id'));
					}
					$dom_elem = $temp_elem;
					$dom_elem.find('select').bind('focus blur', function(e) {
						if(e.type == 'focus') $(this).parent().addClass('focus');
						else if(e.type == 'blur') $(this).parent().removeClass('focus');
					});
					if(typeof $dom_elem.find('select').attr('input_name') != 'undefined') {
						$dom_elem.find('select').bind('change', function(e) {
							var $target = $('[name="'+$(this).attr('input_name')+'"]');
							if($target && $target.length) {
								var val = $(this).find('option:selected').attr('input_value');
								//var val = $(this).val();
								if($target.get(0).nodeName == 'INPUT' || $target.get(0).nodeName == 'TEXTBOX') $target.val(val);
								else $target.text(val);
							}
						});
					}
					break;
					
				case 'OPTION':
					
					$dom_elem.html($dom_elem.attr('text')).removeAttr('text');
					break;
					
				case 'RAIT':
					
					$dom_elem = parseRaitTag($dom_elem.attr('ic'));
					break;
					
				case 'RADIO':
				case 'CHBOX':
					
					var input_type = 'radio',
						checked = 0,
						name = $dom_elem.attr('name') || '',
						value = $dom_elem.attr('value');
					if(dom[i].nodeName == 'CHBOX') input_type = 'checkbox';
					$temp_elem = $('<div class="'+input_type+'"></div>');
					var predefined_value = getPredefinedInputValue(name);
					if(predefined_value != null) $dom_elem.attr('value', predefined_value);
					if(input_type == 'checkbox') {
						if(
							(typeof $dom_elem.attr('value') != 'undefined' && (parseInt($dom_elem.attr('value')) == 1 || $dom_elem.attr('value') == 'checked')) ||
							(typeof $dom_elem.attr('checked') != 'undefined' && (parseInt($dom_elem.attr('checked')) == 1 || $dom_elem.attr('checked') == 'checked'))
						) {
							$temp_elem.addClass('checked');
							checked = 1;
						}
					} else if(input_type == 'radio') {
						if(
							(typeof $dom_elem.attr('checked') != 'undefined' && (parseInt($dom_elem.attr('checked')) == 1 || $dom_elem.attr('checked') == 'checked'))
						) {
							$temp_elem.addClass('checked');
							checked = 1;
						}
					}
					if(typeof $dom_elem.attr('disabled') != 'undefined' && (parseInt($dom_elem.attr('disabled')) == 1 || $dom_elem.attr('disabled') == 'disabled')) {
						$temp_elem.addClass('disabled');
					}
					$temp_elem.append('<input type="hidden" data-type="'+input_type+'" name="'+name+'" value="'+(input_type == 'radio' ? value : checked)+'" />');
					copyStyleClass($dom_elem, $temp_elem);
					if(typeof $dom_elem.attr('margin') != 'undefined' && $dom_elem.attr('margin') != '') {
						$temp_elem.css('margin', getMarginPadding($dom_elem.attr('margin')));
					}
					if(typeof $dom_elem.attr('id') != 'undefined') {
						$temp_elem.attr('id', $dom_elem.attr('id'));
					}
					$temp_elem.append('<div class="'+input_type+'__thumb"></div>');
					$dom_elem = $temp_elem;
					bindClick($dom_elem, function(e, $elem) {
						e.preventDefault();
						e.stopPropagation();
						if($elem.hasClass('disabled')) return;
						if($elem.hasClass('radio')) {
							$BROWSER.find(':hidden[name="'+$elem.find(':hidden').attr('name')+'"]').parent().removeClass('checked');
							if($elem.hasClass('checked')) $elem.removeClass('checked');
							else $elem.addClass('checked');
						} else {
							if($elem.hasClass('checked')) $elem.removeClass('checked').find(':hidden').val(0);
							else $elem.addClass('checked').find(':hidden').val(1);
						}


					});
					break;
					
				case 'LABEL':
					
					if(h) {
						$dom_elem.height(h);
					}
					if(typeof $dom_elem.attr('padding') != 'undefined' && $dom_elem.attr('padding') != '') {
						$dom_elem.css('padding', getMarginPadding($dom_elem.attr('padding')));
						if ($dom_elem[0] && ($dom_elem[0].tagName == 'TABLE' || $dom_elem[0].tagName == 'TR')) $dom_elem.find('td').css('padding', getMarginPadding($dom_elem.attr('padding')));
					}
					if(typeof $dom_elem.attr('margin') != 'undefined' && $dom_elem.attr('margin') != '') {
						$dom_elem.css('margin', getMarginPadding($dom_elem.attr('margin')));
					}
					if(typeof $dom_elem.attr('disabled') != 'undefined' && (parseInt($dom_elem.attr('disabled')) == 1 || $dom_elem.attr('disabled') == 'disabled')) {
						$dom_elem.addClass('disabled');
					}
					bindClick($dom_elem, function(e, $elem) {
						if($elem.hasClass('disabled')) return;
						var for_id = $elem.attr('for') || null,
							$for_elem = null,
							$checkboxes = null,
							$radios = null, radio_i = 0,
							$switches = null;
						if(for_id) {
							$for_elem = $BROWSER.find('#'+for_id);
							if($for_elem && $for_elem.length) {
								if($for_elem.hasClass('checkbox')) $checkboxes = $for_elem;
								if($for_elem.hasClass('radio')) $radios = $for_elem;
								if($for_elem.hasClass('switch')) $switches = $for_elem;
							}
						} else {
							$checkboxes = $elem.find('.checkbox').not('.disabled');
							$radios = $elem.find('.radio').not('.disabled');
							$switches = $elem.find('.switch').not('.disabled');
						}
						if($checkboxes && $checkboxes.length) {
							e.preventDefault();
							$checkboxes.each(function() {
								var $e = $(this);
								if($e.hasClass('checked')) $e.removeClass('checked').find(':hidden').val(0);
								else $e.addClass('checked').find(':hidden').val(1);
							});
						}
						if($radios && $radios.length) {
							e.preventDefault();
							$radios.each(function() {
								$BROWSER.find(':hidden[name="'+$(this).find(':hidden').attr('name')+'"]').parent().removeClass('checked');
								if(radio_i == 0) $(this).addClass('checked');
								radio_i++;
							});
						}
						if($switches && $switches.length) {
							e.preventDefault();
							$switches.each(function() {
								switchClicked($(this));
							});
						}
					}, true);
					break;
					
				case 'PERS':
					
					$temp_elem = $('<div class="pers"></div>');
					parsePersTag($temp_elem, $dom_elem, hlinks);
					$dom_elem = $temp_elem;
					break;
					
				case 'PERS_REG':
					
					$temp_elem = $('<div class="pers-reg"></div>');
					parsePersRegTag($temp_elem, $dom_elem, hlinks);
					$dom_elem = $temp_elem;
					break;
					
				case 'PORTAL':
					
					var descr = $dom_elem.attr('descr'),
						colors = $dom_elem.attr('colors'),
						portal_params = [];
					if (descr) portal_params = descr.split(/\s+/);
					else if (colors) portal_params = colors.split(':');
					if (portal_params.length) {
						$temp_elem = $('<canvas class="portal"></canvas>');
						var h = $dom_elem.attr('h');
						if (!h) h = 70;
						$temp_elem.attr('height', h).attr('width', 110);

						animElements.add(new Portal($temp_elem, portal_params, null, null, true, true, 0), 10);
						$dom_elem = $temp_elem;
					}
					break;
					
				case 'CLIENT_SETTINGS':
					
					if(LOCAL.CLIENT_SETTINGS) {
						$temp_elem = $('<a class="button" action="client_settings"><div class="button__wrapper">' + LOCAL.CLIENT_SETTINGS + '</div></a>');
						$dom_elem = $temp_elem;
						var href = getHrefForModalView($dom_elem.get(0));
						if(href) {
							bindClickNavigate($dom_elem, href);
						}
					}
					break;
					
				default:
					break;
				
			}
			
		} catch(err) {
			
			//log('convertDOM ERROR: '+dom[i].nodeName+' ', err);
			
		}
		
		if(allow_append) {
			$parent.append($dom_elem);
			$dom_elem = null;
		}
		
	}

	dom = null;
	$dom_elem = null;
	$temp_elem = null;
	
	if(cb && typeof cb === 'function') cb($parent);
	
}

function showMsgBoxDelay(title, text, time) {
	
	setTimeout(function() {
		
		var href = null;
		var attrs_array = ['title='+title, 'hint='+text];
		if(attrs_array.length > 0) {
			href = attrs_array.join(' ' + ATRRIBUTE_SEPARATOR);
			var httpStr = isHttps()? 'https:' : 'http:';
			httpStr += SYSTEM.isIOS? '//' : '////';
			// var httpStr = 'http:////';
			// if(SYSTEM.isIOS) httpStr = 'http://';
			href = httpStr + PREFIX_FOR_ACTION_LINKS + ' ' + ATRRIBUTE_SEPARATOR + href + ' ' + ATRRIBUTE_SEPARATOR;
			if(SYSTEM.isIOS()) href = encodeURI(href);//encodeURIComponent(JSON.stringify(href));
			if(SYSTEM.isAndroid()) href = encodeURI(href);
		}
		navigate(href);
		
	}, time*1000);
    
}

function isHttps() {
	return curUrl.indexOf('https')===0;
}

if(typeof ktml == 'undefined') var ktml = '';
if(typeof curUrl == 'undefined') {
	var curUrl = '';
} else {
	if(curUrl && curUrl.length && curUrl.indexOf('&op;')) {
		curUrl = curUrl.replace(/&op;/g, '#');
	}
}
if(typeof USERID == 'undefined') var USERID = '';
if(typeof USERPASS == 'undefined') var USERPASS = '';
if(typeof DEVICE_WIDTH == 'undefined') var DEVICE_WIDTH = $(document).width();
if(typeof DEVICE_HEIGHT == 'undefined') var DEVICE_HEIGHT = $(document).height();
if(typeof LOCAL == 'undefined') var LOCAL = {};
if(typeof $parent == 'undefined' || !$parent || !$parent.length) $parent = $BROWSER;
var windowScroll = 0;

var hasPagesTag = false,
	hasTabsTag = false,
	pagesTagSelIndex = 0,
	tabsTagSelIndex = 0;

function makeNormalSrcs(data) {
	var begin_data = '',
		src = '',
		last_data = '',
		i = data.indexOf('src="');
	if(i != -1) {
		begin_data = data.substr(0, i + 5);
		last_data = data.substr(i + 5);
		i = last_data.indexOf('"');
		//data = begin_data + '" data-src="' + imageSrc(last_data.substr(0, i),0,1) + makeNormalSrcs(last_data.substr(i));
		data = begin_data + '" data-src="' + last_data.substr(0, i) + makeNormalSrcs(last_data.substr(i));
	}
	return data;
}

function renderNewPage(ktml, url, user_id, user_pass, device_width, device_height, fix_src_attr) {
	try {
		
		startTime = new Date().getTime();
		
		if(ktml && ktml.indexOf('<body') != 0) {
			ktml = '<body>' + ktml + '</body>';
		}
		
		html = ktml;
		curUrl = url || curUrl;
		USERID = user_id || USERID;
		USERPASS = user_pass || USERPASS;
		DEVICE_WIDTH = device_width || DEVICE_WIDTH;
		DEVICE_HEIGHT = device_height || DEVICE_HEIGHT;
		
		hasPagesTag = false;
		hasTabsTag = false;
		pagesTagSelIndex = 0;
		tabsTagSelIndex = 0;
		
		isOnBack = 0;
		
		pageAutorefreshParams = null;
		
		msgBoxArray = [];
		noteArray = [];
		lastHlinks = [];
		lastMeta = {};
		
		$parent.find('*').remove();
		$parent.html('');
		
		stopPageAutorefresh();
		
		if(typeof fix_src_attr == 'undefined') {
		    html = makeNormalSrcs(html);
		}
		
		windowScroll = 0;
		
		convertDOM(html, $parent, null, function() {
			try {
				
				var $content = $parent;
				
				setTimeout(function() {
					if($content.find('.pers-reg').hasClass('minimized')) {
						$content.find('[name="reg-container1"]').hide();
						$content.find('[name="reg-container2"]').show();
					} else {
						$content.find('[name="reg-container1"]').show();
						$content.find('[name="reg-container2"]').hide();
					}
				}, 1);
				
				$content.find('.exit-button').next('br').remove();
				$content.find('.exit-button').next('br').remove();
				$content.find('.exit-button').remove();

				$content.find('.space.need-br-after').each(function () {
					$(this).after('<br/>');
				});
				
				$content.find('p.font1').each(function () {
					try {
						var $next = $(this).next();
						if (($next.get(0) && $next.get(0).nodeName) && ($next.get(0).nodeName == 'A') && ($next.hasClass('a-menu') || $next.hasClass('a-hint'))) {
							$next.css('font-size', $(this).css('font-size'));
						}
					} catch (err) {
						
					}
				});
				
				$content.find('.plank.border-bottom').each(function () {
					$(this).next('br').remove();
				});
				
				$content.find('.smart-input[data-type="select"]').each(function() {
					
					var sel = parseInt($(this).attr('sel')) || 0;
					$(this).find('option:nth-child('+(sel+1)+')').attr('selected', 'selected');
					
				});
				
				$content.find('.tabs').each(function() {
					
					parseTabsTag($(this));
					
				});
				
				$content.find('.pages').each(function () {
					
					parsePagesTag($(this));
					
				});
				
				$content.find('a.a-ajax[open="1"]').each(function () {
					self.openAjaxContainer($(this));
				});
				
				$content.find('.hasAnimationChain').each(function() {
					configureAnimationChain($(this));
				});
				
				var anchor = '',
					$anchor = null;
				if(curUrl.indexOf('#') != -1) {
					anchor = curUrl.substr(curUrl.indexOf('#') + 1);
					$anchor = $BROWSER.find('#anchor_' + anchor);
				}
				
				if($anchor && $anchor.length) {
					var offset = $anchor.position().top - 20;
					if(offset < 0) offset = 0;
					windowScroll = offset;
				}
				
				var window_scroll_delay = 1;
				if(SYSTEM.isWP()) window_scroll_delay = 50;
				setTimeout(function() {
					window.scrollTo(0, windowScroll);
				}, window_scroll_delay);
				
				if(msgBoxArray) {
					for(var i = 0, ii = msgBoxArray.length; i < ii; i++) {
						if(msgBoxArray[i].time) {
							showMsgBoxDelay(msgBoxArray[i].title, msgBoxArray[i].text, msgBoxArray[i].time);
						}
					}
				}
				
			} catch(err) {
				
				//log('convertDOM CB ERROR:', err);
				
			}
			
			refreshBrowserMinHeight();
			
			var hlinksStr = 'hlinks:';
			if(lastHlinks) {
				for(var i = 0, n = lastHlinks.length; i < n; i++) {
					hlinksStr += ' ' + lastHlinks[i][0] + ' ' + lastHlinks[i][1];
				}
			}
			
			var msgBoxStr = '';
			if(msgBoxArray) {
				for(var i = 0, ii = msgBoxArray.length; i < ii; i++) {
					if(msgBoxArray[i].time) continue;
					var str = '';
					str += 'qout;titleqout;=qout;' + msgBoxArray[i].title + 'qout;&';
					str += 'qout;textqout;=qout;' + msgBoxArray[i].text + 'qout;&';
					if(SYSTEM.isAndroid() || SYSTEM.isWP()) {
						str = encodeURI(str);
					}
					msgBoxStr += str;
				}
			}
			
			var noteStr = '';
			if(noteArray) {
				for(var i = 0, ii = noteArray.length; i < ii; i++) {
					if(SYSTEM.isAndroid() || SYSTEM.isWP()) noteStr += encodeURI('qout;' + noteArray[i].params + 'qout;&');
					else noteStr += 'qout;' + noteArray[i].params + 'qout;&';
				}
			}
			
			var newLoc = 'js: endPageLoading ' + hlinksStr;
			if(SYSTEM.isAndroid()) newLoc = 'js: endPageLoading ' + document.body.offsetHeight + ' ' + windowScroll + ' ' + hlinksStr;
			if(SYSTEM.isWP()) newLoc = 'js: endPageLoading ' + document.body.offsetHeight + ' ' + windowScroll + ' ' + hlinksStr;
			if(SYSTEM.isIOS()) newLoc = 'js: endPageLoading ' + document.body.offsetHeight + ' ' + hlinksStr;
			if(isOnBack > 0) newLoc += ' isOnBack:';
			
			if(msgBoxStr) newLoc += ' msgBox: ' + msgBoxStr;
			
			if(noteStr) newLoc += ' noteStr: ' + noteStr;
			
			//bodyLog('time '+((new Date().getTime() - startTime) / 1000) + 's');
			
			predefinedInputsValues = null;
			
			navigate(newLoc);

			startPageAutorefresh();
			
		}, scrollTo);
		
	} catch(err) {
		
		var newLoc = 'js: pageLoadingError ' + err;
		navigate(newLoc);
		
	}
}

if(!SYSTEM.isWP()) {
	renderNewPage(html);
}
